@@ -32,9 +32,9 @@
 #ifndef _GSToolbarView_h_INCLUDE
 #define _GSToolbarView_h_INCLUDE
 
-#include <Foundation/NSGeometry.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSColor.h>
+#import <Foundation/NSGeometry.h>
+#import <AppKit/NSView.h>
+#import <AppKit/NSColor.h>
 
 @class NSMutableArray;
 @class NSClipView;
@@ -55,12 +55,14 @@ typedef enum {
   NSToolbar *_toolbar;
   NSClipView *_clipView;
   GSToolbarClippedItemsButton *_clippedItemsMark;
-  NSMutableArray *_visibleBackViews;
   unsigned int _borderMask;
   NSRect _rectAvailable;
   float _heightFromLayout;
 }
 
++ (int) draggedItemIndex;
++ (void) setDraggedItemIndex:(int)sourceIndex;
+
 - (id) initWithFrame: (NSRect)frame;
 
 // Accessors
