@@ -30,7 +30,7 @@
 #define _GNUstep_H_NSAlert
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
+#import <Foundation/NSObject.h>
 
 @class NSArray;
 @class NSError;
@@ -66,6 +66,8 @@ enum {
   id _delegate;
   NSAlertStyle _style;
   BOOL _shows_help;
+  id _modalDelegate;
+  SEL _didEndSelector;
   int	_result;
 }
 
