@@ -41,8 +41,11 @@
 
 #import <AppKit/NSResponder.h>
 
+#if defined(__cplusplus)
+extern "C" {
+#endif
+
 @class NSArray;
-@class NSAutoreleasePool;
 @class NSDate;
 @class NSError;
 @class NSException;
@@ -152,7 +155,7 @@ APPKIT_EXPORT NSString	*NSEventTrackingRunLoopMode;
      a single exception (which is why it is stored here as an ivar): the
      -terminate: method will destroy this autorelease pool before exiting
      the program.  */
-  NSAutoreleasePool     *_runLoopPool;
+  id             _runLoopPool;
 }
 
 /*
@@ -685,4 +688,8 @@ NSShowSystemInfoPanel(NSDictionary *options);
  */
 APPKIT_EXPORT NSApplication	*NSApp;
 
+#if defined(__cplusplus)
+}
+#endif
+
 #endif // _GNUstep_H_NSApplication
