@@ -29,8 +29,9 @@
 #define BEZIERPATH_H
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/Foundation.h>
-#include <AppKit/NSFont.h>
+#import <Foundation/NSGeometry.h>
+#import <Foundation/NSObject.h>
+#import <AppKit/NSFont.h>
 
 @class NSAffineTransform;
 @class NSImage;
@@ -82,11 +83,19 @@ typedef enum {
   int _dash_count;
   float _dash_phase;
   float *_dash_pattern;
-  BOOL _shouldRecalculateBounds;
   NSRect _bounds;
   NSRect _controlPointBounds;
-  BOOL _cachesBezierPath;
   NSImage *_cacheImage;
+#ifndef	_IN_NSBEZIERPATH_M
+#define	GSIArray	void*
+#endif
+  GSIArray _pathElements;
+#ifndef	_IN_NSBEZIERPATH_M
+#undef	GSIArray
+#endif
+  BOOL _cachesBezierPath;
+  BOOL _shouldRecalculateBounds;
+  BOOL _flat;
 }
 
 //
@@ -105,8 +114,14 @@ typedef enum {
 // Immediate mode drawing of common paths
 //
 + (void)fillRect:(NSRect)aRect;
+
+/** Using default stroke color and default drawing attributes, strokes 
+    a rectangle using the specified rect. */
 + (void)strokeRect:(NSRect)aRect;
 + (void)clipRect:(NSRect)aRect;
+
+/** Using default stroke color and default drawing attributes, draws a line 
+    between the two points specified. */
 + (void)strokeLineFromPoint:(NSPoint)point1 toPoint:(NSPoint)point2;
 + (void)drawPackedGlyphs: (const char *)packedGlyphs atPoint: (NSPoint)aPoint;
 
