@@ -31,13 +31,14 @@
 #define _GNUstep_H_NSBitmapImageRep
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSImageRep.h>
-#include <Foundation/NSDictionary.h>
+#import <AppKit/NSImageRep.h>
 
 @class NSArray;
 @class NSString;
 @class NSData;
+@class NSDictionary;
 @class NSMutableData;
+@class NSMutableDictionary;
 @class NSColor;
 
 /** Describes the type of compression used on an image.  Possible compressions:
@@ -132,7 +133,7 @@ APPKIT_EXPORT NSString *NSImageEXIFData; // No GNUstep support yet; for reading
   NSMutableDictionary   *_properties;
   BOOL			_isPlanar;
   unsigned char		**_imagePlanes;
-  NSMutableData		*_imageData;
+  NSData		*_imageData;
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
   NSBitmapFormat  _format;
 #else
@@ -150,8 +151,8 @@ APPKIT_EXPORT NSString *NSImageEXIFData; // No GNUstep support yet; for reading
 - (id) initWithBitmapDataPlanes: (unsigned char**)planes
 		     pixelsWide: (int)width
 		     pixelsHigh: (int)height
-		  bitsPerSample: (int)bps
-		samplesPerPixel: (int)spp
+		  bitsPerSample: (int)bitsPerSample
+		samplesPerPixel: (int)samplesPerPixel
 		       hasAlpha: (BOOL)alpha
 		       isPlanar: (BOOL)isPlanar
 		 colorSpaceName: (NSString*)colorSpaceName
