@@ -31,17 +31,16 @@
 #define _GNUstep_H_NSCell
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSGeometry.h>
+#import <Foundation/NSGeometry.h>
 
 // For tint
-#include <AppKit/NSColor.h>
+#import <AppKit/NSColor.h>
 // for NSWritingDirection
-#include <AppKit/NSParagraphStyle.h>
+#import <AppKit/NSParagraphStyle.h>
 // For text alignment
-#include <AppKit/NSText.h>
+#import <AppKit/NSText.h>
 // for NSFocusRingType
-#include <AppKit/NSView.h>
+#import <AppKit/NSView.h>
 
 @class NSString;
 @class NSMutableDictionary;
@@ -119,10 +118,33 @@ enum {
   NSMixedState			= -1
 };
 
+/**
+ *  <p>Enumeration of the ways that you can display an image in an
+ *  NSImageCell.  The available ones are:</p>
+ *  <p><code>NSScaleNone</code>: The image is always displayed with
+ *  its natural size.  If it's bigger than the cell size, it is
+ *  cropped.</p>
+ *  <p><code>NSScaleProportionally</code>: If the image is bigger
+ *  than the cell size, it is displayed in its natural size.  If it
+ *  is smaller than the cell size, it is resized down proportionally
+ *  to fit the cell size.</p>
+ *  <p><code>NSScaleToFit</code>: The image is always resized (up
+ *  or down) to fit exactly in the cell size.</p>
+ */
+typedef enum {
+  NSScaleProportionally = 0,
+  NSScaleToFit = 1,
+  NSScaleNone = 2,
+  NSImageScaleProportionallyDown = 0,
+  NSImageScaleAxesIndependently = 1,
+  NSImageScaleNone = 2,
+  NSImageScaleProportionallyUpOrDown = 3
+} NSImageScaling;
+
 @interface NSCell : NSObject <NSCopying, NSCoding>
 {
   // Attributes
-  NSString *_contents;
+  id _contents;
   NSImage *_cell_image;
   NSFont *_font;
   id _object_value;
@@ -140,7 +162,7 @@ enum {
     unsigned is_bordered: 1;   
     unsigned is_bezeled: 1;   
     unsigned is_scrollable: 1;
-    unsigned wraps: 1;
+    unsigned reserved: 1;
     unsigned text_align: 3; // 5 values
     unsigned is_selectable: 1;
     unsigned allows_mixed_state: 1;
@@ -205,7 +227,11 @@ enum {
 - (void)setFloatValue:(float)aFloat;
 - (void)setIntValue:(int)anInt;
 - (void)setStringValue:(NSString *)aString;
-
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (NSInteger) integerValue;
+- (void) setIntegerValue: (NSInteger)anInt;
+- (void) takeIntegerValueFrom: (id)sender;
+#endif
 //
 // Setting Parameters 
 //
@@ -241,11 +267,11 @@ enum {
 //
 // Setting the NSCell's State 
 //
-- (void)setState:(int)value;
-- (int)state;
+- (void)setState:(NSInteger)value;
+- (NSInteger)state;
 - (BOOL)allowsMixedState;
 - (void)setAllowsMixedState:(BOOL)flag;
-- (int)nextState;
+- (NSInteger)nextState;
 - (void)setNextState;
 
 //
@@ -301,8 +327,8 @@ enum {
 //
 // Assigning a Tag 
 //
-- (void)setTag:(int)anInt;
-- (int)tag;
+- (void)setTag:(NSInteger)anInt;
+- (NSInteger)tag;
 
 //
 // Formatting Data and Validating Input 
@@ -352,8 +378,8 @@ enum {
 - (BOOL)showsFirstResponder;
 - (void)setTitleWithMnemonic:(NSString *)aString;
 - (NSString *)mnemonic;
-- (void)setMnemonicLocation:(unsigned int)location;
-- (unsigned int)mnemonicLocation;
+- (void)setMnemonicLocation:(NSUInteger)location;
+- (NSUInteger)mnemonicLocation;
 - (BOOL)refusesFirstResponder;
 - (void)setRefusesFirstResponder:(BOOL)flag;
 
@@ -491,13 +517,11 @@ enum {
                                     inView: (NSView*)controlView;
 - (void) _drawFocusRingWithFrame: (NSRect)cellFrame 
                           inView: (NSView*)controlView;
+- (void) _drawEditorWithFrame: (NSRect)cellFrame
+		       inView: (NSView*)controlView;
+- (void) _setInEditing: (BOOL)flag;
+- (void) _updateFieldEditor: (NSText*)textObject;
 @end
 
-//
-// Function which should be somewhere else
-//
-inline NSSize 
-_sizeForBorderType (NSBorderType aType);
-
 #endif // _GNUstep_H_NSCell
 
