@@ -31,7 +31,7 @@
 #define _GNUstep_H_NSColorWell
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSControl.h>
+#import <AppKit/NSControl.h>
 
 @class NSColor;
 
@@ -45,6 +45,8 @@
   NSRect _wellRect;
   id _target;
   SEL _action;
+  // Mouse tracking
+  NSPoint _mouseDownPoint;
 }
 
 //
