@@ -29,8 +29,9 @@
 #define _GNUstep_H_NSComboBox
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSTextField.h>
+#import <AppKit/NSTextField.h>
 
+@class NSArray;
 @class NSString;
 @class NSNotification;
 
@@ -86,6 +87,11 @@
 - (void) setCompletes: (BOOL)completes;
 - (BOOL) completes;
 #endif
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
+- (BOOL) isButtonBordered;
+- (void) setButtonBordered:(BOOL)flag;
+#endif
 @end
 
 @interface NSObject (NSComboBoxDataSource)
