@@ -32,8 +32,8 @@
 #import <GNUstepBase/GSVersionMacros.h>
 
 // for NSWritingDirection
-#include <AppKit/NSParagraphStyle.h>
-#include <AppKit/NSView.h>
+#import <AppKit/NSParagraphStyle.h>
+#import <AppKit/NSView.h>
 
 @class NSString;
 @class NSNotification;
@@ -47,7 +47,7 @@
 @interface NSControl : NSView
 {
   // Attributes
-  int _tag;
+  NSInteger _tag;
   id _cell; // id so compiler wont complain too much for subclasses
   BOOL _ignoresMultiClick;
 }
@@ -70,7 +70,7 @@
 // Identifying the Selected Cell 
 //
 - (id)selectedCell;
-- (int)selectedTag;
+- (NSInteger)selectedTag;
 
 //
 // Setting the Control's Value 
@@ -84,6 +84,12 @@
 - (void) setIntValue: (int)anInt;
 - (int) intValue;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (NSInteger) integerValue;
+- (void) setIntegerValue: (NSInteger)anInt;
+- (void) takeIntegerValueFrom: (id)sender;
+#endif
+
 - (void) setStringValue: (NSString *)aString;
 - (NSString *) stringValue;
 
@@ -166,8 +172,8 @@
 //
 // Assigning a Tag 
 //
-- (void)setTag:(int)anInt;
-- (int)tag;
+- (void)setTag:(NSInteger)anInt;
+- (NSInteger)tag;
 
 //
 // Activation
