@@ -12,19 +12,19 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor,
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
    Boston, MA 02110-1301, USA.
 */ 
 
@@ -32,16 +32,17 @@
 #define _GNUstep_H_NSFont
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSGeometry.h>
-#include <Foundation/NSString.h>
-#include <AppKit/AppKitDefines.h>
+#import <Foundation/NSObject.h>
+#import <Foundation/NSGeometry.h>
+#import <AppKit/AppKitDefines.h>
 // For NSControlSize
-#include <AppKit/NSColor.h>
+#import <AppKit/NSColor.h>
 
-@class NSDictionary;
+@class NSAffineTransform;
 @class NSCharacterSet;
+@class NSDictionary;
 @class NSFontDescriptor;
+@class NSGraphicsContext;
 
 typedef unsigned int NSGlyph;
 
@@ -51,7 +52,6 @@ enum {
   NSNullGlyph = 0x0
 };
 
-
 typedef enum _NSGlyphRelation {
   NSGlyphBelow,
   NSGlyphAbove,
@@ -65,12 +65,22 @@ typedef enum _NSMultibyteGlyphPacking {
   NSFourByteGlyphPacking
 } NSMultibyteGlyphPacking;
 
-APPKIT_EXPORT const float *NSFontIdentityMatrix;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
+typedef enum _NSFontRenderingMode
+{
+  NSFontDefaultRenderingMode = 0,
+  NSFontAntialiasedRenderingMode,
+  NSFontIntegerAdvancementsRenderingMode,
+  NSFontAntialiasedIntegerAdvancementsRenderingMode
+} NSFontRenderingMode;
+#endif
+
+APPKIT_EXPORT const CGFloat *NSFontIdentityMatrix;
 
 @interface NSFont : NSObject <NSCoding, NSCopying>
 {
   NSString *fontName;
-  float matrix[6];
+  CGFloat matrix[6];
   BOOL matrixExplicitlySet;
   BOOL screenFont;
 
@@ -103,34 +113,43 @@ APPKIT_EXPORT const float *NSFontIdentityMatrix;
 //
 // Creating a Font Object
 //
-+ (NSFont*) boldSystemFontOfSize: (float)fontSize;
++ (NSFont*) boldSystemFontOfSize: (CGFloat)fontSize;
 + (NSFont*) fontWithName: (NSString*)aFontName 
-		  matrix: (const float*)fontMatrix;
+		  matrix: (const CGFloat*)fontMatrix;
 + (NSFont*) fontWithName: (NSString*)aFontName
-		    size: (float)fontSize;
-+ (NSFont*) systemFontOfSize: (float)fontSize;
-+ (NSFont*) userFixedPitchFontOfSize: (float)fontSize;
-+ (NSFont*) userFontOfSize: (float)fontSize;
+		    size: (CGFloat)fontSize;
++ (NSFont*) systemFontOfSize: (CGFloat)fontSize;
++ (NSFont*) userFixedPitchFontOfSize: (CGFloat)fontSize;
++ (NSFont*) userFontOfSize: (CGFloat)fontSize;
 
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
-+ (NSFont*) titleBarFontOfSize: (float)fontSize;
-+ (NSFont*) menuFontOfSize: (float)fontSize;
-+ (NSFont*) messageFontOfSize: (float)fontSize;
-+ (NSFont*) paletteFontOfSize: (float)fontSize;
-+ (NSFont*) toolTipsFontOfSize: (float)fontSize;
-+ (NSFont*) controlContentFontOfSize: (float)fontSize;
-+ (NSFont*) labelFontOfSize: (float)fontSize;
-+ (NSFont*) menuBarFontOfSize: (float)fontSize;
++ (NSFont*) titleBarFontOfSize: (CGFloat)fontSize;
++ (NSFont*) menuFontOfSize: (CGFloat)fontSize;
++ (NSFont*) messageFontOfSize: (CGFloat)fontSize;
++ (NSFont*) paletteFontOfSize: (CGFloat)fontSize;
++ (NSFont*) toolTipsFontOfSize: (CGFloat)fontSize;
++ (NSFont*) controlContentFontOfSize: (CGFloat)fontSize;
++ (NSFont*) labelFontOfSize: (CGFloat)fontSize;
++ (NSFont*) menuBarFontOfSize: (CGFloat)fontSize;
+#endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
++ (NSFont*) fontWithDescriptor: (NSFontDescriptor*)descriptor size: (CGFloat)size;
++ (NSFont*) fontWithDescriptor: (NSFontDescriptor*)descriptor 
+                 textTransform: (NSAffineTransform*)transform;
+// This method was a mistake in the 10.4 documentation
++ (NSFont*) fontWithDescriptor: (NSFontDescriptor*)descriptor 
+                          size: (CGFloat)size
+                 textTransform: (NSAffineTransform*)transform;
 #endif
 
 //
 // Font Sizes
 //
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
-+ (float) labelFontSize;
-+ (float) smallSystemFontSize;
-+ (float) systemFontSize;
-+ (float) systemFontSizeForControlSize: (NSControlSize)controlSize;
++ (CGFloat) labelFontSize;
++ (CGFloat) smallSystemFontSize;
++ (CGFloat) systemFontSize;
++ (CGFloat) systemFontSizeForControlSize: (NSControlSize)controlSize;
 #endif
 
 //
@@ -146,6 +165,10 @@ APPKIT_EXPORT const float *NSFontIdentityMatrix;
 + (void) setUserFont: (NSFont*)aFont;
 + (void) useFont: (NSString*)aFontName;
 - (void) set;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
+- (void) setInContext: (NSGraphicsContext*)context;
+- (NSAffineTransform*) textTransform;
+#endif
 
 //
 // Querying the Font
@@ -159,26 +182,30 @@ APPKIT_EXPORT const float *NSFontIdentityMatrix;
 - (NSString*) encodingScheme;
 - (BOOL) isFixedPitch;
 - (BOOL) isBaseFont;
-- (const float*) matrix;
-- (float) pointSize;
+- (const CGFloat*) matrix;
+- (CGFloat) pointSize;
 - (NSFont*) printerFont;
 - (NSFont*) screenFont;
-- (float) ascender;
-- (float) descender;
-- (float) capHeight;
-- (float) italicAngle;
+- (CGFloat) ascender;
+- (CGFloat) descender;
+- (CGFloat) capHeight;
+- (CGFloat) italicAngle;
 - (NSSize) maximumAdvancement;
 - (NSSize) minimumAdvancement;
-- (float) underlinePosition;
-- (float) underlineThickness;
-- (float) xHeight;
-- (float) widthOfString: (NSString*)string;
-- (float) defaultLineHeightForFont;
+- (CGFloat) underlinePosition;
+- (CGFloat) underlineThickness;
+- (CGFloat) xHeight;
+- (CGFloat) widthOfString: (NSString*)string;
+- (CGFloat) defaultLineHeightForFont;
 
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
-- (unsigned) numberOfGlyphs;
+- (NSUInteger) numberOfGlyphs;
 - (NSCharacterSet*) coveredCharacterSet;
+#endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
 - (NSFontDescriptor*) fontDescriptor;
+- (NSFontRenderingMode) renderingMode;
+- (NSFont*) screenFontWithRenderingMode: (NSFontRenderingMode)mode;
 #endif
 
 //
