@@ -29,11 +29,15 @@
 #define __NSGraphics_h__
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSGeometry.h>
+#import <Foundation/NSObject.h>
+#import <Foundation/NSGeometry.h>
 
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/AppKitDefines.h>
+#import <AppKit/NSGraphicsContext.h>
+#import <AppKit/AppKitDefines.h>
+
+#if defined(__cplusplus)
+extern "C" {
+#endif
 
 @class NSString;
 @class NSColor;
@@ -142,7 +146,6 @@ NSDrawBezel(const NSRect aRect, const NSRect clipRect)
   NSDrawGrayBezel(aRect, clipRect);
 }
 
-
 /** Draws a rectangle along the inside of aRect.  The rectangle will be
     black, dotted (using 1 point dashes), and will have a line width
     of 1 point.  */
@@ -177,12 +180,22 @@ APPKIT_EXPORT void NSDrawBitmap(NSRect rect,
                                 NSString *colorSpaceName,
                                 const unsigned char *const data[5]);
 
+APPKIT_EXPORT void
+NSDrawNinePartImage(NSRect aRect, NSImage *topLeft, NSImage *topMiddle,
+		    NSImage *topRight, NSImage *centerLeft,
+		    NSImage *centerMiddle, NSImage *centerRight,
+		    NSImage *bottomLeft, NSImage *bottomMiddle,
+		    NSImage *bottomRight, NSCompositingOperation op,
+		    CGFloat fraction, BOOL flipped);
+
 static inline void
 NSBeep(void)
 {
   NSGraphicsContext *ctxt = GSCurrentContext();
-  (ctxt->methods->NSBeep)
-    (ctxt, @selector(NSBeep));
+  if (ctxt != nil) {
+    (ctxt->methods->NSBeep)
+      (ctxt, @selector(NSBeep));
+  }
 }
 
 static inline void
@@ -244,4 +257,19 @@ APPKIT_EXPORT int NSGetWindowServerMemory(int context, int *virtualMemory,
 
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_1, GS_API_LATEST)
+typedef enum _NSFocusRingPlacement
+{
+    NSFocusRingOnly=0,
+    NSFocusRingBelow,
+    NSFocusRingAbove
+} NSFocusRingPlacement;
+
+void NSSetFocusRingStyle(NSFocusRingPlacement placement);
+#endif
+
+#if defined(__cplusplus)
+}
+#endif
+
 #endif /* __NSGraphics_h__ */
