@@ -279,8 +279,8 @@ APPKIT_EXPORT NSGraphicsContext	*GSCurrentContext(void);
 
 - (void) GSSetFillColorspace: (void *)spaceref;
 - (void) GSSetStrokeColorspace: (void *)spaceref;
-- (void) GSSetFillColor: (const float *)values;
-- (void) GSSetStrokeColor: (const float *)values;
+- (void) GSSetFillColor: (const CGFloat *)values;
+- (void) GSSetStrokeColor: (const CGFloat *)values;
 
 /* ----------------------------------------------------------------------- */
 /* Text operations */
@@ -307,6 +307,7 @@ APPKIT_EXPORT NSGraphicsContext	*GSCurrentContext(void);
 - (void) GSSetTextPosition: (NSPoint)loc;
 - (void) GSShowText: (const char *)string : (size_t) length;
 - (void) GSShowGlyphs: (const NSGlyph *)glyphs : (size_t) length;
+- (void) GSShowGlyphsWithAdvances: (const NSGlyph *)glyphs : (const NSSize *)advances : (size_t) length;
 
 /* ----------------------------------------------------------------------- */
 /* Gstate Handling */
