@@ -31,7 +31,7 @@
 #define _GNUstep_H_NSMatrix
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSControl.h>
+#import <AppKit/NSControl.h>
 
 @class NSArray;
 @class NSMutableArray;
@@ -51,7 +51,7 @@ typedef enum _NSMatrixMode {
 
 @interface NSMatrix : NSControl <NSCoding>
 {
-  id		**_cells;
+  __strong id		**_cells;
   BOOL		**_selectedCells;
   int		_maxRows;
   int		_maxCols;
@@ -290,7 +290,6 @@ typedef enum _NSMatrixMode {
 		 to: (id)anObject
 	forAllCells: (BOOL)flag;
 - (void) sendDoubleAction;
-- (void) setErrorAction: (SEL)aSelector;
 
 /*
  * Handling Event and Action Messages 
