@@ -9,19 +9,19 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor,
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
    Boston, MA 02110-1301, USA.
 */ 
 
@@ -29,23 +29,23 @@
 #define _GNUstep_H_NSMenuView
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSProcessInfo.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSNotification.h>
-
-#include <AppKit/NSMenu.h>
-#include <AppKit/NSMenuItem.h>
-#include <AppKit/NSMenuItemCell.h>
-#include <AppKit/NSScreen.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSWindow.h>
+#import <AppKit/NSMenu.h>
+#import <AppKit/NSMenuItem.h>
+//#import <AppKit/NSMenuItemCell.h>
+#import <AppKit/NSView.h>
 
+@class NSArray;
+@class NSMutableArray;
+@class NSNotification;
+@class NSString;
 @class NSColor;
+@class NSMenuItemCell;
 @class NSPopUpButton;
+@class NSEvent;
 @class NSFont;
+@class NSScreen;
+@class NSWindow;
+
 
 /**
    The NSMenu class uses an object implementing the NSMenuView protocol to
