@@ -18,19 +18,19 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor,
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
    Boston, MA 02110-1301, USA.
 */ 
 
@@ -38,14 +38,13 @@
 #define _GNUstep_H_NSOpenPanel
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSSavePanel.h>
+#import <AppKit/NSSavePanel.h>
 
 @class NSString;
 @class NSArray;
 
 @interface NSOpenPanel : NSSavePanel <NSCoding>
 {
-  NSArray *_fileTypes;
   BOOL _canChooseDirectories;
   BOOL _canChooseFiles;
 }
@@ -53,16 +52,16 @@
 + (NSOpenPanel *) openPanel;
 
 // Running an NSOpenPanel 
-- (int) runModalForTypes: (NSArray *)fileTypes;
-- (int) runModalForDirectory: (NSString *)path
-                        file: (NSString *)name
-                       types: (NSArray *)fileTypes;
+- (NSInteger) runModalForTypes: (NSArray *)fileTypes;
+- (NSInteger) runModalForDirectory: (NSString *)path
+                              file: (NSString *)name
+                             types: (NSArray *)fileTypes;
 
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
-- (int) runModalForDirectory: (NSString *)path
-			file: (NSString *)name
-		       types: (NSArray *)fileTypes
-	    relativeToWindow: (NSWindow*)window;
+- (NSInteger) runModalForDirectory: (NSString *)path
+                              file: (NSString *)name
+                             types: (NSArray *)fileTypes
+                  relativeToWindow: (NSWindow*)window;
 - (void) beginSheetForDirectory: (NSString *)path
 			   file: (NSString *)name
 			  types: (NSArray *)fileTypes
@@ -71,9 +70,16 @@
 		 didEndSelector: (SEL)didEndSelector
 		    contextInfo: (void *)contextInfo;
 #endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
+- (void) beginForDirectory: (NSString *)absoluteDirectoryPath
+                      file: (NSString *)filename
+                     types: (NSArray *)fileTypes
+          modelessDelegate: (id)modelessDelegate
+            didEndSelector: (SEL)didEndSelector
+               contextInfo: (void *)contextInfo;
+#endif
 
 - (NSArray *) filenames;
-
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (NSArray *) URLs; 
 #endif
@@ -83,7 +89,7 @@
 - (BOOL) canChooseFiles;
 - (void) setCanChooseDirectories: (BOOL)flag;
 - (void) setCanChooseFiles: (BOOL)flag;
-#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_1, GS_API_LATEST)
 - (void) setResolvesAliases: (BOOL)flag; 
 - (BOOL) resolvesAliases; 
 #endif
