@@ -7,9 +7,9 @@
 
    Author:  Scott Christley <scottc@net-community.com>
    Date: 1996
-   Author:  Daniel B?hringer <boehring@biomed.ruhr-uni-bochum.de>
+   Author:  Daniel Boehringer <boehring@biomed.ruhr-uni-bochum.de>
    Date: August 1998
-   Source by Daniel B?hringer integrated into Scott Christley's preliminary
+   Source by Daniel Boehringer integrated into Scott Christley's preliminary
    implementation by Felipe A. Rodriguez <far@ix.netcom.com> 
 
    This file is part of the GNUstep GUI Library.
@@ -35,7 +35,7 @@
 #define _GNUstep_H_NSSavePanel
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSPanel.h>
+#import <AppKit/NSPanel.h>
 
 @class NSString;
 @class NSURL;
@@ -79,6 +79,7 @@ enum {
   BOOL _canCreateDirectories;
   BOOL _canSelectHiddenExtension;
   BOOL _isExtensionHidden;
+  BOOL _showsHiddenFiles;
 
   BOOL _delegateHasCompareFilter;
   BOOL _delegateHasShowFilenameFilter;
@@ -89,6 +90,8 @@ enum {
 
   // YES when we stopped because the user pressed 'OK'
   BOOL _OKButtonPressed;
+
+  NSMenu *_showsHiddenFilesMenu;
 }
 
 /*
@@ -106,29 +109,41 @@ enum {
 - (void) setPrompt: (NSString *)prompt;
 - (NSString *) prompt;
 
-#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+- (void) setNameFieldStringValue:(NSString *)value;
+- (NSString *) nameFieldStringValue;
+#endif
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 - (void) setNameFieldLabel: (NSString *)label;
 - (NSString *) nameFieldLabel;
 - (void) setMessage: (NSString *)message;
 - (NSString *) message;
+#endif
 
 /*
  * Hidding extensions
  */
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2, GS_API_LATEST)
 - (void) setCanSelectHiddenExtension: (BOOL) flag;
 - (BOOL) canSelectHiddenExtension;
 - (BOOL) isExtensionHidden;
 - (void) setExtensionHidden: (BOOL) flag;
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+- (BOOL) showsHiddenFiles;
+- (void) setShowsHiddenFiles: (BOOL) flag;
+#endif
+
 /*
  * Setting Directory and File Type
  */
-- (NSString *) requiredFileType;
 - (void) setDirectory: (NSString *)path;
+- (NSString *) requiredFileType;
 - (void) setRequiredFileType: (NSString *)fileType;
 
-#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 - (void) setAllowedFileTypes: (NSArray *)types;
 - (void) setAllowsOtherFileTypes: (BOOL)flag;
 - (NSArray *) allowedFileTypes;
@@ -140,7 +155,7 @@ enum {
 
 - (void) validateVisibleColumns;
 
-#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 - (void) setCanCreateDirectories: (BOOL)flag;
 - (BOOL) canCreateDirectories;
 #endif
@@ -148,13 +163,13 @@ enum {
 /*
  * Running the NSSavePanel
  */
-- (int) runModalForDirectory: (NSString *)path file: (NSString *)filename;
-- (int) runModal;
+- (NSInteger) runModal;
+- (NSInteger) runModalForDirectory: (NSString *)path file: (NSString *)filename;
 
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
-- (int) runModalForDirectory: (NSString *)path
-			file: (NSString *)filename
-	    relativeToWindow: (NSWindow*)window;
+- (NSInteger) runModalForDirectory: (NSString *)path
+                              file: (NSString *)filename
+                  relativeToWindow: (NSWindow*)window;
 - (void) beginSheetForDirectory: (NSString *)path
 			   file: (NSString *)filename
 		 modalForWindow: (NSWindow *)docWindow
@@ -172,6 +187,9 @@ enum {
 - (NSURL *) URL;
 - (BOOL) isExpanded;
 #endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+- (NSURL *) directoryURL;
+#endif
 
 /*
  * Target and Action Methods
