@@ -33,11 +33,11 @@
 #define _GNUstep_H_NSTextView
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSText.h>
-#include <AppKit/NSInputManager.h>
-#include <AppKit/NSDragging.h>
-#include <AppKit/NSTextAttachment.h>
-#include <AppKit/NSUserInterfaceValidation.h>
+#import <AppKit/NSText.h>
+#import <AppKit/NSInputManager.h>
+#import <AppKit/NSDragging.h>
+#import <AppKit/NSTextAttachment.h>
+#import <AppKit/NSUserInterfaceValidation.h>
 
 @class NSTimer;
 @class NSTextContainer;
@@ -111,6 +111,7 @@ therefore be stored in the NSLayoutManager to avoid problems.
     would be very awkward if they weren't. */
     unsigned allows_undo:1;
     unsigned smart_insert_delete:1;
+    unsigned continuous_spell_checking:1;
   /* End of shared attributes. */
 
 
@@ -228,13 +229,12 @@ therefore be stored in the NSLayoutManager to avoid problems.
 
   /* Ivar to store the location where text is going to be inserted during
    * a DnD operation.
-   * Note: This used to be a range attribute with a different meaning; the
-   * _dragTargetUnused attribute is present just to preserve the size of
-   * NSTextView instances.
-   * FIXME: Drop this attribute on the next major release.
    */
-  unsigned int _dragTargetLocation;
-  unsigned int _dragTargetUnused;
+  NSUInteger _dragTargetLocation;
+
+  /* Ivar used to implement coalescing of undo operations */
+  id _undoObject;
+
   /*
   TODO:
   Still need to figure out what "proper behavior" is when moving between two
@@ -244,6 +244,10 @@ therefore be stored in the NSLayoutManager to avoid problems.
   NSParagraphStyle *_defaultParagraphStyle;
   NSDictionary *_linkTextAttributes;
   NSRange _markedRange;
+
+  // Text checking (spelling/grammar)
+  NSTimer *_textCheckingTimer;
+  NSRect _lastCheckedRect;
 }
 
 
@@ -581,11 +585,18 @@ already been laid out. */
 
 /*** Smart copy/paste/delete support ***/
 
--(NSRange) smartDeleteRangeForProposedRange: (NSRange)proposedCharRange;
--(void) smartInsertForString: (NSString *)aString
-	      replacingRange: (NSRange)charRange
-		beforeString: (NSString **)beforeString
-		 afterString: (NSString **)afterString;
+- (NSRange) smartDeleteRangeForProposedRange: (NSRange)proposedCharRange;
+- (void) smartInsertForString: (NSString *)aString
+	       replacingRange: (NSRange)charRange
+		 beforeString: (NSString **)beforeString
+		  afterString: (NSString **)afterString;
+- (NSString *) smartInsertAfterStringForString: (NSString *)aString
+				replacingRange: (NSRange)charRange;
+- (NSString *) smartInsertBeforeStringForString: (NSString *)aString
+				 replacingRange: (NSRange)charRange;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (void) toggleSmartInsertDelete: (id)sender;
+#endif
 
 
 /** TODO: categorize */
