@@ -151,6 +151,7 @@ PACKAGE_SCOPE
 
   unsigned int _autoresizingMask;
   NSFocusRingType _focusRingType;
+  NSRect _autoresizingFrameError;
 }
 
 /*
@@ -178,6 +179,9 @@ PACKAGE_SCOPE
 - (void) sortSubviewsUsingFunction: (int (*)(id ,id ,void*))compare
 			   context: (void*)context;
 - (NSArray*) subviews;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (void) setSubviews: (NSArray *)newSubviews;
+#endif
 - (NSView*) superview;
 - (NSWindow*) window;
 - (void) viewWillMoveToSuperview: (NSView*)newSuper;
@@ -198,26 +202,26 @@ PACKAGE_SCOPE
 /*
  * Modifying the Frame Rectangle
  */
-- (float) frameRotation;
+- (CGFloat) frameRotation;
 - (NSRect) frame;
 - (void) setFrame: (NSRect)frameRect;
 - (void) setFrameOrigin: (NSPoint)newOrigin;
-- (void) setFrameRotation: (float)angle;
+- (void) setFrameRotation: (CGFloat)angle;
 - (void) setFrameSize: (NSSize)newSize;
 
 /*
  * Modifying the Coordinate System
  */
-- (float) boundsRotation;
+- (CGFloat) boundsRotation;
 - (NSRect) bounds;
 - (void) setBounds: (NSRect)aRect;
 - (void) setBoundsOrigin: (NSPoint)newOrigin;
-- (void) setBoundsRotation: (float)angle;
+- (void) setBoundsRotation: (CGFloat)angle;
 - (void) setBoundsSize: (NSSize)newSize;
 
 - (void) translateOriginToPoint: (NSPoint)point;
 - (void) scaleUnitSquareToSize: (NSSize)newSize;
-- (void) rotateByAngle: (float)angle;
+- (void) rotateByAngle: (CGFloat)angle;
 
 - (BOOL) isFlipped;
 - (BOOL) isRotatedFromBase;
@@ -517,15 +521,15 @@ PACKAGE_SCOPE
 /*
  * Pagination
  */
-- (void) adjustPageHeightNew: (float*)newBottom
-                         top: (float)oldTop
-                      bottom: (float)oldBottom
-                       limit: (float)bottomLimit;
-- (void) adjustPageWidthNew: (float*)newRight
-                       left: (float)oldLeft
-                      right: (float)oldRight
-                      limit: (float)rightLimit;
-- (float) heightAdjustLimit;
+- (void) adjustPageHeightNew: (CGFloat*)newBottom
+                         top: (CGFloat)oldTop
+                      bottom: (CGFloat)oldBottom
+                       limit: (CGFloat)bottomLimit;
+- (void) adjustPageWidthNew: (CGFloat*)newRight
+                       left: (CGFloat)oldLeft
+                      right: (CGFloat)oldRight
+                      limit: (CGFloat)rightLimit;
+- (CGFloat) heightAdjustLimit;
 - (BOOL) knowsPagesFirst: (int*)firstPageNum
                     last: (int*)lastPageNum;
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
@@ -533,7 +537,7 @@ PACKAGE_SCOPE
 #endif
 - (NSPoint) locationOfPrintRect: (NSRect)aRect;
 - (NSRect) rectForPage: (int)page;
-- (float) widthAdjustLimit;
+- (CGFloat) widthAdjustLimit;
 
 /*
  * Writing Conforming PostScript
