@@ -29,33 +29,21 @@
 
 #include <math.h>
 #include <config.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSTask.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSValue.h>
-#include <AppKit/AppKitExceptions.h>
-#include <AppKit/NSAffineTransform.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSPrinter.h>
-#include <AppKit/NSPrintPanel.h>
-#include <AppKit/NSPrintInfo.h>
-#include <AppKit/NSPrintOperation.h>
-#include <AppKit/NSWorkspace.h>
-#include <AppKit/PSOperators.h>
-#include "GSCUPSPrintOperation.h"
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSString.h>
+#import <AppKit/NSGraphicsContext.h>
+#import <AppKit/NSView.h>
+#import <AppKit/NSPrinter.h>
+#import <AppKit/NSPrintPanel.h>
+#import <AppKit/NSPrintInfo.h>
+#import <AppKit/NSPrintOperation.h>
+#import "GSGuiPrivate.h"
+#import "GSCUPSPrintOperation.h"
 #include <cups/cups.h>
 
 
-
-
 //A subclass of GSPrintOperation, NOT NSPrintOperation.
 @implementation GSCUPSPrintOperation
 //
@@ -63,7 +51,6 @@
 //
 + (id) allocWithZone: (NSZone*)zone
 {
-  NSDebugMLLog(@"GSPrinting", @"");
   return NSAllocateObject(self, 0, zone);
 }
 
@@ -73,6 +60,8 @@
 {
   self = [super initWithView: aView
                    printInfo: aPrintInfo];
+  if (self == nil)
+    return nil;
 
   _path = [NSTemporaryDirectory()
 	    stringByAppendingPathComponent: @"GSCUPSPrintJob-"];
@@ -91,41 +80,16 @@
 
 - (BOOL) _deliverSpooledResult
 {
-  //int copies;
-  //NSDictionary *dict;
-  //NSTask *task;
   NSString *name, *status;
-  //NSMutableArray *args;
   
-  NSDebugMLLog(@"GSPrinting", @"");
   name = [[[self printInfo] printer] name];
-  status = [NSString stringWithFormat: @"Spooling to printer %@.", name];
+  status = [NSString stringWithFormat: _(@"Spooling to printer %@."), name];
   [[self printPanel] _setStatusStringValue: status];
 
   cupsPrintFile( [name UTF8String], 
                  [_path UTF8String], 
                  [_path UTF8String], 
                  0, NULL );
-                  
-
-  //dict = [[self printInfo] dictionary];
-  //args = [NSMutableArray array];
-  //copies = [[dict objectForKey: NSPrintCopies] intValue];
-  //if (copies > 1)
-  //  [args addObject: [NSString stringWithFormat: @"-#%0d", copies]];
-  //if ([name isEqual: @"Unknown"] == NO)
-  // {
-  //    [args addObject: @"-P"];
-  //    [args addObject: name];
-  //  }
-  //[args addObject: _path];
-
-  //task = [NSTask new];
-  //[task setLaunchPath: @"lpr"];
-  //[task setArguments: args];
-  //[task launch];
-  //[task waitUntilExit];
-  //AUTORELEASE(task);
   return YES;
 }
 
@@ -134,7 +98,6 @@
   NSMutableDictionary *info;
   NSString *output;
   
-  //NSDebugMLLog(@"GSPrinting", @"_path is %@", _path);
   if (_context)
     {
       NSDebugMLLog(@"GSPrinting", @"Already had context, returning it.");
@@ -154,9 +117,17 @@
   
   [info setObject: _path 
            forKey: @"NSOutputFile"];
-  
-  [info setObject: NSGraphicsContextPSFormat
-           forKey: NSGraphicsContextRepresentationFormatAttributeName];
+
+  if ([[[_path pathExtension] lowercaseString] isEqualToString: @"pdf"])
+    {
+      [info setObject: NSGraphicsContextPDFFormat
+	       forKey: NSGraphicsContextRepresentationFormatAttributeName];
+    }
+  else
+    {
+      [info setObject: NSGraphicsContextPSFormat
+	       forKey: NSGraphicsContextRepresentationFormatAttributeName];
+    }
            
   _context = RETAIN([NSGraphicsContext graphicsContextWithAttributes: info]);
 
