@@ -60,7 +60,6 @@ NSString *GSCUPSDummyPrinterName = @"GSCUPSDummyPrinter";
 //
 +(void) initialize
 {
-  NSDebugMLLog(@"GSPrinting", @"");
   if (self == [GSCUPSPrinter class])
     {
       // Initial version
@@ -71,7 +70,6 @@ NSString *GSCUPSDummyPrinterName = @"GSCUPSDummyPrinter";
 
 +(id) allocWithZone: (NSZone*) zone
 {
-  NSDebugMLLog(@"GSPrinting", @"");
   return NSAllocateObject(self, 0, zone);
 }
 
@@ -111,7 +109,7 @@ static BOOL didWarn;
 
       [printer parsePPDAtPath: ppdPath];
 
-      return printer;
+      return AUTORELEASE(printer);
     }
 
   printer = [[GSCUPSPrinter alloc]
@@ -144,16 +142,16 @@ static BOOL didWarn;
   int n;
 
   set = [[NSMutableSet alloc] init];
-  AUTORELEASE( set );
+  AUTORELEASE(set);
 
-  numDests = cupsGetDests( &dests );
+  numDests = cupsGetDests(&dests);
   
-  for( n = 0; n < numDests; n++ )
+  for (n = 0; n < numDests; n++)
     {
       [set addObject: [NSString stringWithCString: dests[n].name]];
     }
 
-  cupsFreeDests( numDests, dests );
+  cupsFreeDests(numDests, dests);
 
   // No printer found, return at least the dummy printer
   if ([set count] == 0)
@@ -164,18 +162,5 @@ static BOOL didWarn;
   return [set allObjects];
 }
 
-
--(id) initWithCoder: (NSCoder*) coder
-{
-  return [super initWithCoder: coder];
-}
-
-
--(void) encodeWithCoder: (NSCoder*) coder
-{
-  [super encodeWithCoder: coder];
-}
-
-
 @end
 
