@@ -32,12 +32,14 @@
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSProcessInfo.h>
 
+#import "GSGuiPrivate.h"
 #import "AppKit/NSApplication.h"
 #import "AppKit/NSBitmapImageRep.h"
 #import "AppKit/NSNibLoading.h"
 #import "AppKit/NSEvent.h"
 #import "AppKit/NSGraphicsContext.h"
 #import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
 #import "AppKit/NSColor.h"
 #import "AppKit/NSView.h"
 #import "AppKit/NSWindow.h"
@@ -90,7 +92,7 @@ NSApplicationMain(int argc, const char **argv)
 
   DESTROY(NSApp);
 
-  RELEASE(pool);
+  [pool drain];
 
   return 0;
 }
@@ -344,6 +346,166 @@ void NSDrawBitmap(NSRect rect,
   RELEASE(bitmap);
 }
 
+/**
+ * Tiles an image in a rect, starting from the lower-left-hand corner
+ */
+static void GSDrawRepeatingImage(NSRect aRect, NSImage *image, NSCompositingOperation op,
+				 CGFloat fraction, BOOL flipped)
+{
+  const NSSize imageSize = [image size];
+  if (imageSize.width <= 0 ||
+      imageSize.height <= 0)
+    {
+      return;
+    }
+
+  [NSGraphicsContext saveGraphicsState];
+  NSRectClip(aRect); 
+  
+  {
+    const NSInteger numHorizontal = ceil(aRect.size.width / imageSize.width);
+    const NSInteger numVertical = ceil(aRect.size.height / imageSize.height);
+    NSInteger x, y;
+    
+    if (numHorizontal > 0 && numVertical > 0)
+      {
+	for (x = 0; x < numHorizontal; x++)
+	  {
+	    for (y = 0; y < numVertical; y++)
+	      {
+		NSRect drawRect;
+		drawRect.size = imageSize;
+		drawRect.origin.x = aRect.origin.x + (x * imageSize.width);
+		drawRect.origin.y = flipped ? (NSMaxY(aRect) - ((y + 1) * imageSize.height))
+		  : (aRect.origin.y + (y * imageSize.height));
+
+		[image drawInRect: drawRect
+			 fromRect: NSZeroRect
+			operation: op
+			 fraction: fraction
+		   respectFlipped: YES
+			    hints: nil];
+	      }
+	  }
+      }
+  }
+
+  [NSGraphicsContext restoreGraphicsState];
+}
+
+void NSDrawNinePartImage(NSRect aRect, NSImage *topLeft, NSImage *topMiddle,
+			 NSImage *topRight, NSImage *centerLeft,
+			 NSImage *centerMiddle, NSImage *centerRight,
+			 NSImage *bottomLeft, NSImage *bottomMiddle,
+			 NSImage *bottomRight, NSCompositingOperation op,
+			 CGFloat fraction, BOOL flipped)
+{
+  NSView *focusView = [NSView focusView];
+
+  if (nil != focusView)
+    {
+      aRect = [focusView centerScanRect: aRect];
+    }
+  
+  [NSGraphicsContext saveGraphicsState];
+
+  // Protects against the case when the smallest source image is larger than aRect
+  NSRectClip(aRect); 
+
+  {
+    NSRect topLeftRect;
+    NSRect topMiddleRect;
+    NSRect topRightRect;
+    NSRect centerLeftRect;
+    NSRect centerMiddleRect;
+    NSRect centerRightRect;
+    NSRect bottomLeftRect; 
+    NSRect bottomMiddleRect;
+    NSRect bottomRightRect;
+
+    // These two images are the only sizes we use in addition to aRect
+    topLeftRect.size = [topLeft size];
+    bottomRightRect.size = [bottomRight size];
+
+    // Fill in the rest of the sizes
+    topMiddleRect.size = NSMakeSize(MAX(0, aRect.size.width - 
+					topLeftRect.size.width - 
+					bottomRightRect.size.width),
+				    topLeftRect.size.height);
+    topRightRect.size = NSMakeSize(bottomRightRect.size.width,
+				   topLeftRect.size.height);
+    centerLeftRect.size = NSMakeSize(topLeftRect.size.width,
+				     MAX(0, aRect.size.height - 
+					 topLeftRect.size.height - 
+					 bottomRightRect.size.height));
+    centerMiddleRect.size = NSMakeSize(topMiddleRect.size.width,
+				       centerLeftRect.size.height);
+    centerRightRect.size = NSMakeSize(topRightRect.size.width,
+				      centerLeftRect.size.height);
+    bottomLeftRect.size = NSMakeSize(topLeftRect.size.width,
+				     bottomRightRect.size.height);
+    bottomMiddleRect.size = NSMakeSize(centerMiddleRect.size.width,
+				       bottomRightRect.size.height);
+
+    // Now fill in the positions
+
+    if (flipped)
+      {
+	topLeftRect.origin = aRect.origin;
+	topMiddleRect.origin = NSMakePoint(NSMaxX(topLeftRect),
+					   aRect.origin.y);
+	topRightRect.origin = NSMakePoint(NSMaxX(aRect) - NSWidth(topRightRect),
+					  aRect.origin.y);
+	centerLeftRect.origin = NSMakePoint(aRect.origin.x,
+					    NSMaxY(topLeftRect));
+	centerMiddleRect.origin = NSMakePoint(NSMaxX(topLeftRect),
+					      NSMaxY(topLeftRect));
+	centerRightRect.origin = NSMakePoint(NSMaxX(topMiddleRect),
+					     NSMaxY(topMiddleRect));
+	bottomLeftRect.origin = NSMakePoint(aRect.origin.x,
+					    NSMaxY(aRect) - NSHeight(bottomLeftRect));
+	bottomMiddleRect.origin = NSMakePoint(NSMaxX(centerLeftRect),
+					      NSMaxY(centerLeftRect));
+	bottomRightRect.origin = NSMakePoint(NSMaxX(aRect) - NSWidth(bottomRightRect),
+					     NSMaxY(aRect) - NSHeight(bottomRightRect));
+      }
+    else
+      {
+	bottomLeftRect.origin = aRect.origin;
+	bottomMiddleRect.origin = NSMakePoint(NSMaxX(bottomLeftRect),
+					      aRect.origin.y);
+	bottomRightRect.origin = NSMakePoint(NSMaxX(aRect) - NSWidth(bottomRightRect),
+					     aRect.origin.y);
+	centerLeftRect.origin = NSMakePoint(aRect.origin.x,
+					    NSMaxY(bottomLeftRect));
+	centerMiddleRect.origin = NSMakePoint(NSMaxX(bottomLeftRect),
+					      NSMaxY(bottomLeftRect));
+	centerRightRect.origin = NSMakePoint(NSMaxX(bottomMiddleRect),
+					     NSMaxY(bottomMiddleRect));
+	topLeftRect.origin = NSMakePoint(aRect.origin.x,
+					 NSMaxY(aRect) - NSHeight(topLeftRect));
+	topMiddleRect.origin = NSMakePoint(NSMaxX(centerLeftRect),
+					   NSMaxY(centerLeftRect));
+	topRightRect.origin = NSMakePoint(NSMaxX(aRect) - NSWidth(topRightRect),
+					  NSMaxY(aRect) - NSHeight(topRightRect));
+      }
+	
+    // Draw the images left-to-right, bottom-to-top
+
+    [bottomLeft drawInRect: bottomLeftRect fromRect: NSZeroRect operation: op fraction: fraction respectFlipped: YES hints: nil];
+    GSDrawRepeatingImage(bottomMiddleRect, bottomMiddle, op, fraction, flipped);
+    [bottomRight drawInRect: bottomRightRect fromRect: NSZeroRect operation: op fraction: fraction respectFlipped: YES hints: nil];   
+    GSDrawRepeatingImage(centerLeftRect, centerLeft, op, fraction, flipped);
+    GSDrawRepeatingImage(centerMiddleRect, centerMiddle, op, fraction, flipped);
+    GSDrawRepeatingImage(centerRightRect, centerRight, op, fraction, flipped);
+    [topLeft drawInRect: topLeftRect fromRect: NSZeroRect operation: op fraction: fraction respectFlipped: YES hints: nil];
+    GSDrawRepeatingImage(topMiddleRect, topMiddle, op, fraction, flipped);
+    [topRight drawInRect: topRightRect fromRect: NSZeroRect operation: op fraction: fraction respectFlipped: YES hints: nil];
+  }
+
+  [NSGraphicsContext restoreGraphicsState]; // Restore clipping region
+}
+
 /*
  * Rectangle Drawing 
  */
@@ -409,12 +571,16 @@ void
 NSRectFillListWithColors(const NSRect *rects, NSColor **colors, int count)
 {
   int i;
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSgsave(ctxt);
 
   for (i = 0; i < count; i++)
     {
       [colors[i] set];
       NSRectFill(rects[i]);
     }
+
+  DPSgrestore(ctxt);
 }
 
 void NSRectFillListWithGrays(const NSRect *rects, const float *grays, 
@@ -422,6 +588,7 @@ void NSRectFillListWithGrays(const NSRect *rects, const float *grays,
 {
   int i;
   NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSgsave(ctxt);
 
   for (i = 0; i < count; i++)
     {
@@ -429,6 +596,8 @@ void NSRectFillListWithGrays(const NSRect *rects, const float *grays,
       DPSrectfill(ctxt,  NSMinX(rects[i]), NSMinY(rects[i]), 
 		  NSWidth(rects[i]), NSHeight(rects[i]));
     }
+
+  DPSgrestore(ctxt);
 }
 
 void NSRectFillUsingOperation(NSRect aRect, NSCompositingOperation op)
@@ -458,12 +627,16 @@ NSRectFillListWithColorsUsingOperation(const NSRect *rects,
 				       NSCompositingOperation op)
 {
   int i;
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSgsave(ctxt);
 
   for (i = 0; i < num; i++)
     {
       [colors[i] set];
       NSRectFillUsingOperation(rects[i], op);
     }
+
+  DPSgrestore(ctxt);
 }
 
 
@@ -473,13 +646,14 @@ void NSDottedFrameRect(const NSRect aRect)
 {
   float dot_dash[] = {1.0, 1.0};
   NSGraphicsContext *ctxt = GSCurrentContext();
-
+  DPSgsave(ctxt);
   DPSsetgray(ctxt, NSBlack);
   DPSsetlinewidth(ctxt, 1.0);
   // FIXME
   DPSsetdash(ctxt, dot_dash, 2, 0.0);
   DPSrectstroke(ctxt,  NSMinX(aRect) + 0.5, NSMinY(aRect) + 0.5,
 		NSWidth(aRect) - 1.0, NSHeight(aRect) - 1.0);
+  DPSgrestore(ctxt);
 }
 
 void NSFrameRect(const NSRect aRect)
@@ -598,10 +772,11 @@ NSDrawButton(const NSRect aRect, const NSRect clipRect)
       rect = NSDrawTiledRects(aRect, clipRect,
 			       up_sides, grays, 6);
     }
-
+  DPSgsave(ctxt);
   DPSsetgray(ctxt, NSLightGray);
   DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
 	      NSWidth(rect), NSHeight(rect));
+  DPSgrestore(ctxt);
 }
 
 void
@@ -616,6 +791,8 @@ NSDrawGrayBezel(const NSRect aRect, const NSRect clipRect)
   NSRect rect;
   NSGraphicsContext *ctxt = GSCurrentContext();
 
+  DPSgsave(ctxt);
+
   if (GSWViewIsFlipped(ctxt) == YES)
     {
       rect = NSDrawTiledRects(aRect, clipRect,
@@ -638,6 +815,7 @@ NSDrawGrayBezel(const NSRect aRect, const NSRect clipRect)
   DPSsetgray(ctxt, NSLightGray);
   DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
 	      NSWidth(rect), NSHeight(rect));
+  DPSgrestore(ctxt);
 }
 
 void 
@@ -663,9 +841,11 @@ NSDrawGroove(const NSRect aRect, const NSRect clipRect)
 			       up_sides, grays, 8);
     }
 
+  DPSgsave(ctxt);
   DPSsetgray(ctxt, NSLightGray);
   DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
 	      NSWidth(rect), NSHeight(rect));
+  DPSgrestore(ctxt);
 }
 
 void 
@@ -691,9 +871,11 @@ NSDrawWhiteBezel(const NSRect aRect,  const NSRect clipRect)
 			       up_sides, grays, 8);
     }
 
+  DPSgsave(ctxt);
   DPSsetgray(ctxt, NSWhite);
   DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
 	      NSWidth(rect), NSHeight(rect));
+  DPSgrestore(ctxt);
 }
 
 void 
@@ -709,6 +891,7 @@ NSDrawDarkBezel(NSRect aRect, NSRect clipRect)
 		   NSLightGray, NSLightGray, NSBlack, NSBlack};
   NSRect rect;
   NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSgsave(ctxt);
 
   if (GSWViewIsFlipped(ctxt) == YES)
     {
@@ -732,6 +915,7 @@ NSDrawDarkBezel(NSRect aRect, NSRect clipRect)
   DPSsetgray(ctxt, NSLightGray);
   DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
 	      NSWidth(rect), NSHeight(rect));
+  DPSgrestore(ctxt);
 }
 
 void 
@@ -745,6 +929,7 @@ NSDrawLightBezel(NSRect aRect, NSRect clipRect)
   		   NSBlack, NSBlack, NSBlack, NSBlack};
   NSRect rect;
   NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSgsave(ctxt);
 
   if (GSWViewIsFlipped(ctxt) == YES)
     {
@@ -768,6 +953,7 @@ NSDrawLightBezel(NSRect aRect, NSRect clipRect)
   DPSsetgray(ctxt, NSWhite);
   DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
 	      NSWidth(rect), NSHeight(rect));
+  DPSgrestore(ctxt);
 }
 
 void
@@ -797,16 +983,21 @@ NSDrawFramePhoto(const NSRect aRect, const NSRect clipRect)
 			       up_sides, grays, 6);
     }
 
+  DPSgsave(ctxt);
   DPSsetgray(ctxt, NSLightGray);
   DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
 	      NSWidth(rect), NSHeight(rect));
+  DPSgrestore(ctxt);
 }
 
 void 
 NSDrawWindowBackground(NSRect aRect)
 {
+  NSGraphicsContext *ctxt = GSCurrentContext();  
+  DPSgsave(ctxt);
   [[NSColor windowBackgroundColor] set];
-  NSRectFill(aRect);  
+  NSRectFill(aRect);
+  DPSgrestore(ctxt);
 }
 
 float 
@@ -818,6 +1009,9 @@ NSLinkFrameThickness(void)
 void 
 NSFrameLinkRect(NSRect aRect, BOOL isDestination)
 {
+  NSGraphicsContext *ctxt = GSCurrentContext();  
+  DPSgsave(ctxt);
+
   if (isDestination)
     {
       [[NSColor redColor] set];
@@ -828,6 +1022,7 @@ NSFrameLinkRect(NSRect aRect, BOOL isDestination)
     }
 
   NSFrameRectWithWidth(aRect, NSLinkFrameThickness());
+  DPSgrestore(ctxt);
 }
 
 void NSSetFocusRingStyle(NSFocusRingPlacement placement)
