@@ -99,7 +99,7 @@ cache fairly aggressively without having to worry about memory consumption.
 #define CACHE_STEP 192
 
 
-typedef struct GSHorizontalTypesetter_glyph_cache_s
+struct GSHorizontalTypesetter_glyph_cache_s
 {
   /* These fields are filled in by the caching: */
   NSGlyph g;
@@ -119,8 +119,8 @@ typedef struct GSHorizontalTypesetter_glyph_cache_s
   NSPoint pos;    /* relative to the line's baseline */
   NSSize size;    /* height is used only for attachments */
   BOOL dont_show, outside_line_frag;
-} glyph_cache_t;
-
+};
+typedef struct GSHorizontalTypesetter_glyph_cache_s glyph_cache_t;
 
 /* TODO: if we could know whether the layout manager had been modified since
 the last time or not, we wouldn't need to clear the cache every time */
@@ -250,6 +250,9 @@ the last time or not, we wouldn't need to clear the cache every time */
 
       g->dont_show = NO;
       g->outside_line_frag = NO;
+
+      // FIXME: This assumes the layout manager implements this GNUstep extension
+      g->size = [curLayoutManager advancementForGlyphAtIndex: cache_base + cache_length];
     }
 }
 
@@ -309,13 +312,13 @@ including gi will have been cached.
 }
 
 
-typedef struct GSHorizontalTypesetter_line_frag_s
+struct GSHorizontalTypesetter_line_frag_s
 {
   NSRect rect;
   float last_used;
   unsigned int last_glyph; /* last_glyph+1, actually */
-} line_frag_t;
-
+};
+typedef struct GSHorizontalTypesetter_line_frag_s line_frag_t;
 
 -(void) fullJustifyLine: (line_frag_t *)lf : (int)num_line_frags
 {
@@ -683,17 +686,6 @@ restart: ;
     
     NSFont *f = cache->font;
 
-    /*
-    TODO: This is kindof ugly, but -advancementForGlyph: is responsible for
-    ~10% of execution time when handling huge amounts of text (according to
-    profiling, 2004-08-09). Would be cleaner to get rid of the font/fontInfo
-    indirection.
-    */
-    id fontInfo =  [f fontInfo];
-    NSSize (*advancementForGlyph)(id, SEL, NSGlyph)
-      = (NSSize(*)(id, SEL, NSGlyph))[fontInfo methodForSelector:
-	  @selector(advancementForGlyph:)];
-
     float f_ascender = [f ascender], f_descender = -[f descender];
 
     NSGlyph last_glyph = NSNullGlyph;
@@ -771,9 +763,6 @@ restart: ;
 	  {
 	    float new_height;
 	    f = g->font;
-	    fontInfo = [f fontInfo];
-	    advancementForGlyph = (NSSize(*)(id, SEL, NSGlyph))
-	      [fontInfo methodForSelector: @selector(advancementForGlyph:)];
 	    f_ascender = [f ascender];
 	    f_descender = -[f descender];
 	    last_glyph = NSNullGlyph;
@@ -983,9 +972,6 @@ restart: ;
 
 	    last_p = g->pos = p;
 	    /* Only the width is used. */
-	    g->size = advancementForGlyph(fontInfo,
-					  @selector(advancementForGlyph:),
-					  g->g);
 	    p.x += g->size.width;
 	  }
 
@@ -1210,7 +1196,7 @@ restart: ;
       we create a new instance and let it handle the call. */
       GSHorizontalTypesetter *temp;
 
-      temp = [[isa alloc] init];
+      temp = [[object_getClass(self) alloc] init];
       ret = [temp layoutGlyphsInLayoutManager: layoutManager
 			      inTextContainer: textContainer
 			 startingAtGlyphIndex: glyphIndex
