@@ -282,10 +282,16 @@ new_label (NSString *value)
   
   if (nil_or_not_of_class (icon, [NSImage class]))
     {
-      icon = [NSImage imageNamed: @"NSApplicationIcon"];
-      
+      icon = [NSImage imageNamed:
+	value_from_info_plist_for_key (@"ApplicationIcon")];
+
       if (nil_or_not_of_class (icon, [NSImage class]))
-	icon = [NSApp applicationIconImage];
+	{
+	  icon = [NSImage imageNamed: @"NSApplicationIcon"];
+	  
+	  if (nil_or_not_of_class (icon, [NSImage class]))
+	    icon = [NSApp applicationIconImage];
+	}
     }
   /* Release */
   if (dictionary)
@@ -348,10 +354,25 @@ new_label (NSString *value)
 
   if (nil_or_not_of_class (authors, [NSArray class]))
     {
-      authors = value_from_info_plist_for_key (@"Authors");
-      
-      // if (nil_or_not_of_class (authors, [NSArray class]))
-      //   authors = [NSArray arrayWithObject: @"Unknown"];
+      if ([authors isKindOfClass: [NSString class]])
+	{
+	  authors = [NSArray arrayWithObject: authors];
+	}
+      else
+	{
+	  authors = value_from_info_plist_for_key (@"Authors");
+	  if (nil_or_not_of_class (authors, [NSArray class]))
+	    {
+	      if ([authors isKindOfClass: [NSString class]])
+		{
+		  authors = [NSArray arrayWithObject: authors];
+		}
+	      else
+		{
+		  authors = [NSArray arrayWithObject: @"Unknown"];
+		}
+	    }
+	}
     }
   /* URL */
   if (dictionary)
@@ -397,13 +418,10 @@ new_label (NSString *value)
   /*
    * Create GUI Objects 
    */
-  f = NSZeroRect;
-  if (icon != nil)
-    {
-      f.size = [icon size];
-    }
+  f = NSMakeRect(0, 0, 48, 48);
 
   iconButton = AUTORELEASE([[NSButton alloc] initWithFrame: f]); 
+  [[iconButton cell] setImageScaling: NSImageScaleProportionallyUpOrDown];
   [iconButton setImage: icon];
   [iconButton setBordered: NO];
   [iconButton setImagePosition: NSImageOnly];
