@@ -498,7 +498,6 @@ Private method used internally by GSLayoutManager for sanity checking.
 -(void) _generateRunsToCharacter: (unsigned int)last
 {
   glyph_run_head_t *context[SKIP_LIST_DEPTH];
-  int positions[SKIP_LIST_DEPTH];
   glyph_run_head_t *h;
   unsigned int pos;
   unsigned int length;
@@ -518,7 +517,6 @@ Private method used internally by GSLayoutManager for sanity checking.
     {
       while (h->next) pos += h->char_length, h = h->next;
       context[level - 1] = h;
-      positions[level - 1] = pos;
       h++;
     }
   h--;
@@ -990,6 +988,43 @@ Fills in all glyph holes up to last. only looking at levels below level
   return cpos + r->glyphs[glyphIndex - pos].char_offset;
 }
 
+/**
+ * GNUstep extension
+ */
+- (NSSize) advancementForGlyphAtIndex: (unsigned int)glyphIndex
+{
+  glyph_run_t *r;
+  unsigned int pos, cpos;
+
+  if (glyphs->glyph_length <= glyphIndex)
+    {
+      [self _generateGlyphsUpToGlyph: glyphIndex];
+      if (glyphs->glyph_length <= glyphIndex)
+        {
+          [NSException raise: NSRangeException
+                       format: @"%s glyph index out of range", __PRETTY_FUNCTION__];
+          return NSMakeSize(0,0);
+        }
+    }
+
+  r = run_for_glyph_index(glyphIndex, glyphs, &pos, &cpos);
+  if (!r)
+    {
+      [NSException raise: NSRangeException
+                   format: @"%s glyph index out of range", __PRETTY_FUNCTION__];
+      return NSMakeSize(0,0);
+    }
+
+  if (r->head.glyph_length <= glyphIndex - pos)
+    {
+      [NSException raise: NSRangeException
+		  format: @"%s internal error!", __PRETTY_FUNCTION__];
+      return NSMakeSize(0,0);
+    }
+
+  return r->glyphs[glyphIndex - pos].advancement;
+}
+
 - (NSRange) characterRangeForGlyphRange: (NSRange)glyphRange
                        actualGlyphRange: (NSRange *)actualGlyphRange
 {
@@ -3130,7 +3165,11 @@ has).
   return _textStorage;
 }
 
+/**
+ * GNUstep extension
+ */
 - (void) insertGlyphs: (const NSGlyph*)glyph_list
+     withAdvancements: (const NSSize*)advancements
                length: (NSUInteger)length
 forStartingGlyphAtIndex: (NSUInteger)glyph
        characterIndex: (NSUInteger)index
@@ -3172,10 +3211,41 @@ forStartingGlyphAtIndex: (NSUInteger)glyph
       // We expect to get a nominal glyph run
       g->char_offset = i + index - cpos;
       g->g = glyph_list[i];
+      g->advancement = advancements[i];
       g++;
     }
 }
 
+- (void) insertGlyphs: (const NSGlyph*)glyph_list
+               length: (NSUInteger)length
+forStartingGlyphAtIndex: (NSUInteger)glyph
+       characterIndex: (NSUInteger)index
+{
+  glyph_run_t *run;
+  int i;
+  unsigned int gpos, cpos;
+  NSSize advances[length];
+
+  run = run_for_character_index(index, glyphs, &gpos, &cpos);
+  if (!run)
+    {
+      [NSException raise: NSRangeException
+                   format: @"%s glyph index out of range", __PRETTY_FUNCTION__];
+      return;
+    }
+    
+  for (i=0; i<length; i++)
+    {
+      advances[i] = [run->font advancementForGlyph: glyph_list[i]];
+    }
+
+  [self insertGlyphs: glyph_list
+    withAdvancements: advances
+	      length: length
+	forStartingGlyphAtIndex: glyph
+      characterIndex: index];
+}
+
 - (NSUInteger) layoutOptions
 {
   NSUInteger options = 0;
