@@ -26,10 +26,16 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "GNUstepGUI/GSModelLoaderFactory.h"
-#include "GNUstepBase/GSObjCRuntime.h"
+#import "config.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSString.h>
+
+#import "GNUstepGUI/GSModelLoaderFactory.h"
 
 @implementation GSModelLoader
 + (NSString *) type
@@ -55,9 +61,26 @@
      externalNameTable: (NSDictionary *)context
               withZone: (NSZone *)zone
 {
-  [NSException raise: NSInternalInconsistencyException
-	       format: @"Abstract model loader."];
-  return NO;
+  NSData *data = [self dataForFile: fileName];
+
+  if (data != nil)
+    {
+      BOOL loaded = [self loadModelData: data
+                      externalNameTable: context
+                               withZone: zone];
+      if (!loaded)
+        NSLog(@"Could not load Nib file: %@", fileName);
+      return loaded;
+    }
+  else
+    {
+      return NO;
+    }
+}
+
+- (NSData *) dataForFile: (NSString *)fileName
+{
+  return [NSData dataWithContentsOfFile: fileName];
 }
 
 + (NSComparisonResult) _comparePriority: (Class)loader
