@@ -66,6 +66,7 @@
 #import "AppKit/NSNib.h"
 #import "AppKit/NSScreen.h"
 #import "AppKit/NSSound.h"
+#import "AppKit/NSToolbar.h"
 #import "GNUstepGUI/GSInstantiator.h"
 #import "GSGuiPrivate.h"
 
@@ -479,6 +480,12 @@ static BOOL _isInInterfaceBuilder = NO;
       [_realObject setMaxSize: _maxSize];
       [_realObject setTitle: _title];
 
+      if ([_viewClass isKindOfClass: [NSToolbar class]])
+	{
+          // FIXME: No idea what is going on here
+	  [_realObject setToolbar: (NSToolbar*)_viewClass];
+	}
+	  
       [_view _fixSubviews];
 
       // FIXME What is the point of calling -setFrame:display: here? It looks
@@ -1239,11 +1246,21 @@ static BOOL _isInInterfaceBuilder = NO;
           realObject = RETAIN([NSImage imageNamed: _resourceName]);
         }
 
-      // if an object has been substituted, then release the placeholder.
-      if (realObject != nil)
+      if (realObject == nil)
         {
-          RELEASE(self);
+          NSLog(@"Could not load NSCustomResource %@ for class %@", _resourceName, _className);
+          // Use a default instead of the missing object
+          if ([_className isEqual: @"NSSound"])
+            {
+              realObject = RETAIN([NSSound soundNamed: @"Ping"]);
+            }
+          else if ([_className isEqual: @"NSImage"])
+            {
+              realObject  = RETAIN([NSImage imageNamed: @"GNUstep"]);
+            }
         }
+      // The object has been substituted, release the placeholder.
+      RELEASE(self);
     }
   else
     {
@@ -1563,6 +1580,20 @@ static BOOL _isInInterfaceBuilder = NO;
   [self setSource: [instantiator instantiateObject: _src]];
   [self setDestination: [instantiator instantiateObject: _dst]];
 }
+
+- (id) nibInstantiate
+{
+  if ([_src respondsToSelector: @selector(nibInstantiate)])
+    {
+      [self setSource: [_src nibInstantiate]];
+    }
+  if ([_dst respondsToSelector: @selector(nibInstantiate)])
+    {
+      [self setDestination: [_dst nibInstantiate]];
+    }
+  return self;
+}
+
 @end
 
 @implementation NSNibControlConnector (NibCompatibility)
@@ -1575,7 +1606,7 @@ static BOOL _isInInterfaceBuilder = NO;
 - (void) instantiateWithInstantiator: (id<GSInstantiator>)instantiator
 {
   NSRange colonRange = [_tag rangeOfString: @":"];
-  unsigned int location = colonRange.location;
+  NSUInteger location = colonRange.location;
   
   if (location == NSNotFound)
     {
@@ -1995,7 +2026,7 @@ static BOOL _isInInterfaceBuilder = NO;
 {
   NSArray *nameKeys = (NSArray *)NSAllMapTableKeys(_names);
   NSArray *nameValues = (NSArray *)NSAllMapTableValues(_names);
-  int i = [nameValues indexOfObject: name];
+  NSUInteger i = [nameValues indexOfObject: name];
   id result = nil;
 
   if (i != NSNotFound)
