@@ -86,6 +86,7 @@ NSString        *GSMenuHorizontalBackground = @"GSMenuHorizontalBackground";
 NSString        *GSMenuVerticalBackground = @"GSMenuVerticalBackground";
 NSString        *GSMenuHorizontalItem = @"GSMenuHorizontalItem";
 NSString        *GSMenuVerticalItem = @"GSMenuVerticalItem";
+NSString        *GSMenuSeparatorItem = @"GSMenuSeparatorItem";
 
 // Progress indicator part names
 NSString        *GSProgressIndicatorBarDeterminate 
@@ -205,10 +206,8 @@ GSStringFromBorderType(NSBorderType borderType)
     }
 }
 
-
-
-@interface	NSImage (GSTheme)
-+ (NSImage*) _setImage: (NSImage*)image name: (NSString*)name;
+@interface	NSImage (Private)
++ (void) _setImagePath: (NSString*)path name: (NSString*)name;
 @end
 
 @interface	GSTheme (Private)
@@ -236,7 +235,6 @@ GSStringFromBorderType(NSBorderType borderType)
 @implementation GSTheme
 
 static GSTheme			*defaultTheme = nil;
-static NSString			*currentThemeName = nil;
 static GSTheme			*theTheme = nil;
 static NSMutableDictionary	*themes = nil;
 static NSNull			*null = nil;
@@ -246,8 +244,7 @@ typedef	struct {
   NSBundle		*bundle;
   NSColorList		*colors;
   NSColorList		*extraColors[GSThemeSelectedState+1];
-  NSMutableDictionary	*images;
-  NSMutableDictionary	*oldImages;
+  NSMutableSet		*imageNames;
   NSMutableDictionary	*tiles[GSThemeSelectedState+1];
   NSMutableSet		*owned;
   NSImage		*icon;
@@ -261,8 +258,7 @@ typedef	struct {
 #define	_bundle			_internal->bundle
 #define	_colors			_internal->colors
 #define	_extraColors		_internal->extraColors
-#define	_images			_internal->images
-#define	_oldImages		_internal->oldImages
+#define	_imageNames		_internal->imageNames
 #define	_tiles			_internal->tiles
 #define	_owned			_internal->owned
 #define	_icon			_internal->icon
@@ -278,10 +274,17 @@ typedef	struct {
 
   defs = [NSUserDefaults standardUserDefaults];
   name = [defs stringForKey: @"GSTheme"];
-  if (name != currentThemeName && [name isEqual: currentThemeName] == NO)
+  if (0 == [name length])
+    {
+      name = @"GNUstep";
+    }
+  else if ([[name pathExtension] isEqual: @"theme"])
+    {
+      name = [name stringByDeletingPathExtension];
+    }
+  if (NO == [[name lastPathComponent] isEqual: [theTheme name]])
     {
       [self setTheme: [self loadThemeNamed: name]];
-      ASSIGN(currentThemeName, name);	// Don't try to load again.
     }
 }
 
@@ -290,21 +293,15 @@ typedef	struct {
   if (themes == nil)
     {
       themes = [NSMutableDictionary new];
-      [[NSNotificationCenter defaultCenter]
-	addObserver: self
-	selector: @selector(defaultsDidChange:)
-	name: NSUserDefaultsDidChangeNotification
-	object: nil];
       null = RETAIN([NSNull null]);
       defaultTheme = [[self alloc] initWithBundle: nil];
       ASSIGN(theTheme, defaultTheme);
-      ASSIGN(currentThemeName, [defaultTheme name]);
       names = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
 	NSIntMapValueCallBacks, 0);
+      /* Establish the theme specified by the user defaults (if any);
+       */
+      [self defaultsDidChange: nil];
     }
-  /* Establish the theme specified by the user defaults (if any);
-   */
-  [self defaultsDidChange: nil];
 }
 
 + (GSTheme*) loadThemeNamed: (NSString*)aName
@@ -314,7 +311,11 @@ typedef	struct {
   GSTheme	*instance;
   NSString	*theme;
 
-  if ([aName length] == 0)
+  if ([[aName pathExtension] isEqual: @"theme"])
+    {
+      aName = [aName stringByDeletingPathExtension];
+    }
+  if ([aName length] == 0 || [[aName lastPathComponent] isEqual: @"GNUstep"])
     {
       return defaultTheme;
     }
@@ -326,21 +327,17 @@ typedef	struct {
   else
     {
       aName = [aName lastPathComponent];
+    }
 
-      /* Ensure that the theme name has the 'theme' extension.
-       */
-      if ([[aName pathExtension] isEqualToString: @"theme"] == YES)
-	{
-	  theme = aName;
-	}
-      else
-	{
-	  theme = [aName stringByAppendingPathExtension: @"theme"];
-	}
-      if ([theme isEqualToString: @"GNUstep.theme"] == YES)
-	{
-	  return defaultTheme;
-	}
+  /* Ensure that the theme name has the 'theme' extension.
+   */
+  if ([[aName pathExtension] isEqualToString: @"theme"] == YES)
+    {
+      theme = aName;
+    }
+  else
+    {
+      theme = [aName stringByAppendingPathExtension: @"theme"];
     }
 
   bundle = [themes objectForKey: theme];
@@ -418,11 +415,24 @@ typedef	struct {
     }
   if (theme != theTheme)
     {
+      /*
+       * Remove any previous observers...
+       */
+      [[NSNotificationCenter defaultCenter]
+	removeObserver: self];
+
       [theTheme deactivate];
-      DESTROY(currentThemeName);
       ASSIGN (theTheme, theme);
       [theTheme activate];
-      ASSIGN(currentThemeName, [theTheme name]);
+
+      /*
+       * Listen to notifications...
+       */ 
+      [[NSNotificationCenter defaultCenter]
+	addObserver: self
+	   selector: @selector(defaultsDidChange:)
+	       name: NSUserDefaultsDidChangeNotification
+	     object: nil];
     }
 }
 
@@ -435,8 +445,8 @@ typedef	struct {
 {
   NSUserDefaults	*defs;
   NSMutableArray	*searchList;
-  NSArray		*imagePaths;
   NSEnumerator		*enumerator;
+  NSArray		*imagePaths;
   NSString		*imagePath;
   NSArray		*imageTypes;
   NSDictionary		*infoDict;
@@ -469,61 +479,12 @@ typedef	struct {
 
       if (ext != nil && [imageTypes containsObject: ext] == YES)
         {
-	  NSImage	*image;
 	  NSString	*imageName;
 
 	  imageName = [imagePath lastPathComponent];
 	  imageName = [imageName stringByDeletingPathExtension];
-
-	  image = [_images objectForKey: imageName];
-	  if (image == nil)
-	    {
-	      image = [[_imageClass alloc] initWithContentsOfFile: imagePath];
-	      if (image == nil)
-		{
-		  NSLog(@"GSTheme failed to load %@ from %@",
-		    imageName, imagePath);
-		}
-	      else
-		{
-		  NSImage	*old;
-
-		  /* OK, we have loaded a new image, so we cache it for the
-		   * lifetime of the theme, and we also make a record of
-		   * any previous/default image of the same name.
-		   */
-		  [_images setObject: image forKey: imageName];
-		  RELEASE(image);
-		  old = [NSImage imageNamed: imageName];
-		  if (old == nil)
-		    {
-		      /* This could potentially be a real problem ... if the
-		       * image from the current theme with this name is used
-		       * and the theme is unloaded, what happens when the
-		       * app tries to draw using the proxy to the unloaded
-		       * image?
-		       * To avoid that possibility, we save the new image
-		       * as if it were the old one ... so when the theme is
-		       * unloaded the old image persists.
-		       */
-		      [_oldImages setObject: image forKey: imageName];
-		    }
-		  else
-		    {
-		      /* Store the actual image, not the proxy.
-		       */
-		      old = [(id)old _resource];
-		      [_oldImages setObject: old forKey: imageName];
-		    }
-		}
-	    }
-
-	  /* We try to ensure that our new image can be found by name.
-	   */
-	  if (image != nil && [[image name] isEqualToString: imageName] == NO)
-	    {
-	      [NSImage _setImage: image name: imageName];
-	    }
+	  [_imageNames addObject: imageName];
+	  [NSImage _setImagePath: imagePath name: imageName];
 	}
     }
 
@@ -545,7 +506,7 @@ typedef	struct {
 		      forName: @"GSThemeDomain"];
       if ([searchList containsObject: @"GSThemeDomain"] == NO)
 	{
-	  unsigned	index;
+	  NSUInteger index;
 
 	  /*
 	   * Higher priority than GSConfigDomain and NSRegistrationDomain,
@@ -709,7 +670,7 @@ typedef	struct {
 	    }
 	  if (_extraColors[elementState] == nil)
 	    {
-	      _extraColors[elementState] = [null retain];
+	      _extraColors[elementState] = (id)[null retain];
 	    }
 	}
       if (_extraColors[elementState] != (id)null)
@@ -747,8 +708,7 @@ typedef	struct {
 - (void) deactivate
 {
   NSEnumerator	*enumerator;
-  NSImage	*image;
-  NSString	*name;
+  NSString	*imageName;
 
   NSDebugMLLog(@"GSTheme", @"%@ %p", [self name], self);
 
@@ -774,27 +734,12 @@ typedef	struct {
 
   /* Unload all images created by this theme.
    */
-  enumerator = [_images objectEnumerator];
-  while ((image = [enumerator nextObject]) != nil)
+  enumerator = [_imageNames objectEnumerator];
+  while ((imageName = [enumerator nextObject]) != nil)
     {
-      [image setName: nil];
+      [NSImage _setImagePath: nil name: imageName];
     }
-  [_images removeAllObjects];
-
-  /* Restore old images in NSImage's lookup dictionary so that the app
-   * still has images to draw.
-   * The remove all cached bundle images from both NSImage's name dictionary
-   * and our cache dictionary, so that we can be sure we reload afresh
-   * when re-activated (in case the images on disk changed ... eg by a
-   * theme editor modifying the theme).
-   */
-  enumerator = [_oldImages keyEnumerator];
-  while ((name = [enumerator nextObject]) != nil)
-    {
-      image = [_oldImages objectForKey: name];
-      [NSImage _setImage: image name: name];
-    }
-  [_oldImages removeAllObjects];
+  [_imageNames removeAllObjects];
 
   [self _revokeOwnerships];
 
@@ -819,8 +764,7 @@ typedef	struct {
 	}
       RELEASE(_bundle);
       RELEASE(_colors);
-      RELEASE(_images);
-      RELEASE(_oldImages);
+      RELEASE(_imageNames);
       RELEASE(_icon);
       [self _revokeOwnerships];
       RELEASE(_overrides);
@@ -889,8 +833,7 @@ typedef	struct {
   _reserved = NSZoneCalloc ([self zone], 1, sizeof(internal));
 
   ASSIGN(_bundle, bundle);
-  _images = [NSMutableDictionary new];
-  _oldImages = [NSMutableDictionary new];
+  _imageNames = [NSMutableSet new];
   for (state = 0; state <= GSThemeSelectedState; state++)
     {
       _tiles[state] = [NSMutableDictionary new];
