@@ -153,9 +153,25 @@
 	    break;
 	  case NSCircularBezelStyle:
 	    frame = NSInsetRect(frame, 3, 3);
-	  case NSHelpButtonBezelStyle:
 	    [self drawCircularBezel: frame withColor: color]; 
 	    break;
+	  case NSHelpButtonBezelStyle:
+	    [self drawCircularBezel: frame withColor: color];
+	    {
+	      NSDictionary *attributes = [NSDictionary dictionaryWithObject: [NSFont controlContentFontOfSize: 0]
+								     forKey: NSFontAttributeName];
+	      NSAttributedString *questionMark = [[[NSAttributedString alloc]
+						    initWithString: _(@"?")
+							attributes: attributes] autorelease];
+
+	      NSRect textRect;
+	      textRect.size = [questionMark size];
+	      textRect.origin.x = NSMidX(frame) - (textRect.size.width / 2);
+	      textRect.origin.y = NSMidY(frame) - (textRect.size.height / 2);
+
+	      [questionMark drawInRect: textRect];
+	    }
+	    break;
 	  case NSDisclosureBezelStyle:
 	  case NSRoundedDisclosureBezelStyle:
 	  case NSRecessedBezelStyle:
@@ -189,12 +205,13 @@
     }
 }
 
-- (NSSize) buttonBorderForCell: (NSCell*)cell
-			 style: (int)style 
-			 state: (GSThemeControlState)state
+- (GSThemeMargins) buttonMarginsForCell: (NSCell*)cell
+				  style: (int)style 
+				  state: (GSThemeControlState)state
 {
   GSDrawTiles	*tiles = nil;
   NSString	*name = [self nameForElement: cell];
+  GSThemeMargins margins;
 
   if (name == nil)
     {
@@ -214,36 +231,48 @@
 	  case NSRoundRectBezelStyle:
 	  case NSTexturedRoundedBezelStyle:
 	  case NSRoundedBezelStyle:
-	    return NSMakeSize(5, 5);
+	    margins.left = 5; margins.top = 5; margins.right = 5; margins.bottom = 5;
+	    return margins;
 	  case NSTexturedSquareBezelStyle:
-	    return NSMakeSize(3, 3);
+	    margins.left = 3; margins.top = 3; margins.right = 3; margins.bottom = 3;
+	    return margins;
 	  case NSSmallSquareBezelStyle:
 	  case NSRegularSquareBezelStyle:
 	  case NSShadowlessSquareBezelStyle:
-	    return NSMakeSize(2, 2);
+	    margins.left = 2; margins.top = 2; margins.right = 2; margins.bottom = 2;
+	    return margins;
 	  case NSThickSquareBezelStyle:
-	    return NSMakeSize(3, 3);
+	    margins.left = 3; margins.top = 3; margins.right = 3; margins.bottom = 3;
+	    return margins;
 	  case NSThickerSquareBezelStyle:
-	    return NSMakeSize(4, 4);
+	    margins.left = 4; margins.top = 4; margins.right = 4; margins.bottom = 4;
+	    return margins;
 	  case NSCircularBezelStyle:
-	    return NSMakeSize(5, 5);
+	    margins.left = 5; margins.top = 5; margins.right = 5; margins.bottom = 5;
+	    return margins;
 	  case NSHelpButtonBezelStyle:
-	    return NSMakeSize(2, 2);
+	    margins.left = 2; margins.top = 2; margins.right = 2; margins.bottom = 2;
+	    return margins;
 	  case NSDisclosureBezelStyle:
 	  case NSRoundedDisclosureBezelStyle:
 	  case NSRecessedBezelStyle:
 	    // FIXME
-	    return NSMakeSize(0, 0);
+	    margins.left = 0; margins.top = 0; margins.right = 0; margins.bottom = 0;
+	    return margins;
 	  default:
-	    return NSMakeSize(3, 3);
+	    margins.left = 2; margins.top = 2; margins.right = 3; margins.bottom = 3;
+	    return margins;
 	}
     }
   else
     {
-      // FIXME: We assume the button's top and right padding are the same as
-      // its bottom and left.
-      return NSMakeSize(tiles->contentRect.origin.x,
-                        tiles->contentRect.origin.y);
+      // FIXME: Move this code to a method in GSDrawTiles?
+      // FIXME: Not correct, need to get the content area of the draw tiles
+      margins.left = tiles->rects[TileCL].size.width;
+      margins.top = tiles->rects[TileTM].size.height;
+      margins.right = tiles->rects[TileCR].size.width;
+      margins.bottom = tiles->rects[TileBM].size.height;
+      return margins;
     }
 }
 
@@ -774,23 +803,73 @@
     }
 }
 
-- (void) drawImage: (NSImage *)image
-      inButtonCell: (NSButtonCell *) cell 
-	 withFrame: (NSRect) aRect
-	  position: (NSPoint) position
+- (NSColor *) menuBackgroundColor
 {
-  BOOL enabled = [cell isEnabled];
-  BOOL dims = [cell imageDimsWhenDisabled];
+  NSColor *color = [self colorNamed: @"menuBackgroundColor"
+                              state: GSThemeNormalState];
+  if (color == nil)
+    {
+      color = [NSColor windowBackgroundColor];
+    }
+  return color;
+}
 
-  if (!enabled && dims)
+- (NSColor *) menuItemBackgroundColor
+{
+  NSColor *color = [self colorNamed: @"menuItemBackgroundColor"
+                              state: GSThemeNormalState];
+  if (color == nil)
     {
-      [image dissolveToPoint: position fraction: 0.5];
+      color = [NSColor controlBackgroundColor];
     }
-  else
+  return color;
+}
+
+- (NSColor *) menuBorderColor
+{
+  NSColor *color = [self colorNamed: @"menuBorderColor"
+                              state: GSThemeNormalState];
+  if (color == nil)
     {
-      [image compositeToPoint: position 
-	            operation: NSCompositeSourceOver];
+      color = [NSColor darkGrayColor];
     }
+  return color;
+}
+
+- (NSColor *) menuBarBackgroundColor
+{
+  NSColor *color = [self colorNamed: @"menuBarBackgroundColor"
+                              state: GSThemeNormalState];
+  if (color == nil)
+    {
+      color = [self menuBackgroundColor];
+    }
+  return color;
+}
+
+- (NSColor *) menuBarBorderColor
+{
+  NSColor *color = [self colorNamed: @"menuBarBorderColor"
+                              state: GSThemeNormalState];
+  if (color == nil)
+    {
+      color = [self menuBorderColor];
+    }
+  return color;
+}
+
+- (NSColor *) menuBorderColorForEdge: (NSRectEdge)edge isHorizontal: (BOOL)horizontal
+{
+  if (horizontal && edge == NSMinYEdge)
+    {
+      return [self menuBorderColor];
+    }
+  else if (edge == NSMinXEdge || edge == NSMaxYEdge)
+    {
+      // Draw the dark gray upper left lines.
+      return [self menuBorderColor];
+    }
+  return nil;
 }
 
 - (void) drawBackgroundForMenuView: (NSMenuView*)menuView
@@ -804,25 +883,15 @@
  
   if (tiles == nil)
     {
-     NSRectEdge sides[2]; 
-     float      grays[] = {NSDarkGray, NSDarkGray};
+      NSRectEdge sides[4] = { NSMinXEdge, NSMaxYEdge, NSMaxXEdge, NSMinYEdge }; 
+      NSColor *colors[] = {[self menuBorderColorForEdge: NSMinXEdge isHorizontal: horizontal], 
+                           [self menuBorderColorForEdge: NSMaxYEdge isHorizontal: horizontal], 
+                           [self menuBorderColorForEdge: NSMaxXEdge isHorizontal: horizontal],
+                           [self menuBorderColorForEdge: NSMinYEdge isHorizontal: horizontal]};
 
-     [[NSColor windowBackgroundColor] set];
-     NSRectFill(NSIntersectionRect(bounds, dirtyRect));
-
-     if (horizontal == YES)
-        {
-          sides[0] = NSMinYEdge;
-          sides[1] = NSMinYEdge;
-          NSDrawTiledRects(bounds, dirtyRect, sides, grays, 2);
-        }
-      else
-        {
-          sides[0] = NSMinXEdge;
-          sides[1] = NSMaxYEdge;
-          // Draw the dark gray upper left lines.
-          NSDrawTiledRects(bounds, dirtyRect, sides, grays, 2);
-        }
+      [[self menuBackgroundColor] set];
+      NSRectFill(NSIntersectionRect(bounds, dirtyRect));
+      NSDrawColorTiledRects(bounds, dirtyRect, sides, colors, 4);
     }
   else
     {
@@ -832,6 +901,13 @@
     }
 }
 
+- (BOOL) drawsBorderForMenuItemCell: (NSMenuItemCell *)cell 
+                              state: (GSThemeControlState)state
+                       isHorizontal: (BOOL)horizontal
+{
+  return [cell isBordered];
+}
+
 - (void) drawBorderAndBackgroundForMenuItemCell: (NSMenuItemCell *)cell
                                       withFrame: (NSRect)cellFrame
                                          inView: (NSView *)controlView
@@ -844,7 +920,6 @@
  
   if (tiles == nil)
     {
- 
       NSColor	*backgroundColor = [cell backgroundColor];
 
       if (isHorizontal)
@@ -859,8 +934,12 @@
       [backgroundColor set];
       NSRectFill(cellFrame);
 
-      if (![cell isBordered])
-	return;
+      if (![self drawsBorderForMenuItemCell: cell 
+                                      state: state 
+                               isHorizontal: isHorizontal])
+        {
+          return;
+        }
 
       if (state == GSThemeSelectedState)
 	{
@@ -879,6 +958,68 @@
     }
 }
 
+- (NSColor *) menuSeparatorColor
+{
+  NSColor *color = [self colorNamed: @"menuSeparatorColor"
+                              state: GSThemeNormalState];
+  NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
+
+  // TODO: Remove the style check... Windows theming should be in a subclass 
+  // probably
+  if (color == nil && style == NSWindows95InterfaceStyle)
+    {
+      color = [NSColor blackColor];
+    }
+  return color;
+}
+
+- (CGFloat) menuSeparatorInset
+{
+  return 3.0;
+}
+
+- (void) drawSeparatorItemForMenuItemCell: (NSMenuItemCell *)cell
+                                withFrame: (NSRect)cellFrame
+                                   inView: (NSView *)controlView
+                             isHorizontal: (BOOL)isHorizontal
+{
+  GSDrawTiles *tiles = [self tilesNamed: GSMenuSeparatorItem state: GSThemeNormalState];
+ 
+  if (tiles == nil)
+    {
+      NSBezierPath *path = [NSBezierPath bezierPath];
+      CGFloat inset = [self menuSeparatorInset];
+      NSPoint start = NSMakePoint(inset, cellFrame.size.height / 2 + 
+				         cellFrame.origin.y + 0.5);
+      NSPoint end = NSMakePoint(cellFrame.size.width - inset, 
+	                        cellFrame.size.height / 2 + cellFrame.origin.y + 0.5);
+
+      [[self menuSeparatorColor] set];
+
+      [path setLineWidth: 0.0];
+      [path moveToPoint: start];
+      [path lineToPoint: end];
+
+      [path stroke];
+    }
+  else
+    {
+      [self fillRect: cellFrame
+           withTiles: tiles
+          background: [NSColor clearColor]];
+    }
+}
+
+- (void) drawTitleForMenuItemCell: (NSMenuItemCell *)cell
+                        withFrame: (NSRect)cellFrame
+                           inView: (NSView *)controlView
+                            state: (GSThemeControlState)state
+                     isHorizontal: (BOOL)isHorizontal
+{
+  [cell _drawText: [[cell menuItem] title]
+          inFrame: [cell titleRectForBounds: cellFrame]];
+}
+
 - (Class) titleViewClassForMenuView: (NSMenuView *)aMenuView
 {
   return [GSTitleView class];
@@ -2357,9 +2498,9 @@ static NSDictionary *titleTextAttributes[3] = {nil, nil, nil};
 
   if (selectingColumns == NO)
     {
-      int selectedRowsCount;
-      int row;
-      int startingRow, endingRow;
+      NSInteger selectedRowsCount;
+      NSUInteger row;
+      NSInteger startingRow, endingRow;
 
       selectedRowsCount = [selectedRows count];      
       if (selectedRowsCount == 0)
@@ -2401,9 +2542,9 @@ static NSDictionary *titleTextAttributes[3] = {nil, nil, nil};
     }
   else // Selecting columns
     {
-      unsigned int selectedColumnsCount;
-      unsigned int column;
-      int startingColumn, endingColumn;
+      NSUInteger selectedColumnsCount;
+      NSUInteger column;
+      NSInteger startingColumn, endingColumn;
       
       selectedColumnsCount = [selectedColumns count];
       
