@@ -37,6 +37,7 @@
 #import "GNUstepGUI/GSWindowDecorationView.h"
 
 #import "NSToolbarFrameworkPrivate.h"
+#import "GSGuiPrivate.h"
 
 @interface NSWindow (Private)
 - (GSWindowDecorationView *) windowView;
@@ -60,46 +61,29 @@
        forWindow: (NSWindow *)window
 {
   GSWindowDecorationView *wv = [window windowView];
-  if ([window menu] == nil && menu != nil)
-    {
-      // NSData *data = [NSArchiver archivedDataWithRootObject: menu]; // copy the view...
-      NSMenu *newMenu = [menu copy]; // [NSUnarchiver unarchiveObjectWithData: data];
-      NSMenuView *menuView = nil;
 
-      /* 
-       * Set the new menu
-       */
-      [window _setMenu: newMenu];
+  // protect against stupid calls from updateAllWindowsWithMenu:
+  if ([window menu] == menu)
+    return;
 
-      /*
-       * And transfer the new menu representation to the window decoration view.
-       */
-      menuView = [newMenu menuRepresentation];
-      if (menuView != nil)
-	{
-	  [menu close];
-	  [menuView setHorizontal: YES];
-	  [wv addMenuView: menuView];
-	  [menuView sizeToFit];
-	}
-    }
-  else
-    {
-      NSMenu *m = [window menu];
-      // NSMenuView *menuView = nil;
+  // Prevent recursion
+  [window _setMenu: menu];
 
-      [m update];      
-      /*
-      menuView = [m menuRepresentation];
-      if (menuView != nil)
-	{
-	  [menu close];
-	  [menuView setHorizontal: YES];
-	  [wv addMenuView: menuView];
-	  [menuView sizeToFit];
-	}
-      */
-    }
+  // Remove any possible old menu view
+  [wv removeMenuView];
+
+  //NSLog(@"Adding menu %@ to window %@", menu, window);
+  if (menu != nil)
+  {
+    NSMenuView *menuView = [[NSMenuView alloc] initWithFrame: NSZeroRect];
+
+    [menuView setMenu: menu];
+    [menuView setHorizontal: YES];
+    [menuView setInterfaceStyle: NSWindows95InterfaceStyle];
+    [wv addMenuView: menuView];
+    [menuView sizeToFit];
+    RELEASE(menuView);
+  } 
 }
 
 - (void) rightMouseDisplay: (NSMenu *)menu
