@@ -25,9 +25,8 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSObject.h>
-#include "GNUstepGUI/GSTrackingRect.h"
-
+#import <Foundation/NSObject.h>
+#import "GNUstepGUI/GSTrackingRect.h"
 
 @class	NSTimer;
 @class	NSView;
@@ -56,6 +55,10 @@
 - (NSToolTipTag) addToolTipRect: (NSRect)aRect
                           owner: (id)anObject
                        userData: (void *)data;
+/**
+ * Removes all of the tool tips in a given rectangle.
+ */
+- (void)removeToolTipsInRect: (NSRect)aRect;
 
 /** Return the number of tooltip rectangles active.
  */
