@@ -31,14 +31,37 @@
 #import "AppKit/NSColor.h"
 #import "AppKit/NSGraphics.h"
 #import "AppKit/NSMenuView.h"
+#import "AppKit/NSScreen.h"
 #import "AppKit/NSWindow.h"
 #import "GNUstepGUI/GSDisplayServer.h"
 #import "GNUstepGUI/GSTheme.h"
 
 #import "NSToolbarFrameworkPrivate.h"
 
+@interface NSWindow (GSWindowDecorationView)
+- (void)_clearContentView;
+@end
+
+@implementation NSWindow (GSWindowDecorationView)
+/* This method prevents a recursion when removing the contet view of a window. 
+   The method [NSWindow -setContentView:] would again lead to a call to -willRemoveSubview:.
+*/
+- (void)_clearContentView
+{
+  _contentView = nil;
+}
+@end
+
 @implementation GSWindowDecorationView
 
+static inline NSRect RectWithSizeScaledByFactor(NSRect aRect, CGFloat factor)
+{
+  return NSMakeRect(aRect.origin.x,
+		    aRect.origin.y,
+		    aRect.size.width * factor,
+		    aRect.size.height * factor);
+}
+
 + (id<GSWindowDecorator>) windowDecorator
 {
   if ([GSCurrentServer() handlesWindowDecorations])
@@ -72,7 +95,16 @@
   aRect.size.height -= t + b;
   aRect.origin.x += l;
   aRect.origin.y += b;
-  return aRect;
+ 
+  if (0 == (aStyle & NSUnscaledWindowMask))
+    {
+      // FIXME: This method should probably take a screen parameter
+      // rather than assuming the mainScreen
+      
+      CGFloat factor = [[NSScreen mainScreen] userSpaceScaleFactor]; 
+      aRect = RectWithSizeScaledByFactor(aRect, 1/factor);
+    }
+ return aRect;
 }
 
 + (NSRect) frameRectForContentRect: (NSRect)aRect
@@ -81,10 +113,20 @@
   float t = 0.0, b = 0.0, l = 0.0, r = 0.0;
 
   [self offsets: &l : &r : &t : &b forStyleMask: aStyle];
+  if (0 == (aStyle & NSUnscaledWindowMask))
+    {
+      // FIXME: This method should probably take a screen parameter
+      // rather than assuming the mainScreen
+
+      CGFloat factor = [[NSScreen mainScreen] userSpaceScaleFactor];
+      aRect = RectWithSizeScaledByFactor(aRect, factor);
+    }
+  
   aRect.size.width += l + r;
   aRect.size.height += t + b;
   aRect.origin.x -= l;
   aRect.origin.y -= b;
+
   return aRect;
 }
 
@@ -113,8 +155,9 @@
       window = w;
       // Content rect will be everything apart from the border
       // that is including menu, toolbar and the like.
-      contentRect = [isa contentRectForFrameRect: frame
-                          styleMask: [w styleMask]];
+      contentRect = [object_getClass(self) 
+                        contentRectForFrameRect: frame
+                        styleMask: [w styleMask]];
     }
   return self;
 }
@@ -142,8 +185,9 @@
 - (NSRect) contentRectForFrameRect: (NSRect)aRect
                          styleMask: (unsigned int)aStyle
 {
-  NSRect content = [isa contentRectForFrameRect: aRect
-                          styleMask: aStyle];
+  NSRect content = [object_getClass(self) 
+                       contentRectForFrameRect: aRect
+                       styleMask: aStyle];
   NSToolbar *tb = [_window toolbar];
 
   if ([_window menu] != nil)
@@ -186,8 +230,8 @@
       aRect.size.height += [tv _heightFromLayout];
     }
 
-  return [isa frameRectForContentRect: aRect
-              styleMask: aStyle];
+  return [object_getClass(self) frameRectForContentRect: aRect
+                                              styleMask: aStyle];
 }
 
 /* If the contentView is removed from the window we must make sure the
@@ -201,21 +245,19 @@
  * the old window will automatically disappear (this is how it works
  * on Apple too).
  */
-- (void) removeSubview: (NSView*)aView
+- (void) willRemoveSubview: (NSView*)aView
 {
-  RETAIN(aView);
   /*
    * If the content view is removed (for example, because it was added
    * to another view in another window), we must let the window know.
    * Otherwise, it would keep trying to resize/manage it as if it was
    * its content view, while it actually is now in another window!
    */
-  [super removeSubview: aView];
+  [super willRemoveSubview: aView];
   if (aView == [_window contentView])
     {
-      [_window setContentView: nil];
+      [_window _clearContentView];
     }
-  RELEASE(aView);
 }
 
 - (void) setBackgroundColor: (NSColor *)color
@@ -246,12 +288,13 @@
   NSView *windowContentView = [_window contentView];
 
   frame.origin = NSZeroPoint;
-  contentViewFrame = [isa contentRectForFrameRect: frame
-                          styleMask: [window styleMask]];
+  contentViewFrame = [object_getClass(self)
+                         contentRectForFrameRect: frame
+                         styleMask: [window styleMask]];
 
   if (hasMenu)
     {
-      NSMenuView *menuView;
+      NSMenuView *menuView = nil;
       GSTheme *theme = [GSTheme theme];
       float menuBarHeight = [theme menuHeightForWindow: _window];
       NSRect menuRect = 
@@ -259,8 +302,17 @@
 		   (NSMaxY(contentRect) + 1) - menuBarHeight,  
 		   contentViewFrame.size.width, 
 		   menuBarHeight);
-
-      menuView = [[_window menu] menuRepresentation];
+      NSEnumerator *e = [[self subviews] objectEnumerator];
+      NSView *v;
+      
+      while ((v = [e nextObject]) != nil)
+        {
+          if ([v isKindOfClass: [NSMenuView class]] == YES)
+            {
+              menuView = (NSMenuView *)v;
+              break;
+            }
+        }
       [menuView setFrame: menuRect];
       contentViewFrame.size.height -= menuBarHeight;
     }
@@ -286,6 +338,13 @@
       contentViewFrame.size.height -= newToolbarViewHeight;
     }
 
+  if (0 == ([window styleMask] & NSUnscaledWindowMask))
+    {
+      CGFloat factor = [window userSpaceScaleFactor];
+      NSRect aRect = RectWithSizeScaledByFactor([self frame], 1/factor);
+      [self setBoundsSize: aRect.size];
+    }
+
   if ([windowContentView superview] == self)
     {
       [windowContentView setFrame:contentViewFrame];
@@ -299,8 +358,9 @@
   NSRect windowContentFrame;
 
   contentRect.size.height += difference;
-  windowFrame = [isa frameRectForContentRect: contentRect
-                     styleMask: [window styleMask]];
+  windowFrame = [object_getClass(self)
+                    frameRectForContentRect: contentRect
+                    styleMask: [window styleMask]];
 
   // Set the local frame without changing the contents view
   windowContentFrame = windowFrame;
@@ -330,8 +390,9 @@
   _autoresizes_subviews = NO;
   [super setFrame: frameRect];
 
-  contentRect = [isa contentRectForFrameRect: frameRect
-                      styleMask: [window styleMask]];
+  contentRect = [object_getClass(self)
+                    contentRectForFrameRect: frameRect
+                    styleMask: [window styleMask]];
 
   // Safety Check.
   [cv setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
@@ -497,7 +558,7 @@
 	  [RETAIN(v) removeFromSuperviewWithoutNeedingDisplay];
 	  
 	  [self changeWindowHeight: -(menubarHeight)];  
-	  return AUTORELEASE(v);
+	  return (NSMenuView *)AUTORELEASE(v);
 	}
     }
 
