@@ -2,7 +2,7 @@
 
    <abstract>Xib (Cocoa XML) model loader</abstract>
 
-   Copyright (C) 2010 Free Software Foundation, Inc.
+   Copyright (C) 2010, 2011 Free Software Foundation, Inc.
 
    Written by: Fred Kiefer <FredKiefer@gmx.de>
    Created: March 2010
@@ -48,6 +48,14 @@
 - (void) _setMainMenu: (NSMenu*)aMenu;
 @end
 
+@interface NSCustomObject (NibCompatibility)
+- (void) setRealObject: (id)obj;
+@end
+
+@interface NSNibConnector (NibCompatibility)
+- (id) nibInstantiate;
+@end
+
 @implementation FirstResponder
 
 + (id) allocWithZone: (NSZone*)zone
@@ -207,6 +215,44 @@
   [super dealloc];
 }
 
+- (NSString*) label
+{
+  return label;
+}
+
+- (id) source
+{
+  return source;
+}
+
+- (id) destination
+{
+  return destination;
+}
+
+- (NSNibConnector*) nibConnector
+{
+  NSString *tag = [self label];
+  NSRange colonRange = [tag rangeOfString: @":"];
+  NSUInteger location = colonRange.location;
+  NSNibConnector *result = nil;
+
+  if (location == NSNotFound)
+    {
+      result = [[NSNibOutletConnector alloc] init];
+    }
+  else
+    {
+      result = [[NSNibControlConnector alloc] init];
+    }
+
+  [result setDestination: [self destination]];
+  [result setSource: [self source]];
+  [result setLabel: [self label]];
+  
+  return result;
+}
+
 - (id) nibInstantiate
 {
   if ([source respondsToSelector: @selector(nibInstantiate)])
@@ -288,6 +334,44 @@
 
 @end
 
+@implementation IBBindingConnection
+
+- (void) dealloc
+{
+  DESTROY(connector);
+  [super dealloc];
+}
+
+- (id) initWithCoder: (NSCoder*)coder
+{
+  self = [super initWithCoder: coder];
+  if (self == nil)
+    return nil;
+
+  if ([coder allowsKeyedCoding])
+    {
+      if ([coder containsValueForKey: @"connector"])
+        {
+          ASSIGN(connector, [coder decodeObjectForKey: @"connector"]);
+        }
+    }
+
+  return self;
+}
+
+- (id) nibInstantiate
+{
+  [connector nibInstantiate];
+  return [super nibInstantiate];
+}
+
+- (void) establishConnection
+{
+  [connector establishConnection];
+}
+
+@end
+
 @implementation IBConnectionRecord
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -318,6 +402,11 @@
   [super dealloc];
 }
 
+- (IBConnection*) connection
+{
+  return connection;
+}
+
 - (id) nibInstantiate
 {
   ASSIGN(connection, [connection nibInstantiate]);
@@ -331,6 +420,87 @@
 
 @end
 
+@implementation IBToolTipAttribute
+
+- (NSString*) toolTip
+{
+  return toolTip;
+}
+
+- (id) initWithCoder: (NSCoder*)coder
+{
+  if ([coder allowsKeyedCoding])
+    {
+      if ([coder containsValueForKey: @"name"])
+        {
+          name = [coder decodeObjectForKey: @"name"];
+        }
+      if ([coder containsValueForKey: @"object"])
+        {
+          ASSIGN(object, [coder decodeObjectForKey: @"object"]);
+        }
+      if ([coder containsValueForKey: @"toolTip"])
+        {
+          ASSIGN(toolTip, [coder decodeObjectForKey: @"toolTip"]);
+        }
+    }
+  else
+    {
+      [NSException raise: NSInvalidArgumentException 
+                   format: @"Can't decode %@ with %@.",NSStringFromClass([self class]),
+                   NSStringFromClass([coder class])];
+    }
+  return self;
+}
+
+- (void) dealloc
+{
+  DESTROY(name);
+  DESTROY(object);
+  DESTROY(toolTip);
+  [super dealloc];
+}
+
+@end
+
+@implementation IBInitialTabViewItemAttribute
+
+- (id) initWithCoder: (NSCoder*)coder
+{
+  if ([coder allowsKeyedCoding])
+    {
+      if ([coder containsValueForKey: @"name"])
+        {
+          name = [coder decodeObjectForKey: @"name"];
+        }
+      if ([coder containsValueForKey: @"object"])
+        {
+          ASSIGN(object, [coder decodeObjectForKey: @"object"]);
+        }
+      if ([coder containsValueForKey: @"initialTabViewItem"])
+        {
+          ASSIGN(initialTabViewItem, [coder decodeObjectForKey: @"initialTabViewItem"]);
+        }
+    }
+  else
+    {
+      [NSException raise: NSInvalidArgumentException 
+                   format: @"Can't decode %@ with %@.",NSStringFromClass([self class]),
+                   NSStringFromClass([coder class])];
+    }
+  return self;
+}
+
+- (void) dealloc
+{
+  DESTROY(name);
+  DESTROY(object);
+  DESTROY(initialTabViewItem);
+  [super dealloc];
+}
+
+@end
+
 @implementation IBObjectRecord
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -376,6 +546,16 @@
   return object;
 }
 
+- (id) parent
+{
+  return parent;
+}
+
+- (NSInteger) objectID
+{
+  return objectID;
+}
+
 @end
 
 @implementation IBMutableOrderedSet
@@ -403,15 +583,32 @@
   [super dealloc];
 }
 
-- (NSArray *)orderedObjects
+- (NSArray*)orderedObjects
 {
   return orderedObjects;
 }
+
+- (id) objectWithObjectID: (NSInteger)objID
+{
+  NSEnumerator *en;
+  IBObjectRecord *obj;
+
+  en = [orderedObjects objectEnumerator];
+  while ((obj = [en nextObject]) != nil)
+    {
+      if ([obj objectID] == objID)
+        {
+          return [obj object];
+        }
+    }
+  return nil;
+}
+
 @end
 
 @implementation IBObjectContainer
 
-- (id) initWithCoder:(NSCoder *)coder
+- (id) initWithCoder: (NSCoder*)coder
 {
   if ([coder allowsKeyedCoding])
     {
@@ -423,6 +620,10 @@
         {
           ASSIGN(objectRecords, [coder decodeObjectForKey: @"objectRecords"]);
         }
+      if ([coder containsValueForKey: @"flattenedProperties"])
+        {
+          ASSIGN(flattenedProperties, [coder decodeObjectForKey: @"flattenedProperties"]);
+        }
       // We could load more data here, but we currently don't need it.
     }
   else
@@ -451,25 +652,103 @@
   [super dealloc];
 }
 
+- (NSEnumerator*) connectionRecordEnumerator
+{
+  return [connectionRecords objectEnumerator];
+}
+
+- (NSEnumerator*) objectRecordEnumerator
+{
+  return [[objectRecords orderedObjects] objectEnumerator];
+}
+
+- (NSDictionary*) propertiesForObjectID: (int)objectID
+{
+  NSEnumerator *en;
+  NSString *idString;
+  NSString *key;
+  NSMutableDictionary *properties;
+  int idLength;
+
+  idString = [NSString stringWithFormat: @"%d.", objectID];
+  idLength = [idString length];
+  properties = [[NSMutableDictionary alloc] init];
+  en = [flattenedProperties keyEnumerator];
+  while ((key = [en nextObject]) != nil)
+    {
+      if ([key hasPrefix: idString])
+        {
+          id value = [flattenedProperties objectForKey: key];
+          [properties setObject: value forKey: [key substringFromIndex: idLength]];
+        }
+    }
+
+  return AUTORELEASE(properties);
+}
+
 - (id) nibInstantiate
 {
   NSEnumerator *en;
   id obj;
 
-  en = [connectionRecords objectEnumerator];
   // iterate over connections, instantiate, and then establish them.
+  en = [connectionRecords objectEnumerator];
   while ((obj = [en nextObject]) != nil)
     {
       [obj nibInstantiate];
       [obj establishConnection];
     }
 
-  return self;
-}
+  // awaken all objects.
+  en = [[objectRecords orderedObjects] objectEnumerator];
+  while ((obj = [en nextObject]) != nil)
+    {
+      id realObj;
+      NSDictionary *properties;
+      id value;
 
-- (NSEnumerator *) objectRecordEnumerator
-{
-  return [[objectRecords orderedObjects] objectEnumerator];
+      realObj = [obj object];
+      if ([realObj respondsToSelector: @selector(nibInstantiate)])
+        {
+          realObj = [realObj nibInstantiate];
+        }
+
+      properties = [self propertiesForObjectID: [obj objectID]];
+      NSDebugLLog(@"XIB", @"object %ld props %@", (long)[obj objectID], properties);
+
+      //value = [properties objectForKey: @"windowTemplate.maxSize"];
+      //value = [properties objectForKey: @"CustomClassName"];
+
+      // Activate windows
+      value = [properties objectForKey: @"NSWindowTemplate.visibleAtLaunch"];
+      if (value != nil)
+        {
+          if ([value boolValue] == YES)
+            {
+              if ([realObj isKindOfClass: [NSWindow class]])
+                {
+                  // bring visible windows to front...
+                  [(NSWindow *)realObj orderFront: self];
+                }
+            }
+        }
+
+      // Tool tips
+      value = [properties objectForKey: @"IBAttributePlaceholdersKey"];
+      if (value != nil)
+        {
+          IBToolTipAttribute *tta = [(NSDictionary*)value objectForKey: @"ToolTip"];
+
+          [realObj setToolTip: [tta toolTip]];
+        }
+
+      if ([realObj respondsToSelector: @selector(awakeFromNib)])
+        {
+          [realObj awakeFromNib];
+        }
+    }
+
+  return self;
 }
 
 @end
@@ -481,7 +760,7 @@
 
 @implementation GSXibLoader
 
-+ (NSString *)type
++ (NSString*) type
 {
   return @"xib";
 }
@@ -491,64 +770,50 @@
   return 4.0;
 }
 
-- (void) awakeData: (NSDictionary*)data withContext: (NSDictionary *)context
+- (void) awake: (NSArray *)rootObjects 
+   inContainer: (IBObjectContainer *)objects 
+   withContext: (NSDictionary *)context
 {
-  NSArray *rootObjects;
   NSEnumerator *en;
   id obj;
-  IBObjectContainer *objects;
   NSMutableArray *topLevelObjects = [context objectForKey: NSNibTopLevelObjects];
-  //id owner = [context objectForKey: NSNibOwner];
+  id owner = [context objectForKey: NSNibOwner];
 
-  objects = [data objectForKey: @"IBDocument.Objects"];
-  [objects nibInstantiate];
-  
-  // FIXME: Use the owner as first root object
-  rootObjects = [data objectForKey: @"IBDocument.RootObjects"];
-  NSDebugLog(@"rootObjects %@", rootObjects);
+  // Use the owner as first root object
+  [(NSCustomObject*)[rootObjects objectAtIndex: 0] setRealObject: owner];
   en = [rootObjects objectEnumerator];
   while ((obj = [en nextObject]) != nil)
     {
       if ([obj respondsToSelector: @selector(nibInstantiate)])
         {
           obj = [obj nibInstantiate];
+        }
+
+      if (obj != nil)
+        {
           [topLevelObjects addObject: obj];
+          // All top level objects must be released by the caller to avoid
+          // leaking, unless they are going to be released by other nib
+          // objects on behalf of the owner.
           RETAIN(obj);
         }
 
-      // instantiate all windows and fill in the top level array.
-      if ([obj isKindOfClass: [NSWindow class]])
-        {
-          // bring visible windows to front...
-          //if ([obj isVisible])
-            {
-              [(NSWindow *)obj orderFront: self];
-            }
-        }
-      else if ([obj isKindOfClass: [NSMenu class]])
+      if ([obj isKindOfClass: [NSMenu class]])
         {
           // add the menu...
           [NSApp _setMainMenu: obj];
         }
     }
-      
-  // awaken all objects.
-  en = [objects objectRecordEnumerator];
-  while ((obj = [en nextObject]) != nil)
-    {
-      obj = [obj object];
-      if ([obj respondsToSelector: @selector(awakeFromNib)])
-        {
-          [obj awakeFromNib];
-        }
-    }
+
+  // Load connections and awaken objects
+  [objects nibInstantiate];
 }
 
 - (BOOL) loadModelData: (NSData *)data
      externalNameTable: (NSDictionary *)context
               withZone: (NSZone *)zone;
 {
-  BOOL		loaded = NO;
+  BOOL loaded = NO;
   NSKeyedUnarchiver *unarchiver = nil;
 
   NS_DURING
@@ -558,26 +823,21 @@
           unarchiver = [[GSXibKeyedUnarchiver alloc] initForReadingWithData: data];
 	  if (unarchiver != nil)
 	    {
-              NSDictionary *root;
-              
-	      NSDebugLog(@"Invoking unarchiver");
-	      [unarchiver setObjectZone: zone];
-              root = [unarchiver decodeObjectForKey: @"root"];
-              if (root != nil)
-                {
-                  [self awakeData: root withContext: context];
-                  loaded = YES;
-                }
-	      else
-		{
-		  NSLog(@"Data not found when loading xib.");
-		}
+              NSArray *rootObjects;
+              IBObjectContainer *objects;
 
+	      NSDebugLLog(@"XIB", @"Invoking unarchiver");
+	      [unarchiver setObjectZone: zone];
+              rootObjects = [unarchiver decodeObjectForKey: @"IBDocument.RootObjects"];
+              objects = [unarchiver decodeObjectForKey: @"IBDocument.Objects"];
+              NSDebugLLog(@"XIB", @"rootObjects %@", rootObjects);
+              [self awake: rootObjects inContainer: objects withContext: context];
+              loaded = YES;
               RELEASE(unarchiver);
 	    }
 	  else
 	    {
-	      NSLog(@"Could not instantiate unarchiver.");
+	      NSLog(@"Could not instantiate Xib unarchiver.");
 	    }
 	}
       else
@@ -600,12 +860,12 @@
   return loaded;
 }
 
-- (NSData *) dataForFile: (NSString *)fileName
+- (NSData*) dataForFile: (NSString*)fileName
 {
   NSFileManager	*mgr = [NSFileManager defaultManager];
   BOOL isDir = NO;
 
-  NSDebugLog(@"Loading Xib `%@'...\n", fileName);
+  NSDebugLLog(@"XIB", @"Loading Xib `%@'...\n", fileName);
   if ([mgr fileExistsAtPath: fileName isDirectory: &isDir])
     {
       if (isDir == NO)
@@ -697,8 +957,8 @@
 - (NSString*) description
 {
   return [NSString stringWithFormat: 
-                     @"GSXibElement <%@> attrs (%@) elements [%@] %@", 
-                   type, attributes, elements, value, nil];
+                     @"GSXibElement <%@> attrs (%@) elements [%@] values [%@] %@", 
+                   type, attributes, elements, values, value, nil];
 }
 
 @end
@@ -742,17 +1002,17 @@
   [super dealloc];
 }
 
-- (void) parser: (NSXMLParser *)parser
-foundCharacters: (NSString *)string
+- (void) parser: (NSXMLParser*)parser
+foundCharacters: (NSString*)string
 {
   [currentElement setValue: string];
 }
 
-- (void) parser: (NSXMLParser *)parser
-didStartElement: (NSString *)elementName
-   namespaceURI: (NSString *)namespaceURI
-  qualifiedName: (NSString *)qualifiedName
-     attributes: (NSDictionary *)attributeDict
+- (void) parser: (NSXMLParser*)parser
+didStartElement: (NSString*)elementName
+   namespaceURI: (NSString*)namespaceURI
+  qualifiedName: (NSString*)qualifiedName
+     attributes: (NSDictionary*)attributeDict
 {
   GSXibElement *element = [[GSXibElement alloc] initWithType: elementName
                                            andAttributes: attributeDict];
@@ -773,28 +1033,27 @@ didStartElement: (NSString *)elementName
       [objects setObject: element forKey: ref];
     }
 
-  if (![@"archive" isEqualToString: elementName])
+  if (![@"archive" isEqualToString: elementName] &&
+      ![@"data" isEqualToString: elementName])
     {
       // only used for the root element
       // push
       [stack addObject: currentElement];
     }
 
-  if ([@"data" isEqualToString: elementName])
+  if (![@"archive" isEqualToString: elementName])
     {
-      // only used for the element below root
-      [currentElement setElement: element forKey: @"root"];
+      currentElement = element;
     }
-
-  currentElement = element;
 }
 
-- (void) parser: (NSXMLParser *)parser
-  didEndElement: (NSString *)elementName
-   namespaceURI: (NSString *)namespaceURI
-  qualifiedName: (NSString *)qName
+- (void) parser: (NSXMLParser*)parser
+  didEndElement: (NSString*)elementName
+   namespaceURI: (NSString*)namespaceURI
+  qualifiedName: (NSString*)qName
 {
-  if (![@"archive" isEqualToString: elementName])
+  if (![@"archive" isEqualToString: elementName] &&
+      ![@"data" isEqualToString: elementName])
     {
       // pop
       currentElement = [stack lastObject];
@@ -802,97 +1061,200 @@ didStartElement: (NSString *)elementName
     }
 }
 
+- (id) allocObjectForClassName: (NSString*)classname
+{
+  Class c = [self classForClassName: classname];
+  id delegate = [self delegate];
+
+  if (c == nil)
+    {
+      c = [[self class] classForClassName: classname];
+      if (c == nil)
+        {
+          c = NSClassFromString(classname);
+          if (c == nil)
+            {
+              c = [delegate unarchiver: self
+                            cannotDecodeObjectOfClassName: classname
+                       originalClasses: nil];
+              if (c == nil)
+                {
+                  [NSException raise: NSInvalidUnarchiveOperationException
+                              format: @"[%@ -%@]: no class for name '%@'",
+                               NSStringFromClass([self class]),
+                               NSStringFromSelector(_cmd),
+                               classname];
+                }
+            }
+        }
+    }
+
+  // Create instance.
+  return [c allocWithZone: [self zone]];
+ }
+
+- (id) decodeObjectForXib: (GSXibElement*)element
+             forClassName: (NSString*)classname
+                   withID: (NSString*)objID
+{
+  GSXibElement *last;
+  id o, r;
+  id delegate = [self delegate];
+
+  // Create instance.
+  o = [self allocObjectForClassName: classname];
+  // Make sure the object stays around, even when replaced.
+  RETAIN(o);
+  if (objID != nil)
+    [decoded setObject: o forKey: objID];
+
+  // push
+  last = currentElement;
+  currentElement = element;
+
+  r = [o initWithCoder: self];
+
+  // pop
+  currentElement = last;
+  
+  if (r != o)
+    {
+      [delegate unarchiver: self
+         willReplaceObject: o
+                withObject: r];
+      ASSIGN(o, r);
+      if (objID != nil)
+        [decoded setObject: o forKey: objID];
+    }
+
+  r = [o awakeAfterUsingCoder: self];
+  if (r != o)
+    {
+      [delegate unarchiver: self
+         willReplaceObject: o
+                withObject: r];
+      ASSIGN(o, r);
+      if (objID != nil)
+        [decoded setObject: o forKey: objID];
+    }
+
+  if (delegate != nil)
+    {
+      r = [delegate unarchiver: self didDecodeObject: o];
+      if (r != o)
+        {
+          [delegate unarchiver: self
+             willReplaceObject: o
+                    withObject: r];
+          ASSIGN(o, r);
+          if (objID != nil)
+            [decoded setObject: o forKey: objID];
+        }
+    }
+
+  // Balance the retain above
+  RELEASE(o);
+  
+  if (objID != nil)
+    {
+      NSDebugLLog(@"XIB", @"decoded object %@ for id %@", o, objID);
+    }
+
+  return AUTORELEASE(o);
+}
+
+/*
+  This method is a copy of decodeObjectForXib:forClassName:withKey:
+  The only difference being in the way we decode the object and the 
+  missing context switch.
+ */
+- (id) decodeDictionaryForXib: (GSXibElement*)element
+                 forClassName: (NSString*)classname
+                       withID: (NSString*)objID
+{
+  id o, r;
+  id delegate = [self delegate];
+
+  // Create instance.
+  o = [self allocObjectForClassName: classname];
+  // Make sure the object stays around, even when replaced.
+  RETAIN(o);
+  if (objID != nil)
+    [decoded setObject: o forKey: objID];
+
+  r = [o initWithDictionary: [self _decodeDictionaryOfObjectsForElement: element]];
+  if (r != o)
+    {
+      [delegate unarchiver: self
+         willReplaceObject: o
+                withObject: r];
+      ASSIGN(o, r);
+      if (objID != nil)
+        [decoded setObject: o forKey: objID];
+    }
+
+  r = [o awakeAfterUsingCoder: self];
+  if (r != o)
+    {
+      [delegate unarchiver: self
+         willReplaceObject: o
+                withObject: r];
+      ASSIGN(o, r);
+      if (objID != nil)
+        [decoded setObject: o forKey: objID];
+    }
+
+  if (delegate != nil)
+    {
+      r = [delegate unarchiver: self didDecodeObject: o];
+      if (r != o)
+        {
+          [delegate unarchiver: self
+             willReplaceObject: o
+                    withObject: r];
+          ASSIGN(o, r);
+          if (objID != nil)
+            [decoded setObject: o forKey: objID];
+        }
+    }
+  // Balance the retain above
+  RELEASE(o);
+  
+  if (objID != nil)
+    {
+      NSDebugLLog(@"XIB", @"decoded object %@ for id %@", o, objID);
+    }
+
+  return AUTORELEASE(o);
+}
+
 - (id) objectForXib: (GSXibElement*)element
 {
   NSString *elementName;
-  NSString *key = [element attributeForKey: @"id"];
+  NSString *objID;
 
   if (element == nil)
     return nil;
 
+  NSDebugLLog(@"XIB", @"decoding element %@", element);
+  objID = [element attributeForKey: @"id"];
+  if (objID)
+    {
+      id new = [decoded objectForKey: objID];
+      if (new != nil)
+        {
+          // The object was already decoded as a reference
+          return new;
+        }
+    }
+
   elementName = [element type];
   if ([@"object" isEqualToString: elementName])
     {
-      GSXibElement *last;
       NSString *classname = [element attributeForKey: @"class"];
-      Class c = [self classForClassName: classname];
-      id o, r;
-      id delegate = [self delegate];
-
-      if (c == nil)
-	{
-	  c = [[self class] classForClassName: classname];
-	  if (c == nil)
-	    {
-	      c = NSClassFromString(classname);
-	      if (c == nil)
-		{
-		  c = [delegate unarchiver: self
-                                 cannotDecodeObjectOfClassName: classname
-                            originalClasses: nil];
-		  if (c == nil)
-		    {
-		      [NSException raise:
-			NSInvalidUnarchiveOperationException
-			format: @"[%@ +%@]: no class for name '%@'",
-			NSStringFromClass([self class]),
-			NSStringFromSelector(_cmd),
-			classname];
-		    }
-		}
-	    }
-	}
-
-      // push
-      last = currentElement;
-      currentElement = element;
-
-      // Create instance.
-      o = [c allocWithZone: [self zone]];
-      // Make sure the object stays around, even when replaced.
-      RETAIN(o);
-      if (key != nil)
-        [decoded setObject: o forKey: key];
-      r = [o initWithCoder: self];
-      if (r != o)
-	{
-	  [delegate unarchiver: self
-	      willReplaceObject: o
-		     withObject: r];
-	  ASSIGN(o, r);
-          if (key != nil)
-            [decoded setObject: o forKey: key];
-	}
-      r = [o awakeAfterUsingCoder: self];
-      if (r != o)
-	{
-	  [delegate unarchiver: self
-	      willReplaceObject: o
-		     withObject: r];
-	  ASSIGN(o, r);
-          if (key != nil)
-            [decoded setObject: o forKey: key];
-	}
-      if (delegate != nil)
-	{
-	  r = [delegate unarchiver: self didDecodeObject: o];
-	  if (r != o)
-	    {
-	      [delegate unarchiver: self
-		  willReplaceObject: o
-			 withObject: r];
-              ASSIGN(o, r);
-              if (key != nil)
-                [decoded setObject: o forKey: key];
-	    }
-	}
-      // Balance the retain above
-      RELEASE(o);
-
-      // pop
-      currentElement = last;
-
-      return AUTORELEASE(o);
+      return [self decodeObjectForXib: element
+                         forClassName: classname
+                               withID: objID];
     }
   else if ([@"string" isEqualToString: elementName])
     {
@@ -907,8 +1269,12 @@ didStartElement: (NSString *)elementName
                                                   encoding: NSUTF8StringEncoding]);
         }
 
-      if (key != nil)
-        [decoded setObject: new forKey: key];
+      // empty strings are not nil!
+      if (new == nil)
+        new = @"";
+
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
       
       return new;
     }
@@ -916,8 +1282,8 @@ didStartElement: (NSString *)elementName
     {
       id new = [NSNumber numberWithInt: [[element value] intValue]];
 
-      if (key != nil)
-        [decoded setObject: new forKey: key];
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
       
       return new;
     }
@@ -925,8 +1291,8 @@ didStartElement: (NSString *)elementName
     {
       id new = [NSNumber numberWithDouble: [[element value] doubleValue]];
 
-      if (key != nil)
-        [decoded setObject: new forKey: key];
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
       
       return new;
     }
@@ -934,11 +1300,40 @@ didStartElement: (NSString *)elementName
     {
       id new = [NSNumber numberWithBool: [[element value] boolValue]];
 
-      if (key != nil)
-        [decoded setObject: new forKey: key];
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
       
       return new;
+    }
+  else if ([@"integer" isEqualToString: elementName])
+    {
+      NSString *value = [element attributeForKey: @"value"];
+      id new = [NSNumber numberWithInteger: [value integerValue]];
 
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
+      
+      return new;
+    }
+  else if ([@"real" isEqualToString: elementName])
+    {
+      NSString *value = [element attributeForKey: @"value"];
+      id new = [NSNumber numberWithFloat: [value floatValue]];
+
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
+      
+      return new;
+    }
+  else if ([@"boolean" isEqualToString: elementName])
+    {
+      NSString *value = [element attributeForKey: @"value"];
+      id new = [NSNumber numberWithBool: [value boolValue]];
+
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
+      
+      return new;
     }
   else if ([@"reference" isEqualToString: elementName])
     {
@@ -952,16 +1347,16 @@ didStartElement: (NSString *)elementName
         {
           id new = [decoded objectForKey: ref];
 
+          // FIXME: We need a marker for nil
           if (new == nil)
             {
-              // push
-              GSXibElement *last = currentElement;
-
+              //NSLog(@"Decoding reference %@", ref);
               element = [objects objectForKey: ref];
-              currentElement = element;
-
-              // pop
-              currentElement = last;
+              if (element != nil)
+                {
+                  // Decode the real object
+                  new = [self objectForXib: element];
+                }
             }
 
           return new;
@@ -975,17 +1370,8 @@ didStartElement: (NSString *)elementName
     {
       id new = [element value];
 
-      if (key != nil)
-        [decoded setObject: new forKey: key];
-      
-      return new;
-    }
-  else if ([@"integer" isEqualToString: elementName])
-    {
-      id new = [NSNumber numberWithInteger: [[element value] integerValue]];
-
-      if (key != nil)
-        [decoded setObject: new forKey: key];
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
       
       return new;
     }
@@ -995,36 +1381,35 @@ didStartElement: (NSString *)elementName
                            allowLossyConversion: NO];
       new = [GSMimeDocument decodeBase64: new];
 
-      if (key != nil)
-        [decoded setObject: new forKey: key];
+      if (objID != nil)
+        [decoded setObject: new forKey: objID];
       
       return new;
     }
-  else if ([@"data" isEqualToString: elementName])
+  else if ([@"array" isEqualToString: elementName])
     {
-      NSDictionary *elements = [element elements];
-      NSEnumerator *keyEnumerator = [elements keyEnumerator];
-      NSString *nextKey;
-      GSXibElement *nextXib;
-      id nextObject;
-      NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity: 
-                                                            [elements count]];
+      NSString *classname = [element attributeForKey: @"class"];
 
-      while ((nextKey = (NSString*)[keyEnumerator nextObject]) != nil)
+      if (classname == nil)
         {
-          nextXib = (GSXibElement*)[elements objectForKey: nextKey];
-          nextObject = [self objectForXib: nextXib];
-          if (nextObject != nil)
-            {
-              [dict setObject: nextObject forKey: nextKey];
-            }
-          else
-            {
-              NSLog(@"nil object in data dictionary for key %@", nextKey);
-            }
+          classname = @"NSArray";
+        }
+      return [self decodeObjectForXib: element
+                         forClassName: classname
+                               withID: objID];
+    }
+  else if ([@"dictionary" isEqualToString: elementName])
+    {
+      NSString *classname = [element attributeForKey: @"class"];
+
+      if (classname == nil)
+        {
+          classname = @"NSDictionary";
         }
 
-      return dict;
+      return [self decodeDictionaryForXib: element
+                             forClassName: classname
+                                   withID: objID];
     }
   else
     {
@@ -1036,7 +1421,14 @@ didStartElement: (NSString *)elementName
 
 - (id) _decodeArrayOfObjectsForKey: (NSString*)aKey
 {
-  NSArray *values = [currentElement values];
+  // FIXME: This is wrong but the only way to keep the code for
+  // [NSArray-initWithCoder:] working
+  return [self _decodeArrayOfObjectsForElement: currentElement];
+}
+
+- (id) _decodeArrayOfObjectsForElement: (GSXibElement*)element
+{
+  NSArray *values = [element values];
   int max = [values count];
   id list[max];
   int i;
@@ -1051,6 +1443,27 @@ didStartElement: (NSString *)elementName
   return [NSArray arrayWithObjects: list count: max];
 }
 
+- (id) _decodeDictionaryOfObjectsForElement: (GSXibElement*)element
+{
+  NSDictionary *elements = [element elements];
+  NSEnumerator *en;
+  NSString *key;
+  NSMutableDictionary *dict;
+
+  dict = [[NSMutableDictionary alloc] init];
+  en = [elements keyEnumerator];
+  while ((key = [en nextObject]) != nil)
+    {
+      id obj = [self objectForXib: [elements objectForKey: key]];
+      if (obj == nil)
+        NSLog(@"No object for %@ at key %@", [elements objectForKey: key], key);
+      else
+        [dict setObject: obj forKey: key];
+    }
+
+  return AUTORELEASE(dict);
+}
+
 - (BOOL) containsValueForKey: (NSString*)aKey
 {
   GSXibElement *element = [currentElement elementForKey: aKey];
