@@ -26,10 +26,11 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include "config.h"
-#include <Foundation/NSCoder.h>
-#include "AppKit/NSActionCell.h"
-#include "AppKit/NSControl.h"
+#import "config.h"
+#import <Foundation/NSCoder.h>
+
+#import "AppKit/NSActionCell.h"
+#import "AppKit/NSControl.h"
 
 @implementation NSActionCell
 
@@ -145,7 +146,7 @@ static Class controlClass;
 /**
  * Retrieve the value of the receiver
  */
-- (id)objectValue
+- (id) objectValue
 {
   if (_cell.in_editing && _control_view)
     if ([_control_view isKindOfClass: controlClass])
@@ -156,7 +157,7 @@ static Class controlClass;
 /**
  * Retrieve the value of the receiver as an NSAttributedString.
  */
-- (NSAttributedString*)attributedStringValue
+- (NSAttributedString*) attributedStringValue
 {
   if (_cell.in_editing && _control_view)
     if ([_control_view isKindOfClass: controlClass])
@@ -167,7 +168,7 @@ static Class controlClass;
 /**
  * Retrieve the value of the receiver as an NSString.
  */
-- (NSString *)stringValue
+- (NSString *) stringValue
 {
   if (_cell.in_editing && _control_view)
     if ([_control_view isKindOfClass: controlClass])
@@ -178,7 +179,7 @@ static Class controlClass;
 /**
  * Retrieve the value of the receiver as a double.
  */
-- (double)doubleValue
+- (double) doubleValue
 {
   if (_cell.in_editing && _control_view)
     if ([_control_view isKindOfClass: controlClass])
@@ -189,7 +190,7 @@ static Class controlClass;
 /**
  * Retrieve the value of the receiver as a float.
  */
-- (float)floatValue
+- (float) floatValue
 {
   if (_cell.in_editing && _control_view)
     if ([_control_view isKindOfClass: controlClass])
@@ -200,7 +201,7 @@ static Class controlClass;
 /**
  * Retrieve the value of the receiver as an int.
  */
-- (int)intValue
+- (int) intValue
 {
   if (_cell.in_editing && _control_view)
     if ([_control_view isKindOfClass: controlClass])
@@ -208,6 +209,17 @@ static Class controlClass;
   return [super intValue];
 }
 
+/**
+ * Retrieve the value of the receiver as an NSInteger.
+ */
+- (NSInteger) integerValue
+{
+  if (_cell.in_editing && _control_view)
+    if ([_control_view isKindOfClass: controlClass])
+      [(NSControl *)_control_view validateEditing];
+  return [super integerValue];
+}
+
 /**
  * Set the value of the receiver from anObject.
  */
@@ -215,21 +227,45 @@ static Class controlClass;
 {
   [super setObjectValue: anObject];
   if (_control_view)
-    if ([_control_view isKindOfClass: controlClass])
-      [(NSControl *)_control_view updateCell: self];
+    {
+	if ([_control_view isKindOfClass: controlClass])
+	{
+	  if (_cell.in_editing)
+	    {
+	      [self _updateFieldEditor:
+		      [(NSControl *)_control_view currentEditor]];
+	    }
+          else
+            {
+              [(NSControl *)_control_view updateCell: self];
+            }
+	}
+    }
 }
 
 /**
  * Set the value of the receiver from aString.
  */
 // This method is currently needed, as NSCells implementation 
-// does not call setObjectValue:
+// sometimes does not call setObjectValue:
 - (void) setStringValue: (NSString*)aString
 {
   [super setStringValue: aString];
   if (_control_view)
-    if ([_control_view isKindOfClass: controlClass])
-      [(NSControl *)_control_view updateCell: self];
+    {
+      if ([_control_view isKindOfClass: controlClass])
+	{
+	  if (_cell.in_editing)
+	    {
+	      [self _updateFieldEditor:
+		      [(NSControl *)_control_view currentEditor]];
+	    }
+          else
+            {
+              [(NSControl *)_control_view updateCell: self];
+            }
+	}
+    }
 }
 
 /*
@@ -271,7 +307,7 @@ static Class controlClass;
 /**
  * Assigning a Tag. 
  */
-- (void) setTag: (int)anInt
+- (void) setTag: (NSInteger)anInt
 {
   _tag = anInt;
 }
@@ -279,7 +315,7 @@ static Class controlClass;
 /**
  * Return the tag.
  */
-- (int) tag
+- (NSInteger) tag
 {
   return _tag;
 }
@@ -287,7 +323,7 @@ static Class controlClass;
 /**
  * Returns the control view of the receiver.
  */
--(NSView *)controlView
+-(NSView *) controlView
 {
   return _control_view;
 }
@@ -318,11 +354,11 @@ static Class controlClass;
   if ([aCoder allowsKeyedCoding])
     {
       [aCoder encodeInt: [self tag] forKey: @"NSTag"];
-      if([self target] != nil)
+      if ([self target] != nil)
 	{
 	  [aCoder encodeObject: [self target] forKey: @"NSTarget"];
 	}
-      if([self action] != NULL)
+      if ([self action] != NULL)
 	{
 	  [aCoder encodeObject: NSStringFromSelector([self action]) forKey: @"NSAction"];
 	}
@@ -341,6 +377,8 @@ static Class controlClass;
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   self = [super initWithCoder: aDecoder];
+  if (!self)
+    return nil;
 
   if ([aDecoder allowsKeyedCoding])
     {
