@@ -2,7 +2,7 @@
 
    Methods for reading jpeg images
 
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003-2010 Free Software Foundation, Inc.
    
    Written by:  Stefan Kleine Stegemann <stefan@wms-network.de>
    Date: Nov 2003
@@ -26,18 +26,18 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include "config.h"
-#include "NSBitmapImageRep+JPEG.h"
+#import "config.h"
+#import <Foundation/NSData.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSGraphics.h"
+#import "NSBitmapImageRep+JPEG.h"
+#import "GSGuiPrivate.h"
 
 #if HAVE_LIBJPEG
 
-#include <Foundation/NSString.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/NSGraphics.h"
-#include "GSGuiPrivate.h"
-
 #include <jerror.h>
 #if defined(__MINGW32__)
 /* Hack so that INT32 is not redefined in jmorecfg.h. MingW defines this
@@ -52,7 +52,7 @@
 typedef int jpeg_boolean;
 #define boolean jpeg_boolean
 #endif
-#endif
+#endif // __MINGW32__
 #include <jpeglib.h>
 #if defined(__CYGWIN__)
 /* Cygwin uses a patched jpeg */
@@ -185,7 +185,6 @@ static void gs_jpeg_memory_src_create(j_decompress_ptr cinfo, NSData *data)
   gs_jpeg_source_ptr src;
   
   cinfo->src = (struct jpeg_source_mgr *)malloc(sizeof(gs_jpeg_source_mgr));
-  src = (gs_jpeg_source_ptr)cinfo->src;
 
   src = (gs_jpeg_source_ptr) cinfo->src;
   src->parent.init_source = gs_init_source;
@@ -632,36 +631,36 @@ static void gs_jpeg_memory_dest_destroy (j_compress_ptr cinfo)
   jpeg_start_compress (&cinfo, TRUE);
 
   if (isRGB && [self hasAlpha])	// strip alpha channel before encoding
-  {
-    unsigned char * RGB, * pRGB, * pRGBA;
-    unsigned int iRGB, iRGBA;
-    OBJC_MALLOC(RGB, unsigned char, 3*width);
-    while (cinfo.next_scanline < cinfo.image_height)
     {
-      iRGBA = cinfo.next_scanline * row_stride;
-      pRGBA = &imageSource[iRGBA];
-      pRGB = RGB;
-      for (iRGB = 0; iRGB < 3*width; iRGB += 3)
-      {
-        memcpy(pRGB, pRGBA, 3);
-        pRGB +=3;
-        pRGBA +=4;
-      }
-      row_pointer[0] = RGB;
-      jpeg_write_scanlines (&cinfo, row_pointer, 1);
+      unsigned char * RGB, * pRGB, * pRGBA;
+      unsigned int iRGB, iRGBA;
+      RGB = malloc(sizeof(unsigned char)*3*width);
+      while (cinfo.next_scanline < cinfo.image_height)
+	{
+	  iRGBA = cinfo.next_scanline * row_stride;
+	  pRGBA = &imageSource[iRGBA];
+	  pRGB = RGB;
+	  for (iRGB = 0; iRGB < 3*width; iRGB += 3)
+	    {
+	      memcpy(pRGB, pRGBA, 3);
+	      pRGB +=3;
+	      pRGBA +=4;
+	    }
+	  row_pointer[0] = RGB;
+	  jpeg_write_scanlines (&cinfo, row_pointer, 1);
+	}
+      free(RGB);
     }
-    OBJC_FREE(RGB);
-  }
   else	// no alpha channel
-  {
-    while (cinfo.next_scanline < cinfo.image_height)
     {
-      int	index = cinfo.next_scanline * row_stride;
+      while (cinfo.next_scanline < cinfo.image_height)
+	{
+	  int	index = cinfo.next_scanline * row_stride;
 
-      row_pointer[0] = &imageSource[index];
-      jpeg_write_scanlines (&cinfo, row_pointer, 1);
+	  row_pointer[0] = &imageSource[index];
+	  jpeg_write_scanlines (&cinfo, row_pointer, 1);
+	}
     }
-  }
 
   jpeg_finish_compress(&cinfo);
 
