@@ -256,6 +256,33 @@ static void reader_func(png_structp png_struct, png_bytep data,
     //NSLog(@"PNG file gamma: %f", file_gamma);
    } 
 
+  if (png_get_valid(png_struct, png_info, PNG_INFO_pHYs))
+  {
+    png_uint_32 xppm = png_get_x_pixels_per_meter(png_struct, png_info);
+    png_uint_32 yppm = png_get_y_pixels_per_meter(png_struct, png_info);
+
+    if (xppm != 0 && yppm != 0)
+      {
+	const CGFloat pointsPerMeter = 39.3700787 * 72.0;
+	NSSize sizeInPoints = NSMakeSize((width / (CGFloat)xppm) * pointsPerMeter,
+					 (height / (CGFloat)yppm) * pointsPerMeter);
+
+	// HACK: PNG can not represent 72DPI exactly. If the ppm value is near 72DPI,
+	// assume it is exactly 72 DPI. Note that the same problem occurrs at 144DPI...
+	// so don't use PNG for resolution independent graphics.
+	if (xppm  == 2834 || xppm == 2835)
+	  {
+	    sizeInPoints.width = width;
+	  }
+	if (yppm == 2834 || yppm == 2835)
+	  {
+	    sizeInPoints.height = height;
+	  }
+
+	[self setSize: sizeInPoints];
+      }
+  }
+
   png_destroy_read_struct(&png_struct, &png_info, &png_end_info);
 
   return self;
