@@ -180,14 +180,7 @@
 */
 + (id) imageRepWithData: (NSData *)imageData
 {
-  NSArray* array;
-
-  array = [self imageRepsWithData: imageData];
-  if ([array count])
-    {
-      return [array objectAtIndex: 0];
-    }
-  return nil;
+  return AUTORELEASE([[self alloc] initWithData: imageData]);
 }
 
 /**<p>Returns an array containing newly allocated NSBitmapImageRep
@@ -211,7 +204,7 @@
       NSBitmapImageRep *rep;
       NSArray *a;
 
-      rep=[[self alloc] _initBitmapFromPNG: imageData];
+      rep = [[self alloc] _initBitmapFromPNG: imageData];
       if (!rep)
         return [NSArray array];
       a = [NSArray arrayWithObject: rep];
@@ -224,7 +217,7 @@
       NSBitmapImageRep *rep;
       NSArray *a;
 
-      rep=[[self alloc] _initBitmapFromPNM: imageData
+      rep = [[self alloc] _initBitmapFromPNM: imageData
 			      errorMessage: NULL];
       if (!rep)
         return [NSArray array];
@@ -238,7 +231,7 @@
       NSBitmapImageRep *rep;
       NSArray *a;
 
-      rep=[[self alloc] _initBitmapFromJPEG: imageData
+      rep = [[self alloc] _initBitmapFromJPEG: imageData
 			       errorMessage: NULL];
       if (!rep)
         return [NSArray array];
@@ -252,7 +245,7 @@
       NSBitmapImageRep *rep;
       NSArray *a;
 
-      rep=[[self alloc] _initBitmapFromGIF: imageData
+      rep = [[self alloc] _initBitmapFromGIF: imageData
 			      errorMessage: NULL];
       if (!rep)
         return [NSArray array];
@@ -263,15 +256,7 @@
 
   if ([self _bitmapIsICNS: imageData])
     {
-      NSBitmapImageRep *rep;
-      NSArray *a;
-
-      rep=[[self alloc] _initBitmapFromICNS: imageData];
-      if (!rep)
-        return [NSArray array];
-      a = [NSArray arrayWithObject: rep];
-      DESTROY(rep);
-      return a;
+      return [self _imageRepsWithICNSData: imageData];
     }
 
   image = NSTiffOpenDataRead((char *)[imageData bytes], [imageData length]);
@@ -290,7 +275,8 @@
       imageRep = [[self alloc] _initFromTIFFImage: image number: i];
       if (imageRep)
 	{
-	  [array addObject: AUTORELEASE(imageRep)];
+	  [array addObject: imageRep];
+          RELEASE(imageRep);
 	}
     }
   NSTiffClose(image);
@@ -298,11 +284,11 @@
   return array;
 }
 
-/** Loads only the default (first) image from the TIFF image contained in
+/** Loads only the default (first) image from the image contained in
    data. */
 - (id) initWithData: (NSData *)imageData
 {
-  TIFF 	*image;
+  TIFF *image;
 
   if (imageData == nil)
     {
@@ -310,25 +296,24 @@
       return nil;
     }
 
-  if ([isa _bitmapIsPNG: imageData])
+  if ([object_getClass(self) _bitmapIsPNG: imageData])
     return [self _initBitmapFromPNG: imageData];
 
-  if ([isa _bitmapIsPNM: imageData])
+  if ([object_getClass(self) _bitmapIsPNM: imageData])
     return [self _initBitmapFromPNM: imageData
 		       errorMessage: NULL];
 
-  if ([isa _bitmapIsJPEG: imageData])
+  if ([object_getClass(self) _bitmapIsJPEG: imageData])
     return [self _initBitmapFromJPEG: imageData
 			errorMessage: NULL];
 
-  if ([isa _bitmapIsGIF: imageData])
+  if ([object_getClass(self) _bitmapIsGIF: imageData])
     return [self _initBitmapFromGIF: imageData
 		       errorMessage: NULL];
 
-  if ([isa _bitmapIsICNS: imageData])
+  if ([object_getClass(self) _bitmapIsICNS: imageData])
     return [self _initBitmapFromICNS: imageData];
 
-
   image = NSTiffOpenDataRead((char *)[imageData bytes], [imageData length]);
   if (image == NULL)
     {
@@ -337,8 +322,9 @@
       return nil;
     }
 
-  [self _initFromTIFFImage:image number: -1];
+  [self _initFromTIFFImage: image number: -1];
   NSTiffClose(image);
+
   return self;
 }
 
@@ -359,7 +345,7 @@
       return nil;
     }
   _imageData = RETAIN([dict objectForKey: @"Data"]);
-  if (_imageData == nil)
+  if (_imageData == nil || [_imageData length] == 0)
     {
       NSLog(@"NSBitmapImageRep initWithFocusedViewRect: failed");
       RELEASE(self);
@@ -521,7 +507,7 @@
     rowBytes = ceil((float)width * _bitsPerPixel / 8);
   _bytesPerRow = rowBytes;
 
-  _imagePlanes = NSZoneMalloc([self zone], sizeof(unsigned char*) * MAX_PLANES);
+  _imagePlanes = NSAllocateCollectable(sizeof(unsigned char*) * MAX_PLANES, 0);
   if (planes) 
     {
       unsigned int i;
@@ -533,26 +519,27 @@
     }
   else
     {
-      unsigned char* bits;
-      long length;
+      unsigned char *bits;
+      NSUInteger length;
       unsigned int i;
 
       // No image data was given, allocate it.
-      length = (long)((_isPlanar) ? _numColors : 1) * _bytesPerRow * 
+      length = (NSUInteger)((_isPlanar) ? _numColors : 1) * _bytesPerRow * 
 	  _pixelsHigh * sizeof(unsigned char);
+      // Create a mutable data object although we never use it as such
       _imageData = [[NSMutableData alloc] initWithLength: length];
-      bits = [_imageData mutableBytes];
+      bits = (unsigned char *)[_imageData bytes];
       _imagePlanes[0] = bits;
       if (_isPlanar) 
 	{
-	  for (i=1; i < _numColors; i++) 
-	    _imagePlanes[i] = bits + i*_bytesPerRow * _pixelsHigh;
-	  for (i= _numColors; i < MAX_PLANES; i++) 
+	  for (i = 1; i < _numColors; i++) 
+	    _imagePlanes[i] = bits + i * _bytesPerRow * _pixelsHigh;
+	  for (i = _numColors; i < MAX_PLANES; i++) 
 	    _imagePlanes[i] = NULL;
 	}
       else
 	{
-	  for (i= 1; i < MAX_PLANES; i++) 
+	  for (i = 1; i < MAX_PLANES; i++) 
 	    _imagePlanes[i] = NULL;
 	}      
     }
@@ -934,10 +921,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
       || [_colorSpace isEqualToString: NSDeviceRGBColorSpace])
 		{
       unsigned int ir, ig, ib, ia;
-      float fr, fg, fb, fa;
-      float scale;
+      CGFloat fr, fg, fb, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       if (_hasAlpha)
         {
           // This order depends on the bitmap format
@@ -1001,10 +988,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
            || [_colorSpace isEqual: NSCalibratedWhiteColorSpace])
 		{
       unsigned int iw, ia;
-      float fw, fa;
-      float scale;
+      CGFloat fw, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       if (_hasAlpha)
         {
           // FIXME: This order depends on the bitmap format
@@ -1053,10 +1040,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
            || [_colorSpace isEqual: NSCalibratedBlackColorSpace])
     {
       unsigned int ib, ia;
-      float fw, fa;
-      float scale;
+      CGFloat fw, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       if (_hasAlpha)
         {
           // This order depends on the bitmap format
@@ -1102,10 +1089,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
   else if ([_colorSpace isEqual: NSDeviceCMYKColorSpace])
     {
       unsigned int ic, im, iy, ib, ia;
-      float fc, fm, fy, fb, fa;
-      float scale;
+      CGFloat fc, fm, fy, fb, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       if (_hasAlpha)
         {
           // This order depends on the bitmap format
@@ -1192,10 +1179,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
       || [_colorSpace isEqualToString: NSDeviceRGBColorSpace])
     {
       unsigned int ir, ig, ib, ia;
-      float fr, fg, fb, fa;
-      float scale;
+      CGFloat fr, fg, fb, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       [conv getRed: &fr green: &fg blue: &fb alpha: &fa];
       if(_hasAlpha)
         {
@@ -1246,10 +1233,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
            || [_colorSpace isEqual: NSCalibratedWhiteColorSpace])
 		{
       unsigned int iw, ia;
-      float fw, fa;
-      float scale;
+      CGFloat fw, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       [conv getWhite: &fw alpha: &fa];
       if (_hasAlpha)
         {
@@ -1285,10 +1272,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
            || [_colorSpace isEqual: NSCalibratedBlackColorSpace])
     {
       unsigned int iw, ia;
-      float fw, fa;
-      float scale;
+      CGFloat fw, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       [conv getWhite: &fw alpha: &fa];
       if (_hasAlpha)
         {
@@ -1323,10 +1310,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
   else if ([_colorSpace isEqual: NSDeviceCMYKColorSpace])
     {
       unsigned int ic, im, iy, ib, ia;
-      float fc, fm, fy, fb, fa;
-      float scale;
+      CGFloat fc, fm, fy, fb, fa;
+      CGFloat scale;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       [conv getCyan: &fc magenta: &fm yellow: &fy black: &fb alpha: &fa];
       if(_hasAlpha)
         {
@@ -1780,21 +1767,21 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
     }
   else
     {
-      unsigned char* bits;
+      unsigned char *bits;
       unsigned int i;
 
-      bits = [copy->_imageData mutableBytes];
+      bits = (unsigned char *)[copy->_imageData bytes];
       copy->_imagePlanes[0] = bits;
       if (_isPlanar) 
 	{
-	  for (i=1; i < _numColors; i++) 
-	    copy->_imagePlanes[i] = bits + i*_bytesPerRow * _pixelsHigh;
-	  for (i= _numColors; i < MAX_PLANES; i++) 
+	  for (i = 1; i < _numColors; i++) 
+	    copy->_imagePlanes[i] = bits + i * _bytesPerRow * _pixelsHigh;
+	  for (i = _numColors; i < MAX_PLANES; i++) 
 	    copy->_imagePlanes[i] = NULL;
 	}
       else
 	{
-	  for (i= 1; i < MAX_PLANES; i++) 
+	  for (i = 1; i < MAX_PLANES; i++) 
 	    copy->_imagePlanes[i] = NULL;
 	}
     }
@@ -1930,6 +1917,14 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
   [_properties setObject: [NSNumber numberWithFloat: _comp_factor]
                   forKey: NSImageCompressionFactor];
 
+  if (info->xdpi > 0 && info->xdpi != 72 &&
+      info->ydpi > 0 && info->ydpi != 72)
+    {
+      NSSize pointSize = NSMakeSize((double)info->width * (72.0 / (double)info->xdpi),
+				    (double)info->height * (72.0 / (double)info->ydpi));
+      [self setSize: pointSize];
+    }
+
   if (NSTiffRead(image, info, [self bitmapData]))
     {
       free(info);
@@ -1995,10 +1990,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
     }
   else
     {
-      float scale;
-      float alpha;
+      CGFloat scale;
+      CGFloat alpha;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       for (y = 0; y < _pixelsHigh; y++)
         {
           for (x = 0; x < _pixelsWide; x++)
@@ -2080,10 +2075,10 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
     }
   else
     {
-      float scale;
-      float alpha;
+      CGFloat scale;
+      CGFloat alpha;
 
-      scale = (float)((1 << _bitsPerSample) - 1);
+      scale = (CGFloat)((1 << _bitsPerSample) - 1);
       for (y = 0; y < _pixelsHigh; y++)
         {
           unsigned int a;
@@ -2098,7 +2093,7 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
                     alpha = scale / a;
                     for (i = start; i < end; i++)
                       {
-                        float new = pixelData[i] * alpha;
+                        CGFloat new = pixelData[i] * alpha;
                         
                         if (new > scale)
                           {
@@ -2171,15 +2166,15 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
           IMP setP = [new methodForSelector: setPSel];
           unsigned int pixelData[5];
           int x, y;
-          float _scale;
-          float scale;
+          CGFloat _scale;
+          CGFloat scale;
 
           NSDebugLLog(@"NSImage", @"Converting %@ bitmap data", _colorSpace);
 
           if (_bitsPerSample != bps)
             {
-              _scale = (float)((1 << _bitsPerSample) - 1);
-              scale = (float)((1 << bps) - 1);
+              _scale = (CGFloat)((1 << _bitsPerSample) - 1);
+              scale = (CGFloat)((1 << bps) - 1);
             }
           else
             {
@@ -2192,7 +2187,7 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
               for (x = 0; x < _pixelsWide; x++)
                 {
                   unsigned int iv[4], ia;
-                  float fv[4], fa;
+                  CGFloat fv[4], fa;
                   int i;
 
                  //[self getPixel: pixelData atX: x y: y];
@@ -2311,8 +2306,8 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
           IMP setP = [new methodForSelector: setPSel];
           unsigned int pixelData[4];
           int x, y;
-          float _scale;
-          float scale;
+          CGFloat _scale;
+          CGFloat scale;
           int max = (1 << bps) - 1;
           BOOL isWhite = [_colorSpace isEqualToString: NSCalibratedWhiteColorSpace] 
               || [_colorSpace isEqualToString: NSDeviceWhiteColorSpace];
@@ -2321,8 +2316,8 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
 
           if (_bitsPerSample != bps)
             {
-              _scale = (float)((1 << _bitsPerSample) - 1);
-              scale = (float)((1 << bps) - 1);
+              _scale = (CGFloat)((1 << _bitsPerSample) - 1);
+              scale = (CGFloat)((1 << bps) - 1);
             }
           else
             {
@@ -2335,7 +2330,7 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
               for (x = 0; x < _pixelsWide; x++)
                 {
                   unsigned int iv, ia;
-                  float fv, fa;
+                  CGFloat fv, fa;
 
                  //[self getPixel: pixelData atX: x y: y];
                   getP(self, getPSel, pixelData, x, y);
@@ -2446,7 +2441,7 @@ _set_bit_value(unsigned char *base, long msb_off, int bit_width,
                   //[new setColor: c atX: i y: j];
                   setC(new, setCSel, c, i, j);
                 }
-              RELEASE(pool);
+              [pool drain];
             }
         }
 
