@@ -106,6 +106,12 @@ typedef struct _GSButtonCellFlags
 #endif
 } GSButtonCellFlags;
 
+@interface NSCell (Private)
+- (NSSize) _scaleImageWithSize: (NSSize)imageSize
+                   toFitInSize: (NSSize)canvasSize
+                   scalingType: (NSImageScaling)scalingType;
+@end
+
 /**<p> TODO Description</p>
  */
 @implementation NSButtonCell
@@ -137,6 +143,7 @@ typedef struct _GSButtonCellFlags
   _keyEquivalent = @"";
   _altContents = @"";
   _gradient_type = NSGradientNone;
+  [self setImageScaling: NSImageScaleNone];
 
   return self;
 }
@@ -200,10 +207,15 @@ typedef struct _GSButtonCellFlags
  */
 - (NSString*) title
 {
+  if (nil == _contents)
+    {
+      return @"";
+    }
+
   if (_cell.contents_is_attributed_string == NO)
     {
       // If we have a formatter this is also the string of the _object_value
-      return _contents;
+      return (NSString *)_contents;
     }
   else
     {
@@ -216,7 +228,14 @@ typedef struct _GSButtonCellFlags
  */
 - (NSString*) alternateTitle
 {
-  return _altContents;
+  if (_altContents != nil)
+    {
+      return _altContents;
+    }
+  else
+    {
+      return @"";
+    }
 }
 
 - (int) cellAttribute: (NSCellAttribute)aParameter
@@ -337,6 +356,14 @@ typedef struct _GSButtonCellFlags
 {
   ASSIGNCOPY(_contents, aString);
   _cell.contents_is_attributed_string = NO;
+
+  if (_control_view)
+    {
+      if ([_control_view isKindOfClass: [NSControl class]])
+        {
+          [(NSControl*)_control_view updateCell: self];
+        }
+    }
 }
 
 /**<p>Sets the NSButtonCell's alternate title ( used when highlighted ) 
@@ -363,7 +390,7 @@ typedef struct _GSButtonCellFlags
   NSAttributedString *attrStr;
 
   dict = [self _nonAutoreleasedTypingAttributes];
-  attrStr = [[NSAttributedString alloc] initWithString: _altContents 
+  attrStr = [[NSAttributedString alloc] initWithString: [self alternateTitle]
                                         attributes: dict];
   RELEASE(dict);
 
@@ -378,7 +405,8 @@ typedef struct _GSButtonCellFlags
 
 - (NSAttributedString *)attributedTitle
 {
-  if (_cell.contents_is_attributed_string)
+  if (_cell.contents_is_attributed_string &&
+      nil != _contents)
     {
       return (NSAttributedString *)_contents;
     }
@@ -388,7 +416,7 @@ typedef struct _GSButtonCellFlags
       NSAttributedString *attrStr;
 
       dict = [self _nonAutoreleasedTypingAttributes];
-      attrStr = [[NSAttributedString alloc] initWithString: _contents 
+      attrStr = [[NSAttributedString alloc] initWithString: [self title]
                                             attributes: dict];
       RELEASE(dict);
       return AUTORELEASE(attrStr);
@@ -399,6 +427,14 @@ typedef struct _GSButtonCellFlags
 {
   ASSIGNCOPY(_contents, aString);
   _cell.contents_is_attributed_string = YES;
+
+  if (_control_view)
+    {
+      if ([_control_view isKindOfClass: [NSControl class]])
+        {
+          [(NSControl*)_control_view updateCell: self];
+        }
+    }
 }
 
 - (void)setTitleWithMnemonic:(NSString *)aString
@@ -413,20 +449,20 @@ typedef struct _GSButtonCellFlags
   return @"";
 }
 
-- (unsigned)alternateMnemonicLocation
+- (NSUInteger)alternateMnemonicLocation
 {
   // TODO
   return NSNotFound;
 }
 
-- (void)setAlternateMnemonicLocation:(unsigned)location
+- (void)setAlternateMnemonicLocation:(NSUInteger)location
 {
   // TODO
 }
 
 - (void)setAlternateTitleWithMnemonic:(NSString *)aString
 {
-  unsigned int location = [aString rangeOfString: @"&"].location;
+  NSUInteger location = [aString rangeOfString: @"&"].location;
 
   [self setAlternateTitle: [aString stringByReplacingString: @"&"
                                     withString: @""]];
@@ -451,6 +487,16 @@ typedef struct _GSButtonCellFlags
   return _cell.image_position;
 }
 
+- (NSImageScaling) imageScaling
+{
+  return _imageScaling;
+}
+
+- (void) setImageScaling: (NSImageScaling)scaling
+{
+  _imageScaling = scaling;
+}
+
 - (void) setImage: (NSImage *)anImage
 {
   if (_cell.image_position == NSNoImage)
@@ -467,6 +513,14 @@ typedef struct _GSButtonCellFlags
 - (void) setAlternateImage: (NSImage*)anImage
 {
   ASSIGN(_altImage, anImage);
+
+  if (_control_view)
+    {
+      if ([_control_view isKindOfClass: [NSControl class]])
+        {
+          [(NSControl*)_control_view updateCell: self];
+        }
+    }
 }
 
 /**<p>Sets the image position. The GNUstep implementation depends only on 
@@ -495,6 +549,14 @@ typedef struct _GSButtonCellFlags
     {
       _cell.type = NSImageCellType;
     }
+
+  if (_control_view)
+    {
+      if ([_control_view isKindOfClass: [NSControl class]])
+        {
+          [(NSControl*)_control_view updateCell: self];
+        }
+    }
 }
 
 /**<p>Gets the NSButtonCell's <var>delay</var> and the <var>interval</var>
@@ -530,7 +592,14 @@ typedef struct _GSButtonCellFlags
  */
 - (NSString*) keyEquivalent
 {
-  return _keyEquivalent;
+  if (nil == _keyEquivalent)
+    {
+      return @"";
+    }
+  else
+    {
+      return _keyEquivalent;
+    }
 }
 
 /**<p>Returns the NSFont of the key equivalent.</p>
@@ -547,7 +616,7 @@ typedef struct _GSButtonCellFlags
     0.</p><p>See Also: -setKeyEquivalentModifierMask:
     -keyEquivalent [NSButton-performKeyEquivalent:]</p>
  */
-- (unsigned int) keyEquivalentModifierMask
+- (NSUInteger) keyEquivalentModifierMask
 {
   return _keyEquivalentModifierMask;
 }
@@ -570,7 +639,7 @@ typedef struct _GSButtonCellFlags
     <p>See Also: -keyEquivalentModifierMask  
     -setKeyEquivalent: [NSButton-performKeyEquivalent:]</p>
 */
-- (void) setKeyEquivalentModifierMask: (unsigned int)mask
+- (void) setKeyEquivalentModifierMask: (NSUInteger)mask
 {
   _keyEquivalentModifierMask = mask;
 }
@@ -669,7 +738,7 @@ typedef struct _GSButtonCellFlags
    NSChangeBackgroundCellMask</p>
    <p>See Also : -setHighlightsBy:</p>
  */
-- (int) highlightsBy
+- (NSInteger) highlightsBy
 {
   return _highlightsByMask;
 }
@@ -679,12 +748,12 @@ typedef struct _GSButtonCellFlags
    NSChangeBackgroundCellMask</p>
    <p>See Also : -highlightsBy</p>
  */
-- (void) setHighlightsBy: (int)mask
+- (void) setHighlightsBy: (NSInteger)mask
 {
   _highlightsByMask = mask;
 }
 
-- (void) setShowsStateBy: (int)mask
+- (void) setShowsStateBy: (NSInteger)mask
 {
   _showAltStateMask = mask;
 }
@@ -750,7 +819,7 @@ typedef struct _GSButtonCellFlags
     }
 }
 
-- (int) showsStateBy
+- (NSInteger) showsStateBy
 {
   return _showAltStateMask;
 }
@@ -914,26 +983,63 @@ typedef struct _GSButtonCellFlags
   // Draw image
   if (imageToDisplay != nil)
     {
-      NSSize size;
-      NSPoint position;
-      
-      size = [imageToDisplay size];
-      position.x = MAX(NSMidX(cellFrame) - (size.width / 2.), 0.);
-      position.y = MAX(NSMidY(cellFrame) - (size.height / 2.), 0.);
-      
-      /*
-       * Images are always drawn with their bottom-left corner at the origin
-       * so we must adjust the position to take account of a flipped view.
-       */
-      if ([controlView isFlipped])
-        {
-          position.y += size.height;
-        }
+      NSPoint offset;
+      NSRect rect;
+      CGFloat fraction;
+      NSSize size = [self _scaleImageWithSize: [imageToDisplay size]
+				  toFitInSize: cellFrame.size
+				  scalingType: _imageScaling];
 
-      [[GSTheme theme] drawImage: imageToDisplay
-		    inButtonCell: self
-		       withFrame: cellFrame
-		        position: position];
+      /* Pixel-align size */
+
+      if (controlView)
+	{
+	  NSSize sizeInBase = [controlView convertSizeToBase: size];
+	  sizeInBase.width = GSRoundTowardsInfinity(sizeInBase.width);
+	  sizeInBase.height = GSRoundTowardsInfinity(sizeInBase.height);
+	  size = [controlView convertSizeFromBase: sizeInBase];
+	}
+
+      /* Calculate an offset from the cellFrame origin */
+     
+      offset = NSMakePoint((NSWidth(cellFrame) - size.width) / 2.0,
+				 (NSHeight(cellFrame) - size.height) / 2.0);
+
+      /* Pixel-align the offset */
+
+      if (controlView)
+	{
+	  NSPoint inBase = [controlView convertPointToBase: offset];
+	  
+	  // By convention we will round down and to the right.
+	  // With the standard button design this looks good
+	  // because the bottom and right edges of the button look 'heavier'
+	  // so if the image's center must be offset from the button's geometric
+	  // center, it looks beter if it's closer to the 'heavier' part
+	  
+	  inBase.x = GSRoundTowardsInfinity(inBase.x);
+	  inBase.y = [controlView isFlipped] ? 
+	    GSRoundTowardsInfinity(inBase.y) :
+	    GSRoundTowardsNegativeInfinity(inBase.y);
+	  
+	  offset = [controlView convertPointFromBase: inBase];
+	}
+
+      /* Draw the image */
+
+      rect = NSMakeRect(cellFrame.origin.x + offset.x,
+   		        cellFrame.origin.y + offset.y,
+			size.width,
+			size.height);
+      fraction = (![self isEnabled] &&
+		  [self imageDimsWhenDisabled]) ? 0.5 : 1.0;
+      
+      [imageToDisplay drawInRect: rect
+			fromRect: NSZeroRect
+		       operation: NSCompositeSourceOver
+			fraction: fraction
+		  respectFlipped: YES
+			   hints: nil];
     }
 }
 
@@ -1237,7 +1343,7 @@ typedef struct _GSButtonCellFlags
 - (NSSize) cellSize
 {
   NSSize s;
-  NSSize borderSize;
+  GSThemeMargins border;
   unsigned mask;
   NSImage *imageToDisplay;
   NSAttributedString *titleToDisplay;
@@ -1337,13 +1443,18 @@ typedef struct _GSButtonCellFlags
           buttonState = GSThemeSelectedState;
         }
 
-      borderSize = [[GSTheme theme] buttonBorderForCell: self
-						  style: _bezel_style 
-                                                  state: buttonState];
+      border = [[GSTheme theme] buttonMarginsForCell: self
+					       style: _bezel_style 
+					       state: buttonState];
     }
   else
-    borderSize = NSZeroSize;
-
+    {
+      border.left = 0;
+      border.top = 0;
+      border.right = 0;
+      border.bottom = 0;
+    }
+      
   /* Add an additional 6 pixels horizontally so that the text is not
    * too near the boundaries of the button.  Without them, autosized
    * buttons look too tiny and crammed.  This might be made
@@ -1356,12 +1467,13 @@ typedef struct _GSButtonCellFlags
   if ((_cell.is_bordered && (_cell.image_position != NSImageOnly))
       || _cell.is_bezeled)
     {
-      borderSize.width += 6;
+      border.left += 6;
+      border.right += 6;
     }
 
   // Add border size
-  s.width += 2 * borderSize.width;
-  s.height += 2 * borderSize.height;
+  s.width += border.left + border.right;
+  s.height += border.top + border.bottom;
 
   return s;
 }
@@ -1370,7 +1482,7 @@ typedef struct _GSButtonCellFlags
 {
   if (_cell.is_bordered)
     {
-      NSSize borderSize;
+      GSThemeMargins border;
       unsigned mask;
       GSThemeControlState buttonState = GSThemeNormalState;
       NSRect interiorFrame;
@@ -1401,10 +1513,16 @@ typedef struct _GSButtonCellFlags
           buttonState = GSThemeSelectedState;
         }
 
-      borderSize = [[GSTheme theme] buttonBorderForCell: self
-						  style: _bezel_style 
-                                                  state: buttonState];
-      interiorFrame = NSInsetRect(theRect, borderSize.width, borderSize.height);
+      border = [[GSTheme theme] buttonMarginsForCell: self
+					       style: _bezel_style 
+					       state: buttonState];
+
+      interiorFrame = theRect;
+      interiorFrame.origin.x += border.left;
+      interiorFrame.size.width -= border.left + border.right;
+      interiorFrame.origin.y += ([_control_view isFlipped] ? 
+				 border.top : border.bottom);
+      interiorFrame.size.height -= border.bottom + border.top;
 
       /* Pushed in buttons contents are displaced to the bottom right 1px.  */
       if (mask & NSPushInCellMask)
@@ -1502,7 +1620,7 @@ typedef struct _GSButtonCellFlags
       NSImage *image = [self image];
       NSButtonImageSource *bi = nil;
 
-      if ([self keyEquivalent] != nil)
+      if ([[self keyEquivalent] length] > 0)
         {
           [aCoder encodeObject: [self keyEquivalent] forKey: @"NSKeyEquivalent"];
         }
@@ -1510,7 +1628,7 @@ typedef struct _GSButtonCellFlags
         {
           [aCoder encodeObject: [self image] forKey: @"NSNormalImage"];
         }
-      if ([self alternateTitle] != nil)
+      if ([[self alternateTitle] length] > 0)
         {
           [aCoder encodeObject: [self alternateTitle] forKey: @"NSAlternateContents"];
         }
@@ -1553,6 +1671,23 @@ typedef struct _GSButtonCellFlags
       bFlags2 |= [self showsBorderOnlyWhileMouseInside] ? 0x8 : 0;
       bFlags2 |= (([self bezelStyle] & 0x7) | (([self bezelStyle] & 0x18) << 2));
       bFlags2 |= [self keyEquivalentModifierMask] << 8;
+
+      switch ([self imageScaling])
+	{
+	case NSImageScaleProportionallyDown:
+	  bFlags2 |= (2 << 6);
+	  break;
+	case NSImageScaleAxesIndependently:
+	  bFlags2 |= (3 << 6);
+	  break;
+	case NSImageScaleNone:
+	default:
+	  break;
+	case NSImageScaleProportionallyUpOrDown:
+	  bFlags2 |= (1 << 6);
+	  break;
+	}
+
       [aCoder encodeInt: bFlags2 forKey: @"NSButtonFlags2"];
 
       // alternate image encoding...
@@ -1704,6 +1839,7 @@ typedef struct _GSButtonCellFlags
         }
       if ([aDecoder containsValueForKey: @"NSButtonFlags2"])
         {
+	  NSUInteger imageScale;
           int bFlags2;
 
           bFlags2 = [aDecoder decodeIntForKey: @"NSButtonFlags2"];
@@ -1711,6 +1847,24 @@ typedef struct _GSButtonCellFlags
           [self setBezelStyle: (bFlags2 & 0x7) | ((bFlags2 & 0x20) >> 2)];
           [self setKeyEquivalentModifierMask: ((bFlags2 >> 8) & 
                                                NSDeviceIndependentModifierFlagsMask)];
+
+	  switch (bFlags2 & (3 << 6))
+	    {
+	    case 2:
+	      imageScale = NSImageScaleProportionallyDown;
+	      break;
+	    case 3:
+	      imageScale = NSImageScaleAxesIndependently;
+	      break;
+	    case 0:
+	    default:
+	      imageScale = NSImageScaleNone;
+	      break;
+	    case 1:
+	      imageScale = NSImageScaleProportionallyUpOrDown;
+	      break;
+	    }
+	  [self setImageScaling: imageScale];
         }
       if ([aDecoder containsValueForKey: @"NSAlternateImage"])
         {
@@ -1793,6 +1947,8 @@ typedef struct _GSButtonCellFlags
           [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &tmp];
           _shows_border_only_while_mouse_inside = tmp;
         }
+      // Not encoded in non-keyed archive
+      _imageScaling = NSImageScaleNone;
     }
 
   // Hack to correct a Gorm problem, there "\n" is used instead of "\r".
