@@ -81,31 +81,57 @@
     sharing it with other images. </p>
  */
 - (id) initWithSize: (NSSize)aSize
-	      depth: (NSWindowDepth)aDepth
+	 pixelsWide: (int)pixelsWide
+	 pixelsHigh: (int)pixelsHigh
+	      depth: (int)aDepth
 	   separate: (BOOL)separate
 	      alpha: (BOOL)alpha
 {
   NSWindow	*win;
-  NSRect	frame;
+
+  if (aSize.width <= 0 || aSize.height <= 0 ||
+      pixelsWide <= 0 || pixelsHigh <= 0)
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"NSCachedImageRep created with size %@ pixelsWide %d pixelsHigh %d",
+		   NSStringFromSize(aSize), pixelsWide, pixelsHigh];
+    }
 
   // FIXME: Only create new window when separate is YES
-  frame.origin = NSMakePoint(0,0);
-  frame.size = aSize;
-  win = [[GSCacheW alloc] initWithContentRect: frame
-				    styleMask: NSBorderlessWindowMask
+  win = [[GSCacheW alloc] initWithContentRect: NSMakeRect(0, 0, pixelsWide, pixelsHigh)
+				    styleMask: NSBorderlessWindowMask | NSUnscaledWindowMask
 				      backing: NSBackingStoreRetained
 					defer: NO];
-  self = [self initWithWindow: win rect: frame];
+ 
+  [[win contentView] setBoundsSize: NSMakeSize(aSize.width, aSize.height)];
+  
+  self = [self initWithWindow: win rect: NSMakeRect(0, 0, aSize.width, aSize.height)];
   RELEASE(win);
   if (!self)
     return nil;
-
+  
   [self setAlpha: alpha];
   [self setBitsPerSample: NSBitsPerSampleFromDepth(aDepth)];
-
+  [self setSize: aSize];
+  [self setPixelsWide: pixelsWide];
+  [self setPixelsHigh: pixelsHigh];
+ 
   return self;
 }
 
+- (id) initWithSize: (NSSize)aSize
+	      depth: (int)aDepth
+	   separate: (BOOL)separate
+	      alpha: (BOOL)alpha
+{
+  return [self initWithSize: aSize
+		 pixelsWide: aSize.width
+		 pixelsHigh: aSize.height
+		      depth: aDepth
+		   separate: separate
+		      alpha: alpha];
+}
+
 /** <p>Initializes and returns a new NSCachedImageRep into a NSWindow 
     <var>aWindow</var>. The image will be draw into the rectange aRect of
     this window. aWindow is retained.</p>
@@ -136,7 +162,7 @@
     }
   if (!_window)
     _window = [[GSCacheW alloc] initWithContentRect: _rect
-					  styleMask: NSBorderlessWindowMask
+					  styleMask: NSBorderlessWindowMask | NSUnscaledWindowMask
 					    backing: NSBackingStoreRetained
 					      defer: NO];
   [self setSize: _rect.size];
@@ -172,14 +198,13 @@
 - (BOOL) draw
 {
 /*
-  FIXME: Horrible hack to get drawing on a scaled or rotated
-  context correct. We need another interface with the backend 
-  to do it properly.
+  Horrible hack to get drawing on a scaled or rotated
+  context correct. Only used on backends not supporting GSdraw:
+  (art/xlib backend).
 */
   NSGraphicsContext *ctxt = GSCurrentContext();
   NSAffineTransform *transform;
   NSAffineTransformStruct ts;
-
   // Is there anything to draw?
   if (NSIsEmptyRect(_rect))
     return YES;
@@ -209,6 +234,52 @@
   return YES;
 }
 
+/**
+ * Overridden private NSImageRep method called for backends supporting
+ * drawGState. This is an efficient implementation which boils down to
+ * cairo calls to set the source surface, clip, and paint.
+ */
+- (void) nativeDrawInRect: (NSRect)dstRect
+                 fromRect: (NSRect)srcRect
+                operation: (NSCompositingOperation)op
+                 fraction: (float)delta
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  NSWindow *window = [self window];
+  int gState = [window gState];
+  NSAffineTransform *transform, *backup;
+  NSRect winSrcRect;
+
+  // Convert srcRect from rep coordinate space to window base coordinate space
+  {
+    CGFloat repToWinWidthScaleFactor = [window frame].size.width / [self size].width;
+    CGFloat repToWinHeightScaleFactor = [window frame].size.height / [self size].height;
+    
+    winSrcRect = NSMakeRect(srcRect.origin.x * repToWinWidthScaleFactor,
+			    srcRect.origin.y * repToWinHeightScaleFactor,
+			    srcRect.size.width * repToWinWidthScaleFactor,
+			    srcRect.size.height * repToWinHeightScaleFactor);
+  }
+
+  //NSLog(@"Using NSCachedImageRep nativeDrawInRect: fast path for %@", self);
+  
+  backup = [ctxt GSCurrentCTM];
+  
+  transform = [NSAffineTransform transform];
+  [transform translateXBy: dstRect.origin.x yBy: dstRect.origin.y];
+  [transform scaleXBy: dstRect.size.width / winSrcRect.size.width
+		  yBy: dstRect.size.height / winSrcRect.size.height];
+  [transform concat];
+  
+  [ctxt GSdraw: gState
+       toPoint: NSMakePoint(0,0)
+      fromRect: winSrcRect
+     operation: op
+      fraction: delta];
+  
+  [ctxt GSSetCTM: backup];
+}
+
 // NSCoding protocol
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
