@@ -2,7 +2,7 @@
 
    <abstract>The abstract cell class</abstract>
 
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996-2012 Free Software Foundation, Inc.
 
    Author:  Scott Christley <scottc@net-community.com>
    Date: 1996
@@ -177,8 +177,8 @@ static NSColor *dtxtCol;
   //_cell.state = 0;
   //_cell.line_break_mode = NSLineBreakByWordWrapping;
   _action_mask = NSLeftMouseUpMask;
-  _menu = [isa defaultMenu];
-  [self setFocusRingType: [isa defaultFocusRingType]];
+  _menu = [object_getClass(self) defaultMenu];
+  [self setFocusRingType: [object_getClass(self) defaultFocusRingType]];
 
   return self;
 }
@@ -208,8 +208,8 @@ static NSColor *dtxtCol;
   //_cell.is_selectable = NO;
   //_cell.line_break_mode = NSLineBreakByWordWrapping;
   _action_mask = NSLeftMouseUpMask;
-  _menu = [isa defaultMenu];
-  [self setFocusRingType: [isa defaultFocusRingType]];
+  _menu = [object_getClass(self) defaultMenu];
+  [self setFocusRingType: [object_getClass(self) defaultFocusRingType]];
 
   return self;
 }
@@ -316,10 +316,15 @@ static NSColor *dtxtCol;
  */
 - (NSString*) stringValue
 {
+  if (nil == _contents)
+    {
+      return @"";
+    }
+
   if (_cell.contents_is_attributed_string == NO)
     {
       // If we have a formatter this is also the string of the _object_value
-      return _contents;
+      return (NSString *)_contents;
     }
   else
     {
@@ -382,7 +387,7 @@ static NSColor *dtxtCol;
         }
     }
 
-  ASSIGN (_contents, newContents);
+  ASSIGNCOPY(_contents, newContents);
 }
 
 
@@ -479,7 +484,7 @@ static NSColor *dtxtCol;
         }
       else
         {
-          ASSIGN (_contents, aString);
+          ASSIGNCOPY(_contents, aString);
           _cell.contents_is_attributed_string = NO;
           _cell.has_valid_object_value = NO;
         }
@@ -774,7 +779,7 @@ static NSColor *dtxtCol;
    implementation. (Currently they match the Cocoa values but they are
    quite strange)</p> <p>See Also: -state</p>
  */
-- (void) setState: (int)value
+- (void) setState: (NSInteger)value
 {
   /* We do exactly as in macosx when value is not NSOnState,
    * NSOffState, NSMixedState, even if their behaviour (value < 0 ==>
@@ -799,7 +804,7 @@ static NSColor *dtxtCol;
 /**<p>Returns the NSCell's state</p>
  <p>See Also: -setState: </p>
 */
-- (int) state
+- (NSInteger) state
 {
   return _cell.state;
 }
@@ -818,7 +823,7 @@ static NSColor *dtxtCol;
     }
 }
 
-- (int) nextState
+- (NSInteger) nextState
 {
   switch (_cell.state)
     {
@@ -1019,7 +1024,8 @@ static NSColor *dtxtCol;
     }
 
   /* In all other cases */
-  if (_cell.contents_is_attributed_string)
+  if (_cell.contents_is_attributed_string &&
+      nil != _contents)
     {
       return (NSAttributedString *)_contents;
     }
@@ -1029,7 +1035,7 @@ static NSColor *dtxtCol;
       NSAttributedString *attrStr;
 
       dict = [self _nonAutoreleasedTypingAttributes];
-      attrStr = [[NSAttributedString alloc] initWithString: _contents 
+      attrStr = [[NSAttributedString alloc] initWithString: [self stringValue]
                                             attributes: dict];
       RELEASE(dict);
       return AUTORELEASE(attrStr);
@@ -1384,7 +1390,7 @@ static NSColor *dtxtCol;
 
 - (NSString*) mnemonic
 {
-  unsigned int location = [self mnemonicLocation];
+  NSUInteger location = [self mnemonicLocation];
   NSString *c = [self title];
 
   if ((location == NSNotFound) || location >= [c length])
@@ -1393,12 +1399,12 @@ static NSColor *dtxtCol;
   return [c substringWithRange: NSMakeRange (location, 1)];
 }
 
-- (void) setMnemonicLocation: (unsigned int)location 
+- (void) setMnemonicLocation: (NSUInteger)location 
 {
   _cell.mnemonic_location = location;
 }
 
-- (unsigned int) mnemonicLocation
+- (NSUInteger) mnemonicLocation
 {
   return _cell.mnemonic_location;
 }
@@ -2021,14 +2027,13 @@ static NSColor *dtxtCol;
             size = [_cell_image size];
             position.x = MAX(NSMidX(drawingRect) - (size.width/2.),0.);
             position.y = MAX(NSMidY(drawingRect) - (size.height/2.),0.);
-            /*
-             * Images are always drawn with their bottom-left corner
-             * at the origin so we must adjust the position to take
-             * account of a flipped view.
-             */
-            if ([controlView isFlipped])
-              position.y += size.height;
-            [_cell_image compositeToPoint: position operation: NSCompositeSourceOver];
+
+            [_cell_image drawInRect: NSMakeRect(position.x, position.y, size.width, size.height)
+			   fromRect: NSZeroRect
+			  operation: NSCompositeSourceOver
+			   fraction: 1.0
+		     respectFlipped: YES
+			      hints: nil];
           }
         break;
 
@@ -2368,7 +2373,6 @@ static NSColor *dtxtCol;
       cFlags |= (_action_mask & NSLeftMouseUpMask) ?  0 : 0x20;
       cFlags |= [self wraps] ?  0x40 : 0;
       cFlags |= (_action_mask & NSLeftMouseDraggedMask) ?  0x100 : 0;
-      cFlags |= ([self lineBreakMode] << 12);
       cFlags |= (_action_mask & NSLeftMouseDownMask) ?  0x40000 : 0;
       cFlags |= [self isContinuous] ? 0x80000 : 0;
       cFlags |= [self isScrollable] ? 0x100000 : 0; 
@@ -2384,6 +2388,7 @@ static NSColor *dtxtCol;
       
       // flags part 2
       cFlags2 |= ([self controlTint] << 5);
+      cFlags2 |= ([self lineBreakMode] << 9);
       cFlags2 |= ([self controlSize] << 17);
       cFlags2 |= [self sendsActionOnEndEditing] ? 0x400000 : 0;
       cFlags2 |= [self allowsMixedState] ? 0x1000000 : 0;
@@ -2526,7 +2531,6 @@ static NSColor *dtxtCol;
           [self setWraps: ((cFlags & 0x40) != 0x40)];
           if ((cFlags & 0x100) == 0x100)
             mask |= NSLeftMouseDraggedMask;
-          [self setLineBreakMode: ((cFlags & 0x7000) >> 12)];
           if ((cFlags & 0x40000) == 0x40000)
             mask |= NSLeftMouseDownMask;
           if ((cFlags & 0x80000) == 0x80000)
@@ -2549,6 +2553,7 @@ static NSColor *dtxtCol;
       
           cFlags2 = [aDecoder decodeIntForKey: @"NSCellFlags2"];
           [self setControlTint: ((cFlags2 & 0xE0) >> 5)];
+	  [self setLineBreakMode: ((cFlags2 & 0xE00) >> 9)];
           [self setControlSize: ((cFlags2 & 0xE0000) >> 17)];
           [self setSendsActionOnEndEditing: ((cFlags2 & 0x400000) == 0x400000)];
           [self setAllowsMixedState: ((cFlags2 & 0x1000000) == 0x1000000)];
@@ -2986,28 +2991,119 @@ static NSColor *dtxtCol;
       contents = [_formatter editingStringForObjectValue: _object_value];
       if (contents == nil)
         {
-          contents = _contents;
+          // We cannot call the stringValue method as this will call
+          // validateEditing in NSActionCell subclasses
+          if (nil == _contents)
+            {
+              contents = @"";
+            }
+
+          if (_cell.contents_is_attributed_string == NO)
+            {
+              contents = (NSString *)_contents;
+            }
+          else
+            {
+              contents = [(NSAttributedString *)_contents string];
+            }
         }
       if (![contents isEqualToString: [textObject string]])
 	[textObject setText: contents];
     }
   else
     {
+      NSString *contents; 
+
+      if (nil == _contents)
+        {
+          contents = @"";
+        }
+
       if (_cell.contents_is_attributed_string == NO)
         {
-	  if (![_contents isEqualToString:[textObject string]])
-	    [textObject setText: _contents];
+          contents = (NSString *)_contents;
         }
       else
         {
-	  // FIXME what about attribute changes?
-          NSRange range = NSMakeRange(0, [[textObject string] length]);
-	  if (![[(NSAttributedString *)_contents string] isEqualToString:
-		 [textObject string]])
-	    [textObject replaceCharactersInRange: range
-			withAttributedString: (NSAttributedString *)_contents];
+          contents = [(NSAttributedString *)_contents string];
+        }
+      if (![contents isEqualToString: [textObject string]])
+        {
+          if (_cell.contents_is_attributed_string == NO)
+            {
+              [textObject setText: contents];
+            }
+          else
+            {
+              // FIXME what about attribute changes?
+              NSRange range = NSMakeRange(0, [[textObject string] length]);
+              [textObject replaceCharactersInRange: range
+                              withAttributedString: (NSAttributedString *)_contents];
+            }
         }
     }
 }
 
+/**
+ * Private method used by NSImageCell and NSButtonCell for calculating
+ * scaled image size
+ */
+
+static inline NSSize
+scaleProportionally(NSSize imageSize, NSSize canvasSize, BOOL scaleUpOrDown)
+{
+  CGFloat ratio;
+
+  if (imageSize.width <= 0
+      || imageSize.height <= 0)
+    {
+      return NSMakeSize(0, 0);
+    }
+
+  /* Get the smaller ratio and scale the image size by it.  */
+  ratio = MIN(canvasSize.width / imageSize.width,
+	      canvasSize.height / imageSize.height);
+  
+  /* Only scale down, unless scaleUpOrDown is YES */
+  if (ratio < 1.0 || scaleUpOrDown)
+    {
+      imageSize.width *= ratio;
+      imageSize.height *= ratio;
+    }
+
+  return imageSize;
+}
+
+- (NSSize) _scaleImageWithSize: (NSSize)imageSize
+                   toFitInSize: (NSSize)canvasSize
+		   scalingType: (NSImageScaling)scalingType
+{
+  NSSize result;
+  switch (scalingType)
+    {
+    case NSImageScaleProportionallyDown: // == NSScaleProportionally
+      {
+	result = scaleProportionally (imageSize, canvasSize, NO);
+	break;
+      }
+    case NSImageScaleAxesIndependently: // == NSScaleToFit
+      {
+	result = canvasSize;
+	break;
+      }
+    default:
+    case NSImageScaleNone: // == NSScaleNone
+      {
+	result = imageSize;
+	break;
+      }
+    case NSImageScaleProportionallyUpOrDown:
+      {
+	result = scaleProportionally (imageSize, canvasSize, YES);
+	break;
+      }
+    }
+  return result;
+}
+
 @end
