@@ -44,8 +44,6 @@
 
 #include <math.h>
 
-DEFINE_RINT_IF_MISSING
-
 @interface NSClipView (Private)
 - (void) _scrollToPoint: (NSPoint)aPoint;
 @end
@@ -257,7 +255,8 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
       return;
     }
       
-  if (_copiesOnScroll && _window && [_window gState])
+  // FIXME: Re-enable after release
+  if (NO) //_copiesOnScroll && _window && [_window gState])
     {
       /* Copy the portion of the view that is common before and after
          scrolling.  Then, document view needs to redraw the remaining
@@ -444,8 +443,8 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
   */
   if (_copiesOnScroll)
     {
-      new.x = _bounds.origin.x + (rint(new.x - _bounds.origin.x));
-      new.y = _bounds.origin.y + (rint(new.y - _bounds.origin.y));
+      new.x = _bounds.origin.x + (GSRoundTowardsInfinity(new.x - _bounds.origin.x));
+      new.y = _bounds.origin.y + (GSRoundTowardsInfinity(new.y - _bounds.origin.y));
     }
 
   return new;
@@ -773,6 +772,11 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   self = [super initWithCoder: aDecoder];
+  if (self == nil)
+    {
+      return nil;
+    }
+
   if ([aDecoder allowsKeyedCoding])
     {
       [self setAutoresizesSubviews: YES];
@@ -793,7 +797,6 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
       if ([[self subviews] count] > 0)
         {
           NSRect rect;
-
 	  id document = [aDecoder decodeObjectForKey: @"NSDocView"];
 
 	  NSAssert([document class] != [NSCustomView class],
@@ -802,14 +805,13 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
 	  rect.origin = NSZeroPoint;
 	  [document setFrame: rect];
 	  RETAIN(document); // prevent it from being released.
-	  [self removeSubview: document];
+	  [document removeFromSuperview];
 	  [self setDocumentView: document];
 	  RELEASE(document);
 	}
     }
   else
     {
-      NSView *document;
       BOOL temp;
       
       [self setAutoresizesSubviews: YES];
@@ -822,9 +824,11 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
       
       if ([[self subviews] count] > 0)
         {
-	  document = AUTORELEASE(RETAIN([[self subviews] objectAtIndex: 0]));
-	  [self removeSubview: document];
+          NSView *document = [[self subviews] objectAtIndex: 0];
+	  RETAIN(document); // prevent it from being released.
+	  [document removeFromSuperview];
 	  [self setDocumentView: document];
+	  RELEASE(document);
 	}
     }
   return self;
