@@ -29,6 +29,7 @@
 #import "config.h"
 #import <Foundation/NSBundle.h>
 #import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSLock.h>
 #import <Foundation/NSNotification.h>
@@ -36,11 +37,13 @@
 #import "AppKit/NSBox.h"
 #import "AppKit/NSButton.h"
 #import "AppKit/NSButtonCell.h"
+#import "AppKit/NSBezierPath.h"
 #import "AppKit/NSColor.h"
 #import "AppKit/NSColorPanel.h"
 #import "AppKit/NSColorPicker.h"
 #import "AppKit/NSColorPicking.h"
 #import "AppKit/NSColorWell.h"
+#import "AppKit/NSCursor.h"
 #import "AppKit/NSEvent.h"
 #import "AppKit/NSGraphics.h"
 #import "AppKit/NSImage.h"
@@ -48,10 +51,13 @@
 #import "AppKit/NSPasteboard.h"
 #import "AppKit/NSPanel.h"
 #import "AppKit/NSSlider.h"
+#import "AppKit/NSScreen.h"
 #import "AppKit/NSSplitView.h"
 #import "AppKit/NSWindow.h"
+#import "GNUstepGUI/GSDisplayServer.h"
 #import "GNUstepGUI/IMLoading.h"
 
+
 #import "GSGuiPrivate.h"
 
 #define MAX_ALPHA_VALUE 100.0
@@ -83,6 +89,7 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
 - (void) _setupPickers;
 - (void) _showNewPicker: (id)sender;
 - (id) _initWithoutGModel;
+- (void) _magnify: (id)sender;
 @end
 
 @implementation NSColorPanel (PrivateMethods)
@@ -358,14 +365,59 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
 - (void) _apply: (id) sender
 {
   // This is currently not used
-  [NSApp sendAction: @selector(changeColor:) to: nil from: self];
-  if ((_action) && (_target != nil))
-    [NSApp sendAction: _action to: _target from: self];  
 }
 
-- (void) _maginify: (id) sender
+- (void) _magnify: (id) sender
 {
-  NSLog(@"Magnification is not implemented");
+  NSEvent *currentEvent;
+  NSCursor *cursor;
+
+  [self _captureMouse: self];
+
+  /**
+   * There was code here to dynamically generate a magnifying glass
+   * cursor with a magnified portion of the screenshot in it,
+   * but changing the cursor rapidly on X seems to cause flicker,
+   * so we just use a plain magnifying glass. (dynamic code is in r33543)
+   */
+  cursor = [[[NSCursor alloc] initWithImage: [NSImage imageNamed: @"MagnifyGlass"]
+					      hotSpot: NSMakePoint(12, 13)] autorelease];
+  [cursor push];
+
+  NS_DURING
+    {
+      do {
+	NSPoint mouseLoc;
+	NSImage *img;
+	
+	currentEvent = [NSApp nextEventMatchingMask: NSLeftMouseDownMask | NSLeftMouseUpMask | NSMouseMovedMask
+					  untilDate: [NSDate distantFuture]
+					     inMode: NSEventTrackingRunLoopMode
+					    dequeue: YES];
+	
+	mouseLoc = [self convertBaseToScreen: [self mouseLocationOutsideOfEventStream]];
+	
+	img = [GSCurrentServer() contentsOfScreen: [[self screen] screenNumber]
+					   inRect: NSMakeRect(mouseLoc.x, mouseLoc.y, 1, 1)];
+	
+	if (img != nil)
+	  {
+	    NSBitmapImageRep *rep = (NSBitmapImageRep *)[img bestRepresentationForDevice: nil];
+	    NSColor *color = [rep colorAtX: 0 y: 0];
+	    [self setColor: color];
+	  }
+      } while ([currentEvent type] != NSLeftMouseUp && 
+	       [currentEvent type] != NSLeftMouseDown);
+    }
+  NS_HANDLER
+    {
+      NSLog(@"Exception occurred in -[NSColorPanel _magnify:]: %@",
+	    localException);
+    }
+  NS_ENDHANDLER
+    
+  [NSCursor pop];
+  [self _releaseMouse: self];
 }
 
 - (void) _updatePicker: (id) sender
@@ -547,7 +599,7 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
     return;
 
   if (_accessoryView != nil)
-    [_splitView removeSubview: _accessoryView];
+    [_accessoryView removeFromSuperview];
   _accessoryView = aView;
   [_splitView addSubview: _accessoryView];
 }
@@ -690,7 +742,12 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
   if ([self showsAlpha])
     [_alphaSlider setFloatValue: [aColor alphaComponent] * MAX_ALPHA_VALUE];
 
-  if (_isContinuous && (_action) && (_target != nil))
+  // FIXME: to support [self isContinuous] the API between color pickers
+  // and NSColorPanel needs some changes. Currently the color panel is
+  // always continuous
+
+  [NSApp sendAction: @selector(changeColor:) to: nil from: self];
+  if ((_action) && (_target != nil))
     [NSApp sendAction: _action to: _target from: self];  
 
   [[NSNotificationCenter defaultCenter]
