@@ -27,28 +27,30 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#import <Foundation/Foundation.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSString.h>
 #import <AppKit/NSColorSpace.h>
-#include "GSGuiPrivate.h"
+#import "GSGuiPrivate.h"
 
 @implementation NSColorSpace
 
 - (id) _initWithColorSpaceModel: (NSColorSpaceModel)model
 {
-	if ((self = [super init]))
-		{
+  if ((self = [super init]))
+    {
       // FIXME: Load corresponding data
 
       _colorSpaceModel = model;
-		}
-	return self;
+    }
+  return self;
 }
 
 #define COLORSPACE(model) \
-	static NSColorSpace *csp; \
-	if (!csp) \
-		csp = [[self alloc] _initWithColorSpaceModel: model]; \
-	return csp;
+  static NSColorSpace *csp = nil;                     \
+  if (!csp)                                                           \
+    csp = [[self alloc] _initWithColorSpaceModel: model];             \
+  return csp;
 
 + (NSColorSpace *) deviceCMYKColorSpace
 {
@@ -82,8 +84,8 @@
 
 - (id) initWithColorSyncProfile: (void *)prof
 {
-	if ((self = [super init]))
-		{
+  if ((self = [super init]))
+    {
       _colorSyncProfile = prof;
       _colorSpaceModel = NSUnknownColorSpaceModel;
     }
@@ -92,8 +94,8 @@
 
 - (id) initWithICCProfileData: (NSData *)iccData
 {
-	if ((self = [super init]))
-		{
+  if ((self = [super init]))
+    {
       ASSIGN(_iccData, iccData);
       _colorSpaceModel = NSUnknownColorSpaceModel;
     }
@@ -163,7 +165,7 @@
 - (void) encodeWithCoder: (NSCoder *)coder
 {
   // FIXME
-	if ([coder allowsKeyedCoding])
+  if ([coder allowsKeyedCoding])
     {
     }
   else
@@ -174,13 +176,13 @@
 - (id) initWithCoder: (NSCoder *)aDecoder
 {
   // FIXME
-	if ([aDecoder allowsKeyedCoding])
+  if ([aDecoder allowsKeyedCoding])
     {
     }
   else
     {
     }
-	return self;
+  return self;
 }
 
 @end
