@@ -261,9 +261,9 @@ static GSComboWindow *gsWindow = nil;
   if (num > max)
     num = max;
    
-  [self setFrame: NSMakeRect(0, 0, popUpWidth, 
+  [self setFrame: [self frameRectForContentRect: NSMakeRect(0, 0, popUpWidth, 
      2 * bsize.height + (itemHeight + intercellSpacing.height) * (num - 1)
-     + itemHeight) display: NO];
+     + itemHeight)] display: NO];
 }
 
 - (void) positionWithComboBoxCell: (NSComboBoxCell *)comboBoxCell
@@ -441,7 +441,7 @@ static GSComboWindow *gsWindow = nil;
     
   _stopped = NO;
     
-  RELEASE(pool);
+  [pool drain];
 }
 
 // onWindow notifications
@@ -1095,7 +1095,7 @@ static GSComboWindow *gsWindow = nil;
    }
  else
    {
-     int i = [_popUpList indexOfObject: object];
+     NSInteger i = [_popUpList indexOfObject: object];
 
      if (i == NSNotFound)
        i = -1;
@@ -1850,7 +1850,7 @@ static inline NSRect buttonCellFrameFromRect(NSRect cellRect)
 - (void) _selectCompleted
 {
   NSString *more;
-  unsigned int index = NSNotFound;
+  NSUInteger index = NSNotFound;
 
   more = [self completedString: [self stringValue]];
   if (_usesDataSource)
