@@ -32,13 +32,18 @@
 
 #import <Foundation/NSDebug.h>
 #import <Foundation/NSException.h>
+#import <Foundation/NSKeyValueCoding.h>
+#import <Foundation/NSKeyValueObserving.h>
 #import <Foundation/NSNotification.h>
+#import <Foundation/NSValue.h>
 #import "AppKit/NSActionCell.h"
 #import "AppKit/NSApplication.h"
 #import "AppKit/NSCell.h"
 #import "AppKit/NSControl.h"
 #import "AppKit/NSColor.h"
 #import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSFontManager.h"
 #import "AppKit/NSKeyValueBinding.h"
 #import "AppKit/NSTextStorage.h"
 #import "AppKit/NSTextView.h"
@@ -77,6 +82,8 @@ static NSNotificationCenter *nc;
       [self exposeBinding: NSEnabledBinding];
       [self exposeBinding: NSAlignmentBinding];
       [self exposeBinding: NSFontBinding];
+      [self exposeBinding: NSFontNameBinding];
+      [self exposeBinding: NSFontSizeBinding];
      }
 }
 
@@ -206,6 +213,15 @@ static NSNotificationCenter *nc;
   return [[self selectedCell] intValue];
 }
 
+/** <p>Returns the value of the NSControl's selected cell as int.</p>
+    <p>See Also: -setIntegerValue: [NSCell-integerValue] -floatValue -doubleValue
+    -stringValue</p>
+ */
+- (NSInteger) integerValue
+{
+  return [[self selectedCell] integerValue];
+}
+
 /** <p>Returns the value of the NSControl's selected cell as NSString.</p>
     <p>See Also: -setStringValue: [NSCell-stringValue] -intValue -floatValue
     -doubleValue -stringValue</p>
@@ -280,6 +296,26 @@ static NSNotificationCenter *nc;
     }
 }
 
+/** <p>Sets the value of the NSControl's selected cell to int.
+    If the selected cell is an action cell, it marks self for display.</p>
+    <p>See Also: -integerValue [NSCell-setIntegerValue:] -setDoubleValue: 
+    -setFloatValue: -setStringValue:</p>
+ */
+- (void) setIntegerValue: (NSInteger)anInt
+{
+  NSCell *selected = [self selectedCell];
+  BOOL wasEditing = [self abortEditing];
+
+  [selected setIntegerValue: anInt];
+  if (![selected isKindOfClass: actionCellClass])
+    [self setNeedsDisplay: YES];
+
+  if (wasEditing)
+    {
+      [[self window] makeFirstResponder: self];
+    }
+}
+
 /** <p>Sets the value of the NSControl's selected cell to NSString.
     If the selected cell is an action cell, it marks self for display.</p>
     <p>See Also: -stringValue [NSCell-setStringValue:] -setIntValue:
@@ -343,7 +379,7 @@ static NSNotificationCenter *nc;
 }
 
 /**<p>Sets the NSControl's selected cell to the sender's float int.</p>
-   <p>See Also: [NSCell-takeDoubleValueFrom:] -takeDoubleValueFrom: 
+   <p>See Also: [NSCell-takeIntValueFrom:] -takeDoubleValueFrom: 
    -takeFloatValueFrom: -takeStringValueFrom:</p> 
 */
 - (void) takeIntValueFrom: (id)sender
@@ -352,6 +388,16 @@ static NSNotificationCenter *nc;
   [self setNeedsDisplay: YES];
 }
 
+/**<p>Sets the NSControl's selected cell to the sender's float int.</p>
+   <p>See Also: [NSCell-takeIntegerValueFrom:] -takeDoubleValueFrom: 
+   -takeFloatValueFrom: -takeStringValueFrom:</p> 
+*/
+- (void) takeIntegerValueFrom: (id)sender
+{
+  [[self selectedCell] takeIntegerValueFrom: sender];
+  [self setNeedsDisplay: YES];
+}
+
 - (void) takeObjectValueFrom: (id)sender
 {
   [[self selectedCell] takeObjectValueFrom: sender];
@@ -1007,8 +1053,6 @@ static NSNotificationCenter *nc;
       GSKeyValueBinding *kvb;
 
       [self unbind: binding];
-      // FIXME: We could also do the mapping via 
-      // setKeys:triggerChangeNotificationsForDependentKey:
       kvb = [[GSKeyValueBinding alloc] initWithBinding: @"objectValue"
                                               withName: NSValueBinding
                                               toObject: anObject
@@ -1041,4 +1085,38 @@ static NSNotificationCenter *nc;
     }
 }
 
+- (void) setValue: (id)anObject forKey: (NSString*)aKey
+{
+  if ([aKey isEqual: NSFontNameBinding])
+    {
+      [self setFont: [[NSFontManager sharedFontManager] convertFont: [self font] 
+                                                             toFace: anObject]];
+    }
+  else if ([aKey isEqual: NSFontSizeBinding])
+    {
+      [self setFont: [[NSFontManager sharedFontManager] convertFont: [self font]
+                                                             toSize: [anObject doubleValue]]];
+    }
+  else
+    {
+      [super setValue: anObject forKey: aKey];
+    }
+}
+
+- (id) valueForKey: (NSString*)aKey
+{
+  if ([aKey isEqual: NSFontNameBinding])
+    {
+      return [[self font] fontName];
+    }
+  else if ([aKey isEqual: NSFontSizeBinding])
+    {
+      return [NSNumber numberWithDouble: (double)[[self font] pointSize]];
+    }
+  else
+    {
+      return [super valueForKey: aKey];
+    }
+}
+
 @end
