@@ -41,11 +41,13 @@
 #import "AppKit/NSDocumentController.h"
 #import "AppKit/NSOpenPanel.h"
 #import "AppKit/NSApplication.h"
+#import "AppKit/NSKeyValueBinding.h"
 #import "AppKit/NSMenu.h"
 #import "AppKit/NSMenuItem.h"
 #import "AppKit/NSWorkspace.h"
 #import "NSDocumentFrameworkPrivate.h"
 #import "GSGuiPrivate.h"
+#import "GSBindingHelpers.h"
 
 static NSString *NSTypesKey             = @"NSTypes";
 static NSString *NSNameKey              = @"NSName";
@@ -246,6 +248,8 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
 	      allTypes = [[NSArray alloc] initWithObjects: valid count: nValid];
 	    }
 	}
+
+      [self exposeBinding: @"autosavingDelay"];
     }
 }
 
@@ -324,6 +328,9 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
 
 - (void) dealloc
 {
+  // Remove all key value bindings for this object.
+  [GSKeyValueBinding unbindAllForObject: self];
+
   [[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver: self];
   RELEASE (_documents);
   RELEASE (_recent_documents);
@@ -1339,7 +1346,7 @@ static BOOL _shouldClose = YES;
 
 - (void) noteNewRecentDocumentURL: (NSURL *)anURL
 {
-  unsigned index = [_recent_documents indexOfObject: anURL];
+  NSUInteger index = [_recent_documents indexOfObject: anURL];
   NSMutableArray *a;
 
   if (index != NSNotFound)
@@ -1706,6 +1713,7 @@ static NSString *processName = nil;
   // action to open recent document by tag index
   NSURL *url;
   int idx = [sender tag];
+  NSError *err = nil;
 
   if (idx < 0 || idx >= [_recent_documents count])
     {
@@ -1727,7 +1735,10 @@ static NSString *processName = nil;
             return;
         }
     }
-  [self openDocumentWithContentsOfURL: url display: YES];
+
+  [self openDocumentWithContentsOfURL: url display: YES error: &err];
+  if (err)
+    [self presentError: err];
 }
 
 @end
