@@ -30,7 +30,9 @@
 #ifndef _GNUstep_H_NSDocumentFramworkPrivate
 #define _GNUstep_H_NSDocumentFramworkPrivate
 
-#include <AppKit/NSDocumentController.h>
+#import "AppKit/NSDocument.h"
+#import "AppKit/NSDocumentController.h"
+#import "AppKit/NSWindowController.h"
 
 @class NSTimer;
 
@@ -43,8 +45,12 @@
 - (void)_recordAutosavedDocument: (NSDocument *)document;
 @end
 
-
-#include <AppKit/NSDocument.h>
+@interface NSDocumentController (RecentDocumentsMenu)
+- (NSMenu *) _recentDocumentsMenu;
+- (void) _setRecentDocumentsMenu: (NSMenu *)menu;
+- (void) _updateRecentDocumentsMenu;
+- (IBAction) _openRecentDocument: (id)sender;
+@end
 
 @interface NSDocument (Private)
 - (void)_removeWindowController:(NSWindowController *)controller;
@@ -52,9 +58,6 @@
 - (void)_removeAutosavedContentsFile;
 @end
 
-
-#include <AppKit/NSWindowController.h>
-
 @interface NSWindowController (Private)
 - (void)_windowDidLoad;
 @end
