@@ -32,23 +32,23 @@
 */
 
 #include "config.h"
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSLock.h>
-#include <Foundation/NSTimer.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSValue.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSTimer.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSValue.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSDebug.h>
 
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSGraphicsContext.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/PSOperators.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSGraphicsContext.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/PSOperators.h"
 
-#include "GNUstepGUI/GSDisplayServer.h"
+#import "GNUstepGUI/GSDisplayServer.h"
 
 /*
  *        gstep-base has a faster mechanism to get the current thread.
@@ -64,6 +64,7 @@
  * Class variables
  */
 static NSString *timerKey = @"NSEventTimersKey";
+static Class dateClass;
 static Class eventClass;
 
 /*
@@ -74,6 +75,7 @@ static Class eventClass;
   if (self == [NSEvent class])
     {
       [self setVersion: 3];
+      dateClass = [NSDate class];
       eventClass = [NSEvent class];
     }
 }
@@ -310,7 +312,7 @@ static Class eventClass;
   NSTimeInterval timeInterval;
   NSEvent        *periodicEvent;
 
-  timeInterval = [[NSDate date] timeIntervalSinceReferenceDate];
+  timeInterval = [dateClass timeIntervalSinceReferenceDate];
   periodicEvent = [self otherEventWithType: NSPeriodic
                                   location: NSZeroPoint
                              modifierFlags: 0
@@ -338,7 +340,7 @@ static Class eventClass;
     NSTimeInterval timeInterval;
     NSEvent        *periodicEvent;
     
-    timeInterval = [[NSDate date] timeIntervalSinceReferenceDate];
+    timeInterval = [dateClass timeIntervalSinceReferenceDate];
     periodicEvent = [self otherEventWithType: NSPeriodic
                           location: NSZeroPoint
                           modifierFlags: 0
@@ -517,16 +519,14 @@ static Class eventClass;
    </p>
    <p>
    This method is only valid for NSMouseMoved, NS*MouseDragged and 
-   NSScrollWheel events, otherwise it will raise an 
-   NSInternalInconsistencyException.
+   NSScrollWheel events, otherwise it will return 0.
    </p>
  */
-- (float)deltaX
+- (float) deltaX
 {
   if (!(NSEventMaskFromType(event_type) & GSMouseMovedEventMask))
     {
-      [NSException raise: NSInternalInconsistencyException
-                  format: @"deltaX requested for invalid event type"];
+      return 0.0;
     }
   return event_data.mouse.deltaX;
 }
@@ -537,16 +537,14 @@ static Class eventClass;
    </p>
    <p>
    This method is only valid for NSMouseMoved, NS*MouseDragged and 
-   NSScrollWheel events, otherwise it will raise an 
-   NSInternalInconsistencyException.
+   NSScrollWheel events, otherwise it will return 0.
    </p>
  */
-- (float)deltaY
+- (float) deltaY
 {
   if (!(NSEventMaskFromType(event_type) & GSMouseMovedEventMask))
     {
-      [NSException raise: NSInternalInconsistencyException
-                  format: @"deltaY requested for invalid event type"];
+      return 0.0;
     }
   return event_data.mouse.deltaY;
 }
@@ -557,19 +555,17 @@ static Class eventClass;
    </p>
    <p>
    This method is only valid for NSMouseMoved, NS*MouseDragged and 
-   NSScrollWheel events, otherwise it will raise an 
-   NSInternalInconsistencyException.
+   NSScrollWheel events, otherwise it will return 0.
    </p>
    <p>
    The value returned is 0.0 in most cases.
    </p>
  */
-- (float)deltaZ
+- (float) deltaZ
 {
   if (!(NSEventMaskFromType(event_type) & GSMouseMovedEventMask))
     {
-      [NSException raise: NSInternalInconsistencyException
-                  format: @"deltaZ requested for invalid event type"];
+      return 0.0;
     }
   return event_data.mouse.deltaZ;
 }
@@ -702,7 +698,7 @@ static const char *eventTypes[] = {
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [aCoder encodeValueOfObjCType: @encode(NSEventType) at: &event_type];
+  [aCoder encodeValueOfObjCType: @encode(int) at: &event_type];
   [aCoder encodePoint: location_point];
   [aCoder encodeValueOfObjCType: @encode(unsigned) at: &modifier_flags];
   [aCoder encodeValueOfObjCType: @encode(NSTimeInterval) at: &event_time];
@@ -785,7 +781,7 @@ static const char *eventTypes[] = {
 {
   int version = [aDecoder versionForClassName: @"NSEvent"];
 
-  [aDecoder decodeValueOfObjCType: @encode(NSEventType) at: &event_type];
+  [aDecoder decodeValueOfObjCType: @encode(int) at: &event_type];
   location_point = [aDecoder decodePoint];
   [aDecoder decodeValueOfObjCType: @encode(unsigned) at: &modifier_flags];
   [aDecoder decodeValueOfObjCType: @encode(NSTimeInterval) at: &event_time];
