@@ -29,14 +29,14 @@
 */ 
 
 #include "config.h"
-#include <Foundation/NSNotification.h>
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSFormCell.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSTextFieldCell.h"
-#include "GNUstepGUI/GSTheme.h"
+#import <Foundation/NSNotification.h>
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSFormCell.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSTextFieldCell.h"
+#import "GNUstepGUI/GSTheme.h"
 
 /** <p>TODO Description </p>
  */
@@ -61,7 +61,8 @@
  */
 - (id) initTextCell: (NSString *)aString
 {
-  if (! ( self = [super initTextCell: @""] ) )
+  self = [super initTextCell: @""];
+  if (nil == self)
     return nil;
   
   _cell.is_bezeled = YES;
@@ -473,7 +474,10 @@
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder: aDecoder];
+  self = [super initWithCoder: aDecoder];
+  if (nil == self)
+    return nil;
+
   if ([aDecoder allowsKeyedCoding])
     {
       if ([aDecoder containsValueForKey: @"NSContents"])
