@@ -561,6 +561,8 @@ NSGraphicsContext	*GSCurrentContext(void)
     GET_IMP(@selector(GSShowText::));
   methodTable.GSShowGlyphs__ =
     GET_IMP(@selector(GSShowGlyphs::));
+  methodTable.GSShowGlyphsWithAdvances__ =
+    GET_IMP(@selector(GSShowGlyphsWithAdvances:::));
 
 /* ----------------------------------------------------------------------- */
 /* Gstate Handling */
@@ -879,7 +881,7 @@ NSGraphicsContext	*GSCurrentContext(void)
     should have n components, where n corresponds to the number of
     color components required to specify the color in the current
     colorspace (Quartz). */
-- (void) GSSetFillColor: (const float *)values
+- (void) GSSetFillColor: (const CGFloat *)values
 {
   [self subclassResponsibility: _cmd];
 }
@@ -888,7 +890,7 @@ NSGraphicsContext	*GSCurrentContext(void)
     should have n components, where n corresponds to the number of
     color components required to specify the color in the current
     colorspace (Quartz). */
-- (void) GSSetStrokeColor: (const float *)values
+- (void) GSSetStrokeColor: (const CGFloat *)values
 {
   [self subclassResponsibility: _cmd];
 }
@@ -1078,6 +1080,13 @@ NSGraphicsContext	*GSCurrentContext(void)
   [self subclassResponsibility: _cmd];
 }
 
+/** Paints the glyphs with the specified advances using the current font.
+    (Quartz). */
+- (void) GSShowGlyphsWithAdvances: (const NSGlyph *)glyphs : (const NSSize *)advances : (size_t) length
+{
+  [self subclassResponsibility: _cmd];
+}
+
 /* ----------------------------------------------------------------------- */
 /* Gstate Handling */
 /* ----------------------------------------------------------------------- */
