@@ -109,8 +109,10 @@ first. Remaining cases, highest priority first:
 #import <Foundation/NSException.h>
 #import <Foundation/NSValue.h>
 #import "AppKit/NSAttributedString.h"
+#import "AppKit/NSBezierPath.h"
 #import "AppKit/NSColor.h"
 #import "AppKit/NSImage.h"
+#import "AppKit/NSKeyValueBinding.h"
 #import "AppKit/NSLayoutManager.h"
 #import "AppKit/NSParagraphStyle.h"
 #import "AppKit/NSRulerMarker.h"
@@ -118,9 +120,9 @@ first. Remaining cases, highest priority first:
 #import "AppKit/NSTextStorage.h"
 #import "AppKit/NSWindow.h"
 #import "AppKit/DPSOperators.h"
-#import "AppKit/NSBezierPath.h"
 
 #import "GNUstepGUI/GSLayoutManager_internal.h"
+#import "GSBindingHelpers.h"
 
 
 @interface NSLayoutManager (spelling)
@@ -455,6 +457,8 @@ container? necessary? */
 {
   NSRange r1, r2;
 
+  // FIXME: should accept {NSNotFound, 0} for selCharRange to ignore that parameter
+
   /* TODO: we can actually do better than this by using the insertion point
   positioning behavior */
   r1 = [self glyphRangeForCharacterRange: charRange
@@ -868,7 +872,13 @@ has the same y origin and height as the line frag rect it is in.
 
 	  /* will be -1 if there are no text containers */
 	  *textContainer = num_textcontainers - 1;
-	  return NSMakeRect(0, 0, 1, [f boundingRectForFont].size.height);
+	  r = NSMakeRect(0, 0, 1, [f boundingRectForFont].size.height);
+	  if (num_textcontainers > 0)
+	    {
+	      tc = textcontainers + num_textcontainers - 1;
+	      r.origin.x += [tc->textContainer lineFragmentPadding];
+	    }
+	  return r;
 	}
       fraction_through = 1.0;
     }
@@ -963,7 +973,6 @@ has the same y origin and height as the line frag rect it is in.
 {
   NSRect from_rect, new_rect;
   int from_tc, new_tc;
-  int i;
   unsigned int new;
   unsigned int length = [_textStorage length];
 
@@ -973,33 +982,84 @@ has the same y origin and height as the line frag rect it is in.
 					   textContainer: &from_tc];
   if (from_tc == -1)
     {
-      NSLog(@"%s: character index not in any text container",
-	__PRETTY_FUNCTION__);
-      return from;
+      /* The from character index is not in a text container, so move the
+	 cursor to the start of the text. */
+      return 0;
     }
 
-  if (direction == GSInsertionPointMoveLeft ||
-      direction == GSInsertionPointMoveRight)
+  /* Simple case which moves one character left/right or one line up/down,
+     but supports moving between text containers. */
+  if (distance == 0.0)
     {
-      float target;
-
-      if (distance == 0.0)
+      if (direction == GSInsertionPointMoveLeft ||
+	  direction == GSInsertionPointMoveRight)
 	{
 	  new = from;
 	  if (direction == GSInsertionPointMoveLeft && new > 0)
 	    new--;
 	  if (direction == GSInsertionPointMoveRight && new < length)
 	    new++;
-
-	  [self _insertionPointRectForCharacterIndex: new
-				       textContainer: &i];
-
-	/* Don't leave the text container. */
-	if (i == from_tc)
+	  
 	  return new;
-	else
-	  return from;
 	}
+      else if (direction == GSInsertionPointMoveUp ||
+	       direction == GSInsertionPointMoveDown)
+	{
+	  int orig_tc;
+	  const float target = [self _insertionPointRectForCharacterIndex: original
+							    textContainer: &orig_tc].origin.x;
+
+	  const int delta = (direction == GSInsertionPointMoveUp) ? -1 : 1;
+
+	  /* First scan forward or backwards until we end up on a new line */
+	  for (new = from; (direction == GSInsertionPointMoveUp) ? (new > 0) : (new < length); new += delta)
+	    {
+	      new_rect = [self _insertionPointRectForCharacterIndex: new
+						      textContainer: &new_tc];
+	      if (new_rect.origin.y != from_rect.origin.y || new_tc != from_tc)
+		break;
+	    }
+
+	  /* We found the start of the line, now find the target character on that line. */
+	  new_rect = [self _insertionPointRectForCharacterIndex: new
+						  textContainer: &new_tc];
+	  while ((direction == GSInsertionPointMoveUp) ? (new > 0) : (new < length))
+	    {
+	      int prev_tc = new_tc;
+	      NSRect prev_rect = new_rect;
+	      new_rect = [self _insertionPointRectForCharacterIndex: new + delta
+						      textContainer: &new_tc];
+
+	      /* 'new+delta' is on a different line than the target line, so the 
+		 target character must be 'new'.*/
+	      if (new_rect.origin.y != prev_rect.origin.y || new_tc != prev_tc)
+		return new;
+
+	      /* If we pass the target point, the character we want is either
+		 'new' or 'new+delta' */
+	      if ((direction == GSInsertionPointMoveDown && NSMinX(new_rect) >= target)
+		  || (direction == GSInsertionPointMoveUp && NSMinX(new_rect) <= target))
+		{
+		  if (fabs(NSMinX(new_rect) - target) > fabs(NSMinX(prev_rect) - target))
+		    return new;
+		  
+		  return new + delta;
+		}
+
+	      new += delta;
+	    }
+	  return new;
+	}
+    }
+
+  /* The following more complex cases are for when a minimum distance is specified.
+     However, they will not move out of from's text container.
+  */
+
+  if (direction == GSInsertionPointMoveLeft ||
+      direction == GSInsertionPointMoveRight)
+    {
+      float target;
 
       /*
       This is probably very inefficient, but it shouldn't be a bottleneck,
@@ -1432,9 +1492,12 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
   */
   NSColor *defaultTextColor = [NSColor textColor];
   NSColor *selectedTextColor = defaultTextColor;
+  NSColor *link_color = nil;
+  id linkValue;
  
 #define GBUF_SIZE 16 /* TODO: tweak */
   NSGlyph gbuf[GBUF_SIZE];
+  NSSize advancementbuf[GBUF_SIZE];
   int gbuf_len, gbuf_size;
   NSPoint gbuf_point = NSZeroPoint;
 
@@ -1490,6 +1553,18 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
   if (run_color == nil)
     run_color = defaultTextColor;
 
+  linkValue = [attributes objectForKey: NSLinkAttributeName];
+  if (linkValue != nil)
+    {
+      if (link_color == nil)
+	{
+	  NSDictionary *link_attributes = [[self firstTextView] linkTextAttributes];
+	  link_color = [link_attributes valueForKey: NSForegroundColorAttributeName];
+	}
+      if (link_color != nil)
+	run_color = link_color;
+    }
+
   if (selectedGlyphRange.length > 0)
     {
       /* Get the text view's color setting for selected text as we will
@@ -1527,7 +1602,7 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
 	  if (gbuf_len)
 	    {
 	      DPSmoveto(ctxt, gbuf_point.x, gbuf_point.y);
-	      GSShowGlyphs(ctxt, gbuf, gbuf_len);
+	      GSShowGlyphsWithAdvances(ctxt, gbuf, advancementbuf, gbuf_len);
 	      DPSnewpath(ctxt);
 	      gbuf_len = 0;
 	    }
@@ -1556,7 +1631,7 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
 	  if (gbuf_len)
 	    {
 	      DPSmoveto(ctxt, gbuf_point.x, gbuf_point.y);
-	      GSShowGlyphs(ctxt, gbuf, gbuf_len);
+	      GSShowGlyphsWithAdvances(ctxt, gbuf, advancementbuf, gbuf_len);
 	      DPSnewpath(ctxt);
 	      gbuf_len = 0;
 	    }
@@ -1585,6 +1660,19 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
 	    {
 	      run_color = defaultTextColor;
 	    }
+
+	  linkValue = [attributes objectForKey: NSLinkAttributeName];
+	  if (linkValue != nil)
+	      {
+		if (link_color == nil)
+		  {
+		    NSDictionary *link_attributes = [[self firstTextView] linkTextAttributes];
+		    link_color = [link_attributes valueForKey: NSForegroundColorAttributeName];
+		  }
+		if (link_color != nil)
+		  run_color = link_color;
+	      }
+
 	  glyph = glyph_run->glyphs;
 
 	  /* If the font has changed or the color has changed (and we are
@@ -1597,7 +1685,7 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
 	      if (gbuf_len)
 		{
 		  DPSmoveto(ctxt, gbuf_point.x, gbuf_point.y);
-		  GSShowGlyphs(ctxt, gbuf, gbuf_len);
+		  GSShowGlyphsWithAdvances(ctxt, gbuf, advancementbuf, gbuf_len);
 		  DPSnewpath(ctxt);
 		  gbuf_len = 0;
 		}
@@ -1668,12 +1756,14 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
 		  if (gbuf_len == gbuf_size)
 		    {
 		      DPSmoveto(ctxt, gbuf_point.x, gbuf_point.y);
-		      GSShowGlyphs(ctxt, gbuf, gbuf_size);
+		      GSShowGlyphsWithAdvances(ctxt, gbuf, advancementbuf, gbuf_size);
 		      DPSnewpath(ctxt);
 		      gbuf_len = 0;
 		      gbuf_point = p;
 		    }
-		  gbuf[gbuf_len++] = glyph->g;
+		  gbuf[gbuf_len] = glyph->g;
+		  advancementbuf[gbuf_len] = [f advancementForGlyph: glyph->g];
+		  gbuf_len++;
 		}
 	    }
 	  p.x += [f advancementForGlyph: glyph->g].width;
@@ -1685,7 +1775,7 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
 printf("%i at (%g %g) 4\n", gbuf_len, gbuf_point.x, gbuf_point.y);
 for (i = 0; i < gbuf_len; i++) printf("   %3i : %04x\n", i, gbuf[i]); */
       DPSmoveto(ctxt, gbuf_point.x, gbuf_point.y);
-      GSShowGlyphs(ctxt, gbuf, gbuf_len);
+      GSShowGlyphsWithAdvances(ctxt, gbuf, advancementbuf, gbuf_len);
       DPSnewpath(ctxt);
     }
 
@@ -1696,14 +1786,37 @@ for (i = 0; i < gbuf_len; i++) printf("   %3i : %04x\n", i, gbuf[i]); */
   {
     const NSRange characterRange = [self characterRangeForGlyphRange: range
 						    actualGlyphRange: NULL];
+    id linkUnderlineValue = nil;
 
     for (i=characterRange.location; i<NSMaxRange(characterRange); )
       {
 	NSRange underlinedCharacterRange;
-	id underlineValue = [[self textStorage] attribute: NSUnderlineStyleAttributeName
-						  atIndex: i
-				    longestEffectiveRange: &underlinedCharacterRange
-						  inRange: characterRange];
+	NSRange linkCharacterRange;
+	id underlineValue = nil;
+
+	linkValue = [_textStorage attribute: NSLinkAttributeName
+				    atIndex: i
+		      longestEffectiveRange: &linkCharacterRange
+				    inRange: characterRange];
+	if (linkValue != nil)
+	  {
+	    if (linkUnderlineValue == nil)
+	      {
+		NSDictionary *link_attributes = [[self firstTextView] linkTextAttributes];
+		linkUnderlineValue = [link_attributes valueForKey: NSUnderlineStyleAttributeName];
+	      }
+	    underlineValue = linkUnderlineValue;
+	    underlinedCharacterRange = linkCharacterRange;
+	  }
+	else
+	  {
+	    underlineValue = [_textStorage attribute: NSUnderlineStyleAttributeName
+					     atIndex: i
+			       longestEffectiveRange: &underlinedCharacterRange
+					     inRange: characterRange];
+	    underlinedCharacterRange = NSIntersectionRange(underlinedCharacterRange, 
+							   linkCharacterRange);
+	  }
 	if (underlineValue != nil && [underlineValue integerValue] != NSUnderlineStyleNone)
 	  {
 	    const NSRange underlinedGylphRange = [self glyphRangeForCharacterRange: underlinedCharacterRange
@@ -1860,6 +1973,7 @@ static void GSDrawPatternLine(NSPoint start, NSPoint end, NSInteger pattern, CGF
   // contiguous regions with the same underline color.
 
   NSUInteger i;
+  NSColor *link_color = nil;
   const NSRange characterRange = [self characterRangeForGlyphRange: underlineRange
 						  actualGlyphRange: NULL];
 
@@ -1870,56 +1984,84 @@ static void GSDrawPatternLine(NSPoint start, NSPoint end, NSInteger pattern, CGF
       return;
     }
 
-  for (i=characterRange.location; i<NSMaxRange(characterRange); )
+  for (i = characterRange.location; i < NSMaxRange(characterRange); )
     {
       NSRange underlineColorCharacterRange, foregroundColorCharacterRange, rangeToDraw;
-      NSColor *underlineColor = (NSColor*)[[self textStorage] attribute: NSUnderlineColorAttributeName
-								atIndex: i
-						  longestEffectiveRange: &underlineColorCharacterRange
-								inRange: NSMakeRange(i, NSMaxRange(characterRange)-i)];
-      NSColor *foregroundColor = (NSColor*)[[self textStorage] attribute: NSForegroundColorAttributeName
-								 atIndex: i
-						   longestEffectiveRange: &foregroundColorCharacterRange
-								 inRange: NSMakeRange(i, NSMaxRange(characterRange)-i)];
-      /*NSLog(@"asked to underline '%@'. at %d, ul color is %@ (%@) and fg color is %@ (%@)", [[[self textStorage] string] substringWithRange: characterRange],
-	    i,
-	    underlineColor,
-	    NSStringFromRange(underlineColorCharacterRange),
-	    foregroundColor,
-	    NSStringFromRange(foregroundColorCharacterRange));*/
+      NSColor *underlineColor = nil;
+      NSRange glyphRangeToDraw;
+      NSRange linkCharacterRange;
+      id linkValue;
+
+      linkValue = [_textStorage attribute: NSLinkAttributeName
+				  atIndex: i
+		    longestEffectiveRange: &linkCharacterRange
+				  inRange: NSMakeRange(i, NSMaxRange(characterRange)-i)];
+      if (linkValue != nil)
+	{
+	  if (link_color == nil)
+	    {
+	      NSDictionary *link_attributes = [[self firstTextView] linkTextAttributes];
+	      link_color = [link_attributes valueForKey: NSForegroundColorAttributeName];
+	    }
+	  if (link_color != nil)
+	    underlineColor = link_color;
+
+	  underlineColorCharacterRange = linkCharacterRange;
+	}
+      else
+	{
+	  underlineColor = (NSColor*)[[self textStorage] 
+				       attribute: NSUnderlineColorAttributeName
+					 atIndex: i
+				       longestEffectiveRange: &underlineColorCharacterRange
+					 inRange: NSMakeRange(i, NSMaxRange(characterRange)-i)];
+	  underlineColorCharacterRange = NSIntersectionRange(underlineColorCharacterRange, 
+							     linkCharacterRange);
+	}
 
       if (underlineColor != nil)
 	{
 	  [underlineColor set];
+	  rangeToDraw = underlineColorCharacterRange;
 	}
-      else if (foregroundColor != nil)
+      else
 	{
-	  [foregroundColor set];
-	}
-      else 
-	{
-	  [[NSColor textColor] set];
-	}
+	  NSColor *foregroundColor = (NSColor*)[[self textStorage] 
+						 attribute: NSForegroundColorAttributeName
+						   atIndex: i
+						 longestEffectiveRange: &foregroundColorCharacterRange
+						   inRange: NSMakeRange(i, NSMaxRange(characterRange)-i)];
 
-      // Draw the smaller range
-      rangeToDraw = underlineColorCharacterRange.length < foregroundColorCharacterRange.length ?
-	underlineColorCharacterRange : foregroundColorCharacterRange;
+	  if (foregroundColor != nil)
+	    {
+	      [foregroundColor set];
+	    }
+	  else 
+	    {
+	      [[NSColor textColor] set];
+	    }
+
+	  // Draw the smaller range
+	  rangeToDraw = underlineColorCharacterRange.length < foregroundColorCharacterRange.length ?
+	    underlineColorCharacterRange : foregroundColorCharacterRange;
+	}
       
-      // do the actual underline
-      {
-
-	  const NSRange glyphRangeToDraw = [self glyphRangeForCharacterRange: rangeToDraw
-						       actualCharacterRange: NULL];
-
+      glyphRangeToDraw = [self glyphRangeForCharacterRange: rangeToDraw
+				      actualCharacterRange: NULL];
+      if (glyphRangeToDraw.length > 0)
+	{
+	  // do the actual underline
+	  
 	  // FIXME: find the largest font within the range to underline
-	  NSFont *largestFont = [self effectiveFontForGlyphAtIndex: glyphRangeToDraw.location // NOTE: GS private method
-						     range: NULL];
+	  // NOTE: GS private method
+	  NSFont *largestFont = [self effectiveFontForGlyphAtIndex: glyphRangeToDraw.location
+							     range: NULL];
 
 	  const CGFloat underlineWidth = [largestFont pointSize] * 
 	    (((type & NSUnderlineStyleDouble) != 0) ? 0.05 : 0.07);
 	  
 	  NSPoint start = [self locationForGlyphAtIndex: glyphRangeToDraw.location];
-	  NSPoint end = [self locationForGlyphAtIndex: NSMaxRange(glyphRangeToDraw) - 1]; //FIXME: check length > 0
+	  NSPoint end = [self locationForGlyphAtIndex: NSMaxRange(glyphRangeToDraw) - 1];
 
 	  // FIXME: remove this hack lowers the underline slightly
 	  start.y += [largestFont pointSize] * 0.07;
@@ -2003,6 +2145,17 @@ static void GSDrawPatternLine(NSPoint start, NSPoint end, NSInteger pattern, CGF
 
 @implementation NSLayoutManager
 
+/*
+ * Class methods
+ */
++ (void) initialize
+{
+  if (self == [NSLayoutManager class])
+    {
+      [self exposeBinding: @"hyphenationFactor"];
+    }
+}
+
 -(void) insertTextContainer: (NSTextContainer *)aTextContainer
 		    atIndex: (unsigned int)index
 {
@@ -2034,6 +2187,9 @@ static void GSDrawPatternLine(NSPoint start, NSPoint end, NSInteger pattern, CGF
 
 -(void) dealloc
 {
+  // Remove all key value bindings for this object.
+  [GSKeyValueBinding unbindAllForObject: self];
+
   DESTROY(_typingAttributes);
   DESTROY(_temporaryAttributes);
   [super dealloc];
