@@ -272,21 +272,7 @@ static BOOL menuBarVisible = YES;
       if (_menu.horizontal == YES)
         {
           NSMutableArray *itemsToMove;
-          NSImage *ti;
-          float bar;
 	  
-	  if([[GSTheme theme] menuShouldShowIcon])
-	    {
-	      ti = [[NSApp applicationIconImage] copy];
-	      if (ti == nil)
-		{
-		  ti = [[NSImage imageNamed: @"GNUstep"] copy];
-		}
-	      [ti setScalesWhenResized: YES];
-	      bar = [NSMenuView menuBarHeight] - 4;
-	      [ti setSize: NSMakeSize(bar, bar)];
-	    }
-
           itemsToMove = [NSMutableArray new];
           
           if (appMenu == nil)
@@ -313,8 +299,18 @@ static BOOL menuBarVisible = YES;
                 }
             }
 
-	  if([[GSTheme theme] menuShouldShowIcon])
+	  if ([[GSTheme theme] menuShouldShowIcon])
 	    {
+              NSImage *ti;
+              float bar;
+	      ti = [[NSApp applicationIconImage] copy];
+	      if (ti == nil)
+		{
+		  ti = [[NSImage imageNamed: @"GNUstep"] copy];
+		}
+	      [ti setScalesWhenResized: YES];
+	      bar = [NSMenuView menuBarHeight] - 4;
+	      [ti setSize: NSMakeSize(bar, bar)];
 	      [appItem setImage: ti];
 	      RELEASE(ti);
 	    }
@@ -581,7 +577,6 @@ static BOOL menuBarVisible = YES;
     {
       [self setVersion: 1];
       nc = [NSNotificationCenter defaultCenter];
-      menuZone = NSCreateZone(0, 0, YES);
     }
 }
 
@@ -914,7 +909,7 @@ static BOOL menuBarVisible = YES;
  */
 - (int) indexOfItem: (id <NSMenuItem>)anObject
 {
-  int index;
+  NSUInteger index;
 
   index = [_items indexOfObjectIdenticalTo: anObject];
 
@@ -1315,18 +1310,17 @@ static BOOL menuBarVisible = YES;
       [_popUpButtonCell selectItemAtIndex: index];
     }
 
-  if ((action = [item action]) != NULL)
+  if (_popUpButtonCell != nil && (action = [_popUpButtonCell action]) != NULL)
     {
       [NSApp sendAction: action
-	     to: [item target]
-	     from: item];
+                     to: [_popUpButtonCell target]
+                   from: [_popUpButtonCell controlView]];
     }
-  else if (_popUpButtonCell != nil)
+  else if ((action = [item action]) != NULL)
     {
-      if ((action = [_popUpButtonCell action]) != NULL)
-	[NSApp sendAction: action
-	       to: [_popUpButtonCell target]
-	       from: [_popUpButtonCell controlView]];
+      [NSApp sendAction: action
+                     to: [item target]
+                   from: item];
     }
 
   [nc postNotificationName: NSMenuDidSendActionNotification
@@ -1417,7 +1411,7 @@ static BOOL menuBarVisible = YES;
   if (_view != nil)
     {
       // remove the old representation
-      [contentView removeSubview: _view];
+      [_view removeFromSuperview];
       [_view setMenu: nil];
     }
 
@@ -2003,6 +1997,16 @@ static BOOL menuBarVisible = YES;
         {
           NSMenuView *newRep;
 
+          if (oldStyle == NSWindows95InterfaceStyle)
+            {
+              /* Remove the menu from all main windows.
+               */
+              [[GSTheme theme] updateAllWindowsWithMenu: nil];
+            }
+          if (newStyle == NSWindows95InterfaceStyle)
+            {
+              [self close];
+            }
           newRep = [[NSMenuView alloc] initWithFrame: NSZeroRect];
           if (newStyle == NSMacintoshInterfaceStyle
               || newStyle == NSWindows95InterfaceStyle)
@@ -2019,43 +2023,30 @@ static BOOL menuBarVisible = YES;
           RELEASE(newRep);
           if (newStyle == NSWindows95InterfaceStyle)
             {
-              /* Put menu in the main window for microsoft style.
+              /* Put menu in all main windows for microsoft style.
                */
-              [[NSApp mainWindow] setMenu: self];
-            }
-          else if ([[NSApp mainWindow] menu] == self)
-            {
-              /* Remove the menu from the main window.
-               */
-              [[NSApp mainWindow] setMenu: nil];
+              [[GSTheme theme] updateAllWindowsWithMenu: self];
             }
         }
       
       /* Adjust the menu window to suit the menu view unless the menu
        * is being displayed in the application main window.
        */
-      if ([[NSApp mainWindow] menu] != self)
+      if (newStyle != NSWindows95InterfaceStyle)
         {
           [[self window] setTitle: [[NSProcessInfo processInfo] processName]];
           [[self window] setLevel: NSMainMenuWindowLevel];
           [self _setGeometry];
           [self sizeToFit];
-        }
-      
-      if ([NSApp isActive])
-        {
-          [self display];
-        }
 
-      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == 
-	  NSWindows95InterfaceStyle)
-	{
-	  [[GSTheme theme] updateAllWindowsWithMenu: self];
-	}
+          if ([NSApp isActive])
+            {
+              [self display];
+            }
+        }
     }
   else 
     {
-      [self close];
       [[self window] setLevel: NSSubmenuWindowLevel];
     }
 }
