@@ -163,6 +163,7 @@ static NSString *commandKeyString = @"#";
       state = GSThemeSelectedState;
     }
 
+  // TODO: Make the color lookup simpler.
   color = [[GSTheme theme] colorNamed: @"NSMenuItem" state: state];
   if (color == nil)
     {
@@ -172,7 +173,7 @@ static NSString *commandKeyString = @"#";
 	}
       else
 	{
-	  color = [NSColor controlBackgroundColor];
+	  color = [[GSTheme theme] menuItemBackgroundColor];
 	}
     }
 
@@ -657,11 +658,7 @@ static NSString *commandKeyString = @"#";
     }
 }
 
-//
-// Drawing.
-//
-- (void) drawBorderAndBackgroundWithFrame: (NSRect)cellFrame
-                                   inView: (NSView *)controlView
+- (GSThemeControlState) themeControlState
 {
   unsigned mask;
   GSThemeControlState state = GSThemeNormalState;
@@ -695,10 +692,19 @@ static NSString *commandKeyString = @"#";
       state = GSThemeSelectedState;
     }
 
+  return state;
+}
+
+//
+// Drawing.
+//
+- (void) drawBorderAndBackgroundWithFrame: (NSRect)cellFrame
+                                   inView: (NSView *)controlView
+{
   [[GSTheme theme] drawBorderAndBackgroundForMenuItemCell: self
                    withFrame: cellFrame
                    inView: controlView
-                   state: state
+                   state: [self themeControlState]
                    isHorizontal: [_menuView isHorizontal]];
 }
 
@@ -753,34 +759,10 @@ static NSString *commandKeyString = @"#";
 - (void) drawSeparatorItemWithFrame:(NSRect)cellFrame
                             inView:(NSView *)controlView
 {
-  NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
-  if (style == NSMacintoshInterfaceStyle
-      || style == NSWindows95InterfaceStyle)
-    {
-      NSBezierPath *path = [NSBezierPath bezierPath];
-      NSPoint start = NSMakePoint(3, cellFrame.size.height/2 + 
-				  cellFrame.origin.y);
-      NSPoint end   = NSMakePoint(cellFrame.size.width - 3, 
-				  cellFrame.size.height/2 + 
-				  cellFrame.origin.y);
-
-      [[NSColor blackColor] set];
-
-      [path moveToPoint: start];
-      [path lineToPoint: end];
-
-      [path stroke];
-      /*
-      NSRect lineFrame = NSMakeRect(cellFrame.origin.x,
-				    cellFrame.origin.y 
-				    + cellFrame.size.height/2,
-				    cellFrame.size.width,
-				    1);
-      NSBox *line = [[NSBox alloc] initWithFrame: lineFrame];
-      [controlView addSubview:line];
-      RELEASE(line);
-      */
-    }
+  [[GSTheme theme] drawSeparatorItemForMenuItemCell: self
+                                          withFrame: cellFrame
+                                             inView: controlView
+                                       isHorizontal: [_menuView isHorizontal]]; 
 }
 
 - (void) drawStateImageWithFrame: (NSRect)cellFrame
@@ -816,8 +798,11 @@ static NSString *commandKeyString = @"#";
 - (void) drawTitleWithFrame: (NSRect)cellFrame
                      inView: (NSView *)controlView
 {
-  [self _drawText: [_menuItem title]
-        inFrame: [self titleRectForBounds: cellFrame]];
+  [[GSTheme theme] drawTitleForMenuItemCell: self 
+                                  withFrame: cellFrame
+                                     inView: controlView
+                                      state: [self themeControlState]
+                               isHorizontal: [_menuView isHorizontal]];
 }
 
 - (void) _drawBorderAndBackgroundWithFrame: (NSRect)cellFrame inView: (NSView*)controlView
