@@ -86,6 +86,17 @@ static NSMapTable *viewInfo = 0;
 /* A menu's title is an instance of this class */
 @class NSButton;
 
+@interface NSMenu (Private)
+- (void) _attachMenu: (NSMenu*)aMenu;
+@end
+
+@implementation NSMenu (Private)
+- (void) _attachMenu: (NSMenu*)aMenu
+{
+  _attachedMenu = aMenu;
+}
+@end
+
 @interface NSMenuView (Private)
 - (BOOL) _rootIsHorizontal: (BOOL*)isAppMenu;
 @end
@@ -548,6 +559,8 @@ static NSMapTable *viewInfo = 0;
   else
     {
       [attachedMenu close];
+      // Unselect the active item
+      [self setHighlightedItemIndex: -1];
     }
 }
 
@@ -577,7 +590,11 @@ static NSMapTable *viewInfo = 0;
   else
     {
       NSDebugLLog (@"NSMenu",  @"Will open normal: %@", attachableMenu);
-      [attachableMenu display];
+      // FIXME: Only resize when needed
+      [attachableMenu sizeToFit];
+      [[attachableMenu window] setFrameOrigin: [self locationForSubmenu: attachableMenu]];
+      [_attachedMenu _attachMenu: attachableMenu];
+      [[attachableMenu window] orderFrontRegardless];
     }
 }
 
@@ -1093,7 +1110,11 @@ static NSMapTable *viewInfo = 0;
   int items = [_itemCells count];
   BOOL growHeight = YES;
   BOOL resizeCell = NO;
-  BOOL resizeScreenRect = NO;
+  CGFloat borderOffsetInBaseCoords;
+
+  // Our window needs to have a nonzero size for the
+  // -convertRect:fromView: and relatead methods to work.
+  [_window setFrame: NSMakeRect(0,0,1,1) display: NO];
 
   // Make sure the menu entries are up to date
   [self update];
@@ -1130,37 +1151,32 @@ static NSMapTable *viewInfo = 0;
             }
         }
     }
-  
-  // Convert the screen rect to our view
-  cellFrame.size = screenRect.size;
-  cellFrame.origin = [_window convertScreenToBase: screenRect.origin];
-  cellFrame = [self convertRect: cellFrame fromView: nil];
- 
+
+  cellFrame = screenRect;
+
   /*
     we should have the calculated cell size, grow the width
     if needed to match the screenRect and vice versa
   */
-  if (cellFrame.size.width > _cellSize.width) 
+  if (cellFrame.size.width > [self convertSizeToBase: _cellSize].width) 
     {
-      _cellSize.width = cellFrame.size.width;
+      _cellSize.width = [self convertSizeFromBase: cellFrame.size].width;
       resizeCell = YES;
     }
   else
     {
-      cellFrame.size.width = _cellSize.width;
-      resizeScreenRect = YES;
+      cellFrame.size.width = [self convertSizeToBase: _cellSize].width;
     }
 
   /* certain pop-ups don't want the height resized */
-  if (growHeight && cellFrame.size.height > _cellSize.height) 
+  if (growHeight && cellFrame.size.height > [self convertSizeToBase: _cellSize].height) 
     {
-      _cellSize.height = cellFrame.size.height;
+      _cellSize.height = [self convertSizeFromBase: cellFrame.size].height;
       resizeCell = YES;
     }
   else
     {
-      cellFrame.size.height = _cellSize.height;
-      resizeScreenRect = YES;
+      cellFrame.size.height = [self convertSizeToBase: _cellSize].height;
     }
 
   /*
@@ -1170,16 +1186,13 @@ static NSMapTable *viewInfo = 0;
   if (resizeCell)
     [self sizeToFit];
 
-  if (resizeScreenRect)
-    {
-      cellFrame = [self convertRect: cellFrame toView: nil];
-    }
-
   /*
-   * Compute the frame
+   * Compute the frame. popUpFrame is in screen coordinates
    */
-  popUpFrame.origin = screenRect.origin;
-  popUpFrame.size = cellFrame.size;
+  popUpFrame = cellFrame;
+
+  borderOffsetInBaseCoords = [self convertSizeToBase: NSMakeSize(_leftBorderOffset, 0)].width;
+   
   if (items > 0)
     {
       float f;
@@ -1187,10 +1200,10 @@ static NSMapTable *viewInfo = 0;
       if (_horizontal == NO)
         {
           f = cellFrame.size.height * (items - 1);
-          popUpFrame.size.height += f + _leftBorderOffset;
+          popUpFrame.size.height += f + borderOffsetInBaseCoords;
           popUpFrame.origin.y -= f;
-          popUpFrame.size.width += _leftBorderOffset;
-          popUpFrame.origin.x -= _leftBorderOffset;
+          popUpFrame.size.width += borderOffsetInBaseCoords;
+          popUpFrame.origin.x -= borderOffsetInBaseCoords;
 
 	  // If the menu is a pull down menu the first item, which would
 	  // appear at the top of the menu, holds the title and is omitted
@@ -1314,11 +1327,19 @@ static NSMapTable *viewInfo = 0;
     }
 
   // Get the frameRect
-  r = [NSWindow frameRectForContentRect: popUpFrame
-                styleMask: [_window styleMask]];
-  
-  // Set the window frame
-  [_window setFrame: r display: NO]; 
+  {
+    NSSize contentSize = [self convertSizeFromBase: popUpFrame.size];
+    NSRect contentRect = NSMakeRect(popUpFrame.origin.x,
+				    popUpFrame.origin.y,
+				    contentSize.width,
+				    contentSize.height);
+    r = [_window frameRectForContentRect: contentRect];
+   
+
+    // Set the window frame. r should be identical to popUpFrame except with 
+    // any borders the window wanted to add.
+    [_window setFrame: r display: NO];
+  }
 }
 
 /*
@@ -1392,13 +1413,6 @@ static NSMapTable *viewInfo = 0;
   NSInterfaceStyle style =
     NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self);
   
-  /*If we have menu in window, close the menu after select
-    an option*/
-  if (style == NSWindows95InterfaceStyle)
-    {
-      [[[[NSApp mainWindow] menu] attachedMenu] close];
-    }
-  
   if (indexOfActionToExecute >= 0
       && [_attachedMenu attachedMenu] != nil && [_attachedMenu attachedMenu] ==
       [[_items_link objectAtIndex: indexOfActionToExecute] submenu])
@@ -1429,6 +1443,7 @@ static NSMapTable *viewInfo = 0;
 }
 
 - (BOOL) _trackWithEvent: (NSEvent*)event
+        startingMenuView: (NSMenuView*)mainWindowMenuView
 {
   unsigned eventMask = NSPeriodicMask;
   NSDate *theDistantFuture = [NSDate distantFuture];
@@ -1441,7 +1456,6 @@ static NSMapTable *viewInfo = 0;
   int firstIndex = -1;
   NSEvent *original;
   NSEventType type;
-  NSEventType end;
 
   /*
    * The original event is unused except to determine whether the method
@@ -1453,40 +1467,41 @@ static NSMapTable *viewInfo = 0;
 
   type = [event type];
 
-  if (type == NSRightMouseDown || type == NSRightMouseDragged)
-    {
-      end = NSRightMouseUp;
-      eventMask |= NSRightMouseUpMask | NSRightMouseDraggedMask;
-      eventMask |= NSRightMouseDownMask;
-    }
-  else if (type == NSOtherMouseDown || type == NSOtherMouseDragged)
-    {
-      end = NSOtherMouseUp;
-      eventMask |= NSOtherMouseUpMask | NSOtherMouseDraggedMask;
-      eventMask |= NSOtherMouseDownMask;
-    }
-  else if (type == NSLeftMouseDown || type == NSLeftMouseDragged)
-    {
-      end = NSLeftMouseUp;
-      eventMask |= NSLeftMouseUpMask | NSLeftMouseDraggedMask;
-      eventMask |= NSLeftMouseDownMask;
+  /**
+   * The following event tracking run loop can run for a long time, during
+   * which new windows (for submenus) are created and put on screen. The 
+   * window manager might not place the menus where we ask them to be placed 
+   * (e.g. Metacity will move menus so they don't overlap the GNOME 
+   * top-of-screen panel).
+   *
+   * The mechanism which updates an NSWindow's frame when an external force 
+   * (such as the window manager) moves a window is an NSAppKitDefined event.
+   * Since we need the frames of the menu windows to be accurate to know which 
+   * menu item the cursor is actually over, and since we are running our own 
+   * event loop and the NSAppKitDefined events aren't handled for us, we need
+   * to request them ourselves and forward them to the relevant window.
+   *
+   * NOTE: While it seems messy to have to handle these events, Cocoa doesn't
+   * handle them automatically either for code which goes in to an event
+   * tracking loop.
+   */
+  eventMask |= NSAppKitDefinedMask;
 
-      /*We need know if the user press a modifier key to close the menu
-	when the menu is in a window*/
-      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
-	  NSWindows95InterfaceStyle)
-	{
-	  eventMask |= NSFlagsChangedMask;
-	}
-    }
-  else
+  eventMask |= NSRightMouseUpMask | NSRightMouseDraggedMask;
+  eventMask |= NSRightMouseDownMask;
+  eventMask |= NSOtherMouseUpMask | NSOtherMouseDraggedMask;
+  eventMask |= NSOtherMouseDownMask;
+  eventMask |= NSLeftMouseUpMask | NSLeftMouseDraggedMask;
+  eventMask |= NSLeftMouseDownMask;
+  
+  /*We need know if the user press a modifier key to close the menu
+    when the menu is in a window*/
+  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
+      NSWindows95InterfaceStyle)
     {
-      NSLog (@"Unexpected event: %d during event tracking in NSMenuView", type);
-      end = NSLeftMouseUp;
-      eventMask |= NSLeftMouseUpMask | NSLeftMouseDraggedMask;
-      eventMask |= NSLeftMouseDownMask;
+      eventMask |= NSFlagsChangedMask;
     }
-
+  
   /* We attempt to mimic Mac OS X behavior for pop up menus. If the user
      presses the mouse button over a pop up button and then drags the mouse
      over the menu, the menu is closed when the user releases the mouse. On the
@@ -1504,20 +1519,22 @@ static NSMapTable *viewInfo = 0;
     }
   do
     {
-      /*Close the menu if the user press a modifier key and menu
-	is in a window*/
-      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
-	  NSWindows95InterfaceStyle && type == NSFlagsChanged)
+      /* Close the menu if the user press a modifier key and menu
+         is in a window */
+      if (mainWindowMenuView != nil && type == NSFlagsChanged)
 	{
 	  [self setHighlightedItemIndex: -1];
-	  [[[[NSApp mainWindow] menu] attachedMenu] close];
+	  [[[mainWindowMenuView menu] attachedMenu] close];
 	  return NO;
 	}
 
-      if (type == end)
+      if (type == NSLeftMouseUp ||
+	  type == NSRightMouseUp ||
+	  type == NSOtherMouseUp)
         {
           shouldFinish = YES;
         }
+
       if (type == NSPeriodic || event == original)
         {
           NSPoint location;
@@ -1555,7 +1572,6 @@ static NSMapTable *viewInfo = 0;
                 [_attachedMenu shiftOnScreen];
             }
 
-
           /*
            * 2 - Check if we have to reset the justAttachedNewSubmenu
            * flag to NO.
@@ -1613,22 +1629,22 @@ static NSMapTable *viewInfo = 0;
                 && NSMouseInRect (locationInScreenCoordinates,
                   [[candidateMenu window] frame], NO))
                 {
-                  BOOL        candidateMenuResult;
+                  BOOL candidateMenuResult;
+                  NSMenuView *subMenuView = [[candidateMenu attachedMenu] menuRepresentation];
 
                   // The call to fetch attachedMenu is not needed. But putting
                   // it here avoids flicker when we go back to an ancestor 
                   // menu and the attached menu is already correct.
-                  [[[candidateMenu attachedMenu] menuRepresentation]
-                    detachSubmenu];
+                  [subMenuView detachSubmenu];
                   
                   // Reset highlighted index for this menu.
                   // This way if we return to this submenu later there 
                   // won't be a highlighted item.
-                  [[[candidateMenu attachedMenu] menuRepresentation]
-                    setHighlightedItemIndex: -1];
+                  [subMenuView setHighlightedItemIndex: -1];
                   
                   candidateMenuResult = [[candidateMenu menuRepresentation]
-                           _trackWithEvent: original];
+                                          _trackWithEvent: original
+                                          startingMenuView: mainWindowMenuView];
                   return candidateMenuResult;
                 }
 
@@ -1642,7 +1658,8 @@ static NSMapTable *viewInfo = 0;
                   BOOL subMenuResult;
 
                   subMenuResult
-                    = [[self attachedMenuView] _trackWithEvent: original];
+                    = [[self attachedMenuView] _trackWithEvent: original
+                                              startingMenuView: mainWindowMenuView];
                   if (subMenuResult
                     && wasTransient == [_attachedMenu isTransient])
                     {
@@ -1651,18 +1668,13 @@ static NSMapTable *viewInfo = 0;
                   return subMenuResult;
                 }
 	      
-	      /*We track the menu correctly when this is located
-		in a window*/
-	      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
-		  NSWindows95InterfaceStyle &&
-		  ![[self menu] isTransient] &&
-		  ![[self menu] _ownedByPopUp])
+	      /* We track the menu correctly when this is located
+		in a window */
+	      if (mainWindowMenuView != nil)
 		{
                   // If the user moves the mouse into the main window
                   // horizontal menu, start tracking again.
-                  NSWindow *mainWindow = [NSApp mainWindow];
-                  NSMenuView *mainWindowMenuView = [[mainWindow menu]
-                    menuRepresentation];
+                  NSWindow *mainWindow = [mainWindowMenuView window];
                   NSPoint locationInMainWindow = [mainWindow 
                     convertScreenToBase: locationInScreenCoordinates];
 		  if ([mainWindowMenuView 
@@ -1677,7 +1689,8 @@ static NSMapTable *viewInfo = 0;
                         {
 		          [self setHighlightedItemIndex: -1];
 		          return [mainWindowMenuView
-			       _trackWithEvent: original];
+                                   _trackWithEvent: original
+                                   startingMenuView: mainWindowMenuView];
                         }
 		    }
 		}
@@ -1703,13 +1716,23 @@ static NSMapTable *viewInfo = 0;
           lastLocation = location;
         }
 
-      event = [NSApp nextEventMatchingMask: eventMask
-        untilDate: theDistantFuture
-        inMode: NSEventTrackingRunLoopMode
-        dequeue: YES];
-      type = [event type];
+      do
+	{
+	  event = [NSApp nextEventMatchingMask: eventMask
+				     untilDate: theDistantFuture
+					inMode: NSEventTrackingRunLoopMode
+				       dequeue: YES];
+	  type = [event type];
+	  if (type == NSAppKitDefined)
+	    {
+	      [[event window] sendEvent: event];
+	    }
+	}
+      while (type == NSAppKitDefined);
     }
-  while (type != end || shouldFinish == NO);
+  while ((type != NSLeftMouseUp &&
+	  type != NSRightMouseUp &&
+	  type != NSOtherMouseUp) || shouldFinish == NO);
 
   /*
    * Ok, we released the mouse
@@ -1776,8 +1799,19 @@ static NSMapTable *viewInfo = 0;
   // Before executing the action, uncapture the mouse
   [_window _releaseMouse: self];
 
-  if([self _executeItemAtIndex: indexOfActionToExecute
-	   removeSubmenu: subMenusNeedRemoving] == NO)
+  /* If we have menu in window, close the menu after select
+    an option */
+  if (mainWindowMenuView != nil)
+    {
+      if (self != mainWindowMenuView)
+        {
+          [mainWindowMenuView setHighlightedItemIndex: -1];
+        }
+      [[[mainWindowMenuView menu] attachedMenu] close];
+    }
+
+  if ([self _executeItemAtIndex: indexOfActionToExecute
+                  removeSubmenu: subMenusNeedRemoving] == NO)
     {
       return NO;
     }
@@ -1800,12 +1834,22 @@ static NSMapTable *viewInfo = 0;
 - (BOOL) trackWithEvent: (NSEvent*)event
 {
   BOOL result = NO;
+  NSMenuView *mainWindowMenuView = nil;
+
+  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
+      NSWindows95InterfaceStyle &&
+      ![[self menu] isTransient] &&
+      ![[self menu] _ownedByPopUp])
+    {
+      mainWindowMenuView = self;
+    }
 
   // Capture the mouse so we get clicks outside the menus and
   // GNUstep windows.
   [_window _captureMouse: self];
   NS_DURING
-    result = [self _trackWithEvent: event];
+    result = [self _trackWithEvent: event
+                  startingMenuView: mainWindowMenuView];
   NS_HANDLER
     [_window _releaseMouse: self];
     [localException raise];
@@ -1869,6 +1913,11 @@ static NSMapTable *viewInfo = 0;
   [self mouseDown: theEvent];
 }
 
+- (void) otherMouseDown: (NSEvent*) theEvent
+{
+  [self mouseDown: theEvent];
+}
+
 - (BOOL) performKeyEquivalent: (NSEvent *)theEvent
 {
   return [_attachedMenu performKeyEquivalent: theEvent];
