@@ -38,23 +38,20 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <AppKit/NSNib.h>
-#include <AppKit/NSNibLoading.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSURL.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSArchiver.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSException.h>
+#import "config.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSArchiver.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSURL.h>
 
-#include "GNUstepGUI/GSModelLoaderFactory.h"
-#include "GNUstepGUI/GSGormLoading.h"
-#include "GNUstepGUI/IMLoading.h"
+#import "AppKit/NSNib.h"
+#import "AppKit/NSNibLoading.h"
+#import "GNUstepGUI/GSModelLoaderFactory.h"
 
 @implementation NSNib
 
@@ -65,48 +62,17 @@
   NS_DURING
     {
       NSString *newFileName = [GSModelLoaderFactory supportedModelFileAtPath: fileName];
-      ASSIGN(_nibData, [NSData dataWithContentsOfFile: newFileName]);
-      ASSIGN(_loader, [GSModelLoaderFactory modelLoaderForFileName: newFileName]);
-      NSDebugLog(@"Loaded data from %@...",newFileName);
+      ASSIGN(_loader, [GSModelLoaderFactory modelLoaderForFileType: [newFileName pathExtension]]);
+      ASSIGN(_nibData, [_loader dataForFile: newFileName]);
+      NSDebugLog(@"Loaded data from %@...", newFileName);
     }
   NS_HANDLER
     {
-      NSLog(@"Exception occured while loading model: %@",[localException reason]);
+      NSLog(@"Exception occured while loading model: %@", [localException reason]);
     }
   NS_ENDHANDLER
 }
 
-- (NSDictionary *) _copyTable: (NSDictionary *)dict
-{
-  NSMutableDictionary *ctx = nil;
-
-  if (dict != nil)
-    {
-      id obj = nil;
-
-      // copy the dictionary...
-      ctx = [NSMutableDictionary dictionaryWithDictionary: dict];
-
-      // remove and set the owner...
-      obj = [ctx objectForKey: @"NSNibOwner"];
-      if (obj != nil)
-	{
-	  [ctx removeObjectForKey: @"NSNibOwner"];
-	  [ctx setObject: obj forKey: @"NSOwner"];
-	}
-
-      // Remove and set the top level objects...
-      obj = [ctx objectForKey: @"NSNibTopLevelObjects"];
-      if (obj != nil)
-	{
-	  [ctx removeObjectForKey: @"NSNibTopLevelObjects"];
-	  [ctx setObject: obj forKey: @"NSTopLevelObjects"];
-	}
-    }
-
-  return ctx;
-}
-
 // Public methods...
 
 /**
@@ -118,9 +84,27 @@
 {
   if ((self = [super init]) != nil)
     {
-      // load the nib data into memory...
-      _nibData = [NSData dataWithContentsOfURL: nibFileURL];
       ASSIGN(_url, nibFileURL);
+
+      if ([nibFileURL isFileURL])
+        {
+          [self _readNibData: [nibFileURL path]];
+        }
+      else
+        {
+          NS_DURING
+            {
+              ASSIGN(_loader, [GSModelLoaderFactory modelLoaderForFileType: 
+                                                      [[nibFileURL path] pathExtension]]);
+              // load the nib data into memory...
+              _nibData = [NSData dataWithContentsOfURL: nibFileURL];
+            }
+          NS_HANDLER
+            {
+              NSLog(@"Exception occured while loading model: %@", [localException reason]);
+            }
+          NS_ENDHANDLER
+        }
     }
   return self;
 }
@@ -136,6 +120,9 @@
     {
       NSString *fileName = nil;
 
+      // Keep the bundle for resource creation
+      ASSIGN(_bundle, bundle);
+
       if (bundle == nil)
 	{
 	  bundle = [NSBundle mainBundle];
@@ -143,6 +130,11 @@
 
       // initialize the bundle...
       fileName = [bundle pathForNibResource: nibNamed];
+      if (fileName == nil)
+        {
+          DESTROY(self);
+          return nil;
+        }
 
       // load the nib data into memory...
       [self _readNibData: fileName];
@@ -186,12 +178,12 @@
   NSMutableDictionary *externalNameTable = [NSMutableDictionary dictionary];
 
   // add the necessary things to the table...
-  [externalNameTable setObject: owner forKey: @"NSNibOwner"];
+  [externalNameTable setObject: owner forKey: NSNibOwner];
 
   if (topLevelObjects != 0)
     {
       *topLevelObjects = [NSMutableArray array];
-      [externalNameTable setObject: *topLevelObjects forKey: @"NSNibTopLevelObjects"];
+      [externalNameTable setObject: *topLevelObjects forKey: NSNibTopLevelObjects];
     }
 
   return [self instantiateNibWithExternalNameTable: externalNameTable]; 
@@ -243,6 +235,7 @@
 {
   RELEASE(_nibData);
   RELEASE(_loader);
+  TEST_RELEASE(_bundle);
   TEST_RELEASE(_url);
   [super dealloc];
 }
