@@ -100,7 +100,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 - (BOOL) _shouldShowExtension: (NSString *)extension
 {
   if (_canChooseFiles == NO ||
-      (_fileTypes != nil && [_fileTypes containsObject: extension] == NO))
+      (_allowedFileTypes != nil && [_allowedFileTypes containsObject: extension] == NO))
     return NO;
 
   return YES;
@@ -113,87 +113,40 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   if (column == -1)
     return;
 
+  if (_delegateHasSelectionDidChange) 
+    {
+      [_delegate panelSelectionDidChange: self];
+    }
+
   matrix = [_browser matrixInColumn: column];
 
-  if ([_browser allowsMultipleSelection])
+  // Validate selection
+
+  BOOL selectionValid = YES;
+
+  NSArray *selectedCells = [matrix selectedCells];
+  NSEnumerator *e = [selectedCells objectEnumerator];
+  id o;
+  while ((o = [e nextObject]) != nil)
     {
-      NSArray  *selectedCells;
-
-      selectedCells = [matrix selectedCells];
-
-      if ([selectedCells count] <= 1)
+      if ((![o isLeaf] && !_canChooseDirectories) ||
+	  ([o isLeaf] && !_canChooseFiles))
 	{
-	  if (_canChooseDirectories == NO ||
-	     [[matrix selectedCell] isLeaf] == YES)
-	    [super _selectTextInColumn: column];
-	  else
-	    [self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
-	}
-      else
-	{
-	  [_form abortEditing];
-	  [[_form cellAtIndex: 0] setStringValue: @""];
-	  [_form setNeedsDisplay: YES];
-	  [_okButton setEnabled: YES];
+	  selectionValid = NO;
 	}
     }
+  [_okButton setEnabled: selectionValid];
+  
+  // Set form label
+
+  if ([selectedCells count] > 1)
+    {
+      [[_form cellAtIndex: 0] setStringValue: @""];
+    }
   else
     {
-      if (_canChooseDirectories == NO || [[matrix selectedCell] isLeaf] == YES)
-	[super _selectTextInColumn: column];
-      else
-	[self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
-    }
-}
-
-- (void) _selectCellName: (NSString *)title
-{
-  NSString           *cellString;
-  NSArray            *cells;
-  NSMatrix           *matrix;
-  NSComparisonResult  result;
-  NSRange             range;
-  int                 i, titleLength, cellLength, numberOfCells;
-
-  matrix = [_browser matrixInColumn: [_browser lastColumn]];
-  if ([matrix selectedCell])
-    return;
-
-  titleLength = [title length];
-  if (!titleLength)
-    {
-      [_okButton setEnabled: _canChooseDirectories];
-      return;
-    }
-
-  range.location = 0;
-  range.length = titleLength;
-
-  cells = [matrix cells];
-  numberOfCells = [cells count];
-
-  for (i = 0; i < numberOfCells; i++)
-    {
-      cellString = [[matrix cellAtRow: i column: 0] stringValue];
-
-      cellLength = [cellString length];
-      if (cellLength < titleLength)
-	continue;
-
-      result = [self _compareFilename: [cellString substringWithRange: range]
-                                 with: title];
-
-      if (result == NSOrderedSame)
-	{
-	  [matrix selectCellAtRow: i column: 0];
-	  [matrix scrollCellToVisibleAtRow: i column: 0];
-	  [_okButton setEnabled: YES];
-	  return;
-	}
-      else if (result == NSOrderedDescending)
-	break;
-    }
-  [_okButton setEnabled: NO];
+      [[_form cellAtIndex: 0] setStringValue: [[matrix selectedCell] stringValue]];
+    }  
 }
 
 - (void) _setupForDirectory: (NSString *)path file: (NSString *)filename
@@ -211,16 +164,6 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   [super _setupForDirectory: path file: filename];
 }
 
-- (void) _setupForTypes: (NSArray *)fileTypes
-{
-  if (_fileTypes != fileTypes)
-    {
-      BOOL reload = ![_fileTypes isEqual: fileTypes];
-      ASSIGN (_fileTypes, fileTypes);
-      if (reload)
-	[self _reloadBrowser];
-    }
-}
 @end
 
 /** 
@@ -272,12 +215,6 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   return _gs_gui_open_panel;
 }
 
-- (void) dealloc
-{
-  TEST_RELEASE(_fileTypes);
-  [super dealloc];
-}
-
 - (id) init
 {
   self = [super init];
@@ -448,7 +385,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
    types.</p>
    <p>See Also: -runModalForDirectory:file:types: and -allowedFileTypes</p>
 */
-- (int) runModal
+- (NSInteger) runModal
 {
   return [self runModalForTypes: [self allowedFileTypes]];
 }
@@ -457,7 +394,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
    (or last selected), and filtering for files that have the specified types.</p>
    <p>See Also: -runModalForDirectory:file:types:</p>
 */
-- (int) runModalForTypes: (NSArray *)fileTypes
+- (NSInteger) runModalForTypes: (NSArray *)fileTypes
 {
   return [self runModalForDirectory: nil
 			       file: @""
@@ -470,21 +407,21 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
     the open panel is the last directory selected.</p>
     <p>See Also: -runModalForTypes:</p>
 */
-- (int) runModalForDirectory: (NSString *)path
-			file: (NSString *)name
-		       types: (NSArray *)fileTypes
+- (NSInteger) runModalForDirectory: (NSString *)path
+                              file: (NSString *)name
+                             types: (NSArray *)fileTypes
 {
-  [self _setupForTypes: fileTypes];
+  [self setAllowedFileTypes: fileTypes];
   return [self runModalForDirectory: path 
 			       file: name];  
 }
 
-- (int) runModalForDirectory: (NSString *)path
-			file: (NSString *)name
-		       types: (NSArray *)fileTypes
-	    relativeToWindow: (NSWindow*)window
+- (NSInteger) runModalForDirectory: (NSString *)path
+                              file: (NSString *)name
+                             types: (NSArray *)fileTypes
+                  relativeToWindow: (NSWindow*)window
 {
-  [self _setupForTypes: fileTypes];
+  [self setAllowedFileTypes: fileTypes];
   return [self runModalForDirectory: path 
 			       file: name
 		   relativeToWindow: window];
@@ -507,6 +444,25 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 		   contextInfo: contextInfo];
 }
 
+- (void) beginForDirectory: (NSString *)path
+                      file: (NSString *)filename
+                     types: (NSArray *)fileTypes
+          modelessDelegate: (id)modelessDelegate
+            didEndSelector: (SEL)didEndSelector
+               contextInfo: (void *)contextInfo
+{
+  // FIXME: This should be modeless
+  [self _setupForTypes: fileTypes];
+  [self _setupForDirectory: path file: filename];
+  if ([filename length] > 0)
+    [_okButton setEnabled: YES];
+  [NSApp beginSheet: self
+	 modalForWindow: nil
+	 modalDelegate: modelessDelegate
+	 didEndSelector: didEndSelector
+	 contextInfo: contextInfo];
+}
+
 - (void) ok: (id)sender
 {
   NSMatrix      *matrix = nil;
