@@ -59,6 +59,7 @@
 #import "AppKit/NSTextFieldCell.h"
 #import "AppKit/NSWindow.h"
 
+#import "GSGuiPrivate.h"
 #include <math.h>
 
 static NSNotificationCenter *nc = nil;
@@ -858,6 +859,52 @@ static NSImage *unexpandable  = nil;
   [super mouseDown: theEvent];
 }
 
+- (void)keyDown: (NSEvent*)event
+{
+   NSString *characters = [event characters];
+
+   if ([characters length] == 1)
+     {
+       unichar c = [characters characterAtIndex: 0];
+
+       NSIndexSet *selected = [self selectedRowIndexes];
+       NSInteger i;
+       for (i = [selected firstIndex]; i != NSNotFound; i = [selected indexGreaterThanIndex: i])
+	 {
+	   id item = [self itemAtRow: i];
+	   switch (c)
+	     {
+	     case NSLeftArrowFunctionKey:
+	       {
+		 if ([self isItemExpanded: item])
+		   {
+		     [self collapseItem: item];
+		   }
+		 else
+		   {
+		     id parent = [self parentForItem: item];
+		     if (parent != nil)
+		       {
+			 NSInteger parentRow = [self rowForItem: parent];
+			 [self selectRow: parentRow
+			       byExtendingSelection: NO];
+			 [self scrollRowToVisible: parentRow];
+		       }
+		   }
+		 return;
+	       }
+	     case NSRightArrowFunctionKey:
+	       [self expandItem: item];
+	       return;
+	     default:
+	       break;
+	     }
+	 }
+     }
+ 
+   [super keyDown: event];
+}
+
 /*
  * Drawing
  */
@@ -1242,9 +1289,9 @@ Also returns the child index relative to this parent. */
   /* _bounds.origin is (0, 0) when the outline view is not clipped.
    * When the view is scrolled, _bounds.origin.y returns the scrolled height. */
   verticalQuarterPosition =
-    rint(((p.y + _bounds.origin.y) / _rowHeight) * 4.);
+    GSRoundTowardsInfinity(((p.y + _bounds.origin.y) / _rowHeight) * 4.);
   horizontalHalfPosition =
-    rint(((p.x + _bounds.origin.y) / _indentationPerLevel) * 2.);
+    GSRoundTowardsInfinity(((p.x + _bounds.origin.y) / _indentationPerLevel) * 2.);
 
   /* We add an extra quarter to shift the insertion row below the hovered row. */
   row = (verticalQuarterPosition + 1) / 4;
@@ -1283,7 +1330,7 @@ Also returns the child index relative to this parent. */
     {
       NSInteger minInsertionLevel = levelAfter;
       NSInteger maxInsertionLevel = levelBefore;
-      NSInteger pointerInsertionLevel = rint((float)horizontalHalfPosition / 2.);
+      NSInteger pointerInsertionLevel = GSRoundTowardsInfinity((float)horizontalHalfPosition / 2.);
 
       /* Save positions to avoid executing this code when the general
        * position of the mouse is unchanged.
@@ -1450,9 +1497,9 @@ Also returns the child index relative to this parent. */
   if ([_dataSource respondsToSelector:
                     @selector(outlineView:namesOfPromisedFilesDroppedAtDestination:forDraggedItems:)])
     {
-      int count = [_selectedRows count];
+      NSUInteger count = [_selectedRows count];
       NSMutableArray *itemArray = [NSMutableArray arrayWithCapacity: count];
-      unsigned int index = [_selectedRows firstIndex];
+      NSUInteger index = [_selectedRows firstIndex];
       
       while (index != NSNotFound)
         {
@@ -1791,9 +1838,9 @@ Also returns the child index relative to this parent. */
 - (BOOL) _writeRows: (NSIndexSet *)rows
        toPasteboard: (NSPasteboard *)pboard
 {
-  int count = [rows count];
+  NSUInteger count = [rows count];
   NSMutableArray *itemArray = [NSMutableArray arrayWithCapacity: count];
-  unsigned int index = [rows firstIndex];
+  NSUInteger index = [rows firstIndex];
 
   while (index != NSNotFound)
     {
@@ -2002,7 +2049,7 @@ Also returns the child index relative to this parent. */
 
 - (void)_closeItem: (id)item
 {
-  int i, numChildren;
+  NSUInteger i, numChildren;
   NSMutableArray *removeAll = [NSMutableArray array];
 
   [self _collectItemsStartingWith: item into: removeAll];
@@ -2026,7 +2073,8 @@ Also returns the child index relative to this parent. */
 
 - (void)_openItem: (id)item
 {
-  int i, insertionPoint, numChildren, numDescendants;
+  NSUInteger insertionPoint, numChildren, numDescendants;
+  NSInteger i;
   id object;
   id sitem = (item == nil) ? (id)[NSNull null] : (id)item;
 
@@ -2063,15 +2111,16 @@ Also returns the child index relative to this parent. */
       // Add all of the children...
       if ([self isItemExpanded: child])
         {
-          int i, numItems;
+          NSUInteger numItems;
+          NSInteger j;
           NSMutableArray *insertAll = [NSMutableArray array];
 
           [self _collectItemsStartingWith: child into: insertAll];
           numItems = [insertAll count];
           numDescendants += numItems;
-          for (i = numItems-1; i >= 0; i--)
+          for (j = numItems-1; j >= 0; j--)
             {
-              [_items insertObject: [insertAll objectAtIndex: i]
+              [_items insertObject: [insertAll objectAtIndex: j]
                       atIndex: insertionPoint];
             }
         }
@@ -2085,7 +2134,7 @@ Also returns the child index relative to this parent. */
 
 - (void) _removeChildren: (id)startitem
 {
-  int i, numChildren;
+  NSUInteger i, numChildren;
   id sitem = (startitem == nil) ? (id)[NSNull null] : (id)startitem;
   NSMutableArray *anarray;
 
@@ -2107,7 +2156,7 @@ Also returns the child index relative to this parent. */
 - (void) _noteNumberOfRowsChangedBelowItem: (id)item by: (int)numItems
 {
   BOOL selectionDidChange = NO;
-  int rowIndex, nextIndex;
+  NSUInteger rowIndex, nextIndex;
 
   // check for trivial case
   if (numItems == 0)
@@ -2159,8 +2208,8 @@ Also returns the child index relative to this parent. */
 	      /* If the item at _selectedRow was removed, we arbitrarily choose
 	       * another selected item (if there is still any). The policy
 	       * implemented below chooses the index most close to item. */
-	      int r1 = [_selectedRows indexLessThanIndex: rowIndex];
-	      int r2 = [_selectedRows indexGreaterThanOrEqualToIndex: rowIndex];
+	      NSUInteger r1 = [_selectedRows indexLessThanIndex: rowIndex];
+	      NSUInteger r2 = [_selectedRows indexGreaterThanOrEqualToIndex: rowIndex];
 	      if (r1 != NSNotFound && r2 != NSNotFound)
 		{
 		  _selectedRow = (rowIndex - r1) <= (r2 - rowIndex) ? r1 : r2;
