@@ -60,7 +60,27 @@
 			   backing: NSBackingStoreBuffered
 			     defer: NO];
 }
+- (id) initWithContentRect: (NSRect)contentRect
+                 styleMask: (unsigned int)aStyle
+                   backing: (NSBackingStoreType)bufferingType
+                     defer: (BOOL)flag
+{
+  self = [super initWithContentRect: contentRect
+			  styleMask: aStyle
+			    backing: bufferingType
+			      defer: flag];
+  if (nil == self)
+    {
+      return nil;
+    }
 
+  if ((_styleMask & NSUtilityWindowMask) == NSUtilityWindowMask)
+    {
+      [self setFloatingPanel: YES];
+    }
+
+  return self;
+}
 - (void) _initDefaults
 {
   [super _initDefaults];
@@ -185,7 +205,10 @@
 {
   BOOL	flag;
 
-  [super initWithCoder: aDecoder];
+  self = [super initWithCoder: aDecoder];
+  if (nil == self)
+    return nil;
+
   if ([aDecoder allowsKeyedCoding])
     {
       // Nothing to do here, for keyed coding this is handled by NSWindowTemplate.
