@@ -65,9 +65,8 @@ Class _nspopupbuttonCellClass = 0;
       [self setCellClass: [NSPopUpButtonCell class]];
 
       [self exposeBinding: NSSelectedIndexBinding];
+      [self exposeBinding: NSSelectedObjectBinding];
       [self exposeBinding: NSSelectedTagBinding];
-      [self setKeys: [NSArray arrayWithObject: NSSelectedIndexBinding] 
-            triggerChangeNotificationsForDependentKey: NSSelectedTagBinding];
    } 
 }
 
@@ -267,9 +266,7 @@ this to return nil to indicate that we have no context menu.
 
 - (void) selectItem: (id <NSMenuItem>)anObject
 {
-  [self willChangeValueForKey: NSSelectedIndexBinding];
   [_cell selectItem: anObject];
-  [self didChangeValueForKey: NSSelectedIndexBinding];
   [self synchronizeTitleAndSelectedItem];
 }
 
@@ -279,9 +276,7 @@ this to return nil to indicate that we have no context menu.
  */
 - (void) selectItemAtIndex: (int)index
 {
-  [self willChangeValueForKey: NSSelectedIndexBinding];
   [_cell selectItemAtIndex: index];
-  [self didChangeValueForKey: NSSelectedIndexBinding];
   [self synchronizeTitleAndSelectedItem];
 }
 
@@ -292,9 +287,7 @@ this to return nil to indicate that we have no context menu.
  */
 - (void) selectItemWithTitle: (NSString*)title
 {
-  [self willChangeValueForKey: NSSelectedIndexBinding];
   [_cell selectItemWithTitle: title];
-  [self didChangeValueForKey: NSSelectedIndexBinding];
   [self synchronizeTitleAndSelectedItem];
 }
 
@@ -575,6 +568,10 @@ this to return nil to indicate that we have no context menu.
     {
       [self selectItemWithTag: [anObject integerValue]];
     }
+  else if ([aKey isEqual: NSSelectedObjectBinding])
+    {
+      [self selectItemWithTag: [anObject intValue]];
+    }
   else
     {
       [super setValue: anObject forKey: aKey];
@@ -591,6 +588,10 @@ this to return nil to indicate that we have no context menu.
     {
       return [NSNumber numberWithInteger: [self selectedTag]];
     }
+  else if ([aKey isEqual: NSSelectedObjectBinding])
+    {
+      return [NSNumber numberWithInt: [self selectedTag]];
+    }
   else
     {
       return [super valueForKey: aKey];
