@@ -33,12 +33,14 @@
 #import "AppKit/NSEvent.h"
 #import "AppKit/NSGraphics.h"
 #import "AppKit/NSImage.h"
+#import "AppKit/NSKeyValueBinding.h"
 #import "AppKit/NSMenu.h"
 #import "AppKit/NSMenuView.h"
 #import "AppKit/NSPopUpButton.h"
 #import "AppKit/NSPopUpButtonCell.h"
 #import "AppKit/NSWindow.h"
 #import "GNUstepGUI/GSTheme.h"
+#import "GSBindingHelpers.h"
 
 /* The image to use in a specific popupbutton depends on type and
  * preferred edge; that is, _pbc_image[0] if it is a
@@ -202,6 +204,7 @@ static NSImage *_pbc_image[5];
       [self setMenuView: nil];
     }
   
+  // FIXME: Select the first or last item?
   [self selectItemAtIndex: [_menu numberOfItems] - 1];
   [self synchronizeTitleAndSelectedItem];
 }
@@ -423,8 +426,8 @@ static NSImage *_pbc_image[5];
   //[anItem setAction: [self action]];
   //[anItem setTarget: [self target]];
   // Or
-  //[anItem setAction: @selector(_popUpItemAction:)];
-  //[anItem setTarget: self];
+  [anItem setAction: @selector(_popUpItemAction:)];
+  [anItem setTarget: self];
 
   // Select the new item if there isn't any selection.
   if (_selectedItem == nil)
@@ -783,6 +786,16 @@ static NSImage *_pbc_image[5];
   return _selectedItem;
 }
 
+- (id) representedObject
+{
+  return [[self selectedItem] representedObject];
+}
+
+- (void) setRepresentedObject: (id)object
+{
+  [[self selectedItem] setRepresentedObject: object];
+}
+
 - (int) indexOfSelectedItem
 {
   return [_menu indexOfItem: [self selectedItem]];
@@ -1221,6 +1234,11 @@ static NSImage *_pbc_image[5];
           
           [self setAltersStateOfSelectedItem: alters];
         }
+      if ([aDecoder containsValueForKey: @"NSPullDown"])
+        {
+	  BOOL pullDown = [aDecoder decodeBoolForKey: @"NSPullDown"];
+	  [self setPullsDown: pullDown];
+	}
       if ([aDecoder containsValueForKey: @"NSUsesItemFromMenu"])
         {
           BOOL usesItem = [aDecoder decodeBoolForKey: @"NSUsesItemFromMenu"];
@@ -1244,13 +1262,17 @@ static NSImage *_pbc_image[5];
         {
 	  int selectedIdx = [aDecoder decodeIntForKey: 
 					@"NSSelectedIndex"];
-	  [self selectItem: [self itemAtIndex: selectedIdx]];
+	  [self selectItemAtIndex: selectedIdx];
 	}
-      if ([aDecoder containsValueForKey: @"NSPullDown"])
+      else
         {
-	  BOOL pullDown = [aDecoder decodeBoolForKey: @"NSPullDown"];
-	  [self setPullsDown: pullDown];
-	}
+	  [self selectItemAtIndex: 0];
+        }
+      if ([aDecoder containsValueForKey: @"NSMenuItem"])
+        {
+          NSMenuItem *item = [aDecoder decodeObjectForKey: @"NSMenuItem"];
+          [self setMenuItem: item];
+        }
     }
   else
     {
@@ -1320,9 +1342,30 @@ static NSImage *_pbc_image[5];
 - (void) _popUpItemAction: (id)sender
 {
   // first, if sender is one of our items, set it as our selected item
-  int index = [_menu indexOfItem:sender];
+  NSUInteger index = [_menu indexOfItem: sender];
   if (index != NSNotFound)
-    [self selectItemAtIndex:index];
+    [self selectItemAtIndex: index];
+
+  if (_control_view)
+    {
+      GSKeyValueBinding *theBinding;
+
+      theBinding = [GSKeyValueBinding getBinding: NSSelectedIndexBinding 
+                                       forObject: _control_view];
+      if (theBinding != nil)
+        [theBinding reverseSetValueFor: NSSelectedIndexBinding];
+
+      theBinding = [GSKeyValueBinding getBinding: NSSelectedTagBinding 
+                                       forObject: _control_view];
+      if (theBinding != nil)
+        [theBinding reverseSetValueFor: NSSelectedTagBinding];
+
+      theBinding = [GSKeyValueBinding getBinding: NSSelectedObjectBinding 
+                                       forObject: _control_view];
+      if (theBinding != nil)
+        [theBinding reverseSetValueFor: NSSelectedObjectBinding];
+    }
+
   [NSApp sendAction: [self action] to: [self target] from: self];
 }
 
