@@ -63,7 +63,7 @@
 #import "GSGuiPrivate.h"
 
 #define NSNUMBER(a) [NSNumber numberWithInt: (a)]
-#define NSFNUMBER(a) [NSNumber numberWithFloat: (a)]
+#define NSFNUMBER(a) [NSNumber numberWithDouble: (a)]
 
 /*
  * When a view gets printed it may need to be split up into segments, if the 
@@ -502,8 +502,7 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
       
       [panel setAccessoryView: _accessory_view];
       [self _setupPrintInfo];
-      [panel updateFromPrintInfo];
-      button = [panel runModal];
+      button = [panel runModalWithPrintInfo: _print_info];
       [panel setAccessoryView: nil];
 
       if (button != NSOKButton)
@@ -511,7 +510,6 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
           [self cleanUpOperation];
           return NO;
         }
-      [panel finalWritePrintInfo];
     }
 
   result = NO;
@@ -549,7 +547,6 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
   */
   [panel setAccessoryView: _accessory_view];
   [self _setupPrintInfo];
-  [panel updateFromPrintInfo];
   [panel beginSheetWithPrintInfo: _print_info 
                   modalForWindow: docWindow 
                         delegate: self 
@@ -714,8 +711,6 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
 
   if (returnCode == NSOKButton)
     {
-      NSPrintPanel *panel = [self printPanel];
-      [panel finalWritePrintInfo];
       if ([self _runOperation])
         success = [self deliverResult];
     }
@@ -765,7 +760,6 @@ scaleRect(NSRect rect, double scale)
   NSNumber *value;
 
   dict = [_print_info dictionary];
-
   info->paperSize = [_print_info paperSize];
   info->orient = [_print_info orientation];
   value = [dict objectForKey: NSPrintScalingFactor];
@@ -843,7 +837,7 @@ scaleRect(NSRect rect, double scale)
    */
   if (((int)(info->nup / 2) & 0x1) == 1)
     {
-      float tmp;
+      CGFloat tmp;
       if (info->orient == NSLandscapeOrientation)
         info->nupScale = 
           info->paperSize.width/(2*info->paperSize.height);
@@ -860,7 +854,7 @@ scaleRect(NSRect rect, double scale)
     }
   else if (info->nup > 1)
     {
-      info->nupScale = 2.0 / (float)info->nup;
+      info->nupScale = 2.0 / (CGFloat)info->nup;
     }
 
   if ([[dict objectForKey: NSPrintPageDirection] isEqual: @"Columns"])
@@ -918,7 +912,7 @@ scaleRect(NSRect rect, double scale)
   wlimit = [_view widthAdjustLimit];
   for (i = first; i <= last; i++)
     {
-      float newVal, limitVal;
+      CGFloat newVal, limitVal;
       pageRect = [self _rectForPage: i info: info xpage: &xpage ypage: &ypage];
       limitVal = NSMaxY(pageRect) - hlimit * NSHeight(pageRect);
       [_view adjustPageHeightNew: &newVal
@@ -946,7 +940,7 @@ scaleRect(NSRect rect, double scale)
 {
   int i, dir;
   BOOL knowsPageRange, allPages;
-  NSRange viewPageRange;
+  NSRange viewPageRange = NSMakeRange(1, 0);
   NSMutableDictionary *dict;
   NSNumber *value;
   page_info_t info;
@@ -1032,10 +1026,9 @@ scaleRect(NSRect rect, double scale)
     {
       /* Calculate page rects we aren't processing to catch up to the
          first page we are */
-      NSRect pageRect;
-      pageRect = [self _adjustPagesFirst: 1
-                                   last: _currentPage - 1 
-                                   info: &info];
+      [self _adjustPagesFirst: 1
+                         last: _currentPage - 1 
+                         info: &info];
     }
 
   /* Print the header information */
@@ -1100,6 +1093,7 @@ scaleRect(NSRect rect, double scale)
     {
       [_view drawSheetBorderWithSize: info.sheetBounds.size];
       [_view _endSheet];
+      DPSgrestore([self context]);
     }
   [_view endDocument];
 
@@ -1123,7 +1117,7 @@ scaleRect(NSRect rect, double scale)
 {
   int currentPage;
   int numberOnSheet;
-  float xoffset, yoffset, scale;
+  CGFloat xoffset, yoffset, scale;
   NSPoint location;
   NSPrintOperation *printOp = [NSPrintOperation currentOperation];
   NSGraphicsContext *ctxt = [printOp context];
@@ -1138,7 +1132,7 @@ scaleRect(NSRect rect, double scale)
       NSString *label;
       NSRect boundsForPage = info.sheetBounds;
 
-      if(knowsPageRange)
+      if (knowsPageRange)
 	{
 	  boundsForPage = pageRect;
 	}
@@ -1147,6 +1141,7 @@ scaleRect(NSRect rect, double scale)
       if (info.nup == 1)
         label = [NSString stringWithFormat: @"%d", currentPage];
 
+      DPSgsave(ctxt);
       [self beginPage: floor((currentPage - info.first)/info.nup)+1
             label: label
             bBox: boundsForPage
@@ -1157,7 +1152,7 @@ scaleRect(NSRect rect, double scale)
           DPStranslate(ctxt, 0, -info.paperSize.height);
         }
       
-      if(!knowsPageRange)
+      if (!knowsPageRange)
 	{
 	  /* Also offset by margins */
 	  DPStranslate(ctxt, NSMinX(info.paperBounds), NSMinY(info.paperBounds));
@@ -1232,6 +1227,7 @@ scaleRect(NSRect rect, double scale)
     {
       [self drawSheetBorderWithSize: info.sheetBounds.size];
       [self _endSheet];
+      DPSgrestore(ctxt);
     }
 }
 
