@@ -26,18 +26,14 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSException.h>
-#include "AppKit/NSRulerMarker.h"
-#include "AppKit/NSRulerView.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSCursor.h"
-
-@interface NSRulerView (GNUstepInternal)
-- (NSRect)_markersRect;
-@end
+#import <Foundation/NSException.h>
+#import "AppKit/NSRulerMarker.h"
+#import "AppKit/NSRulerView.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSCursor.h"
 
 @implementation NSRulerMarker
 
@@ -238,7 +234,6 @@
   NSPoint mousePositionInWindow;
   NSPoint previousMousePositionInWindow;
   NSPoint mouseOffset;
-  NSRect markersRect;
   float location;
   NSRect drawRect;
   NSRect bounds = [_rulerView bounds];
@@ -270,7 +265,6 @@
   [_rulerView lockFocus];
 
   /* cache some values */
-  markersRect = [_rulerView _markersRect];
   flipped = [_rulerView isFlipped];
   window = [_rulerView window];
   
