@@ -896,15 +896,24 @@ selectCellWithString: (NSString*)title
   return _accessoryView;
 }
 
+- (void) setNameFieldStringValue:(NSString *)value
+{
+  [[_form cellAtIndex: 0] setStringValue: value];
+}
+
+- (NSString *) nameFieldStringValue
+{
+  return [[_form cellAtIndex: 0] stringValue];
+}
+
 - (void) setNameFieldLabel: (NSString *)label
 {
-  // FIXME
+  [[_form cellAtIndex: 0] setTitle: label];
 }
 
 - (NSString *) nameFieldLabel
 {
-  // FIXME
-  return _(@"Save As");
+  return [[_form cellAtIndex: 0] title];
 }
 
 - (void) setMessage: (NSString *)message
@@ -1085,7 +1094,7 @@ selectCellWithString: (NSString*)title
   NSCancelButton (if the user clicks the Cancel button).</p>
   <p>See Also: -runModalForDirectory:file:</p>
  */
-- (int) runModal
+- (NSInteger) runModal
 {
   return [self runModalForDirectory: nil file: @""];
 }
@@ -1100,7 +1109,7 @@ selectCellWithString: (NSString*)title
   selected directory or as a last resort, the current working directory.</p>
   <p>See Also: -runModal</p>
  */
-- (int) runModalForDirectory: (NSString*)path file: (NSString*)filename
+- (NSInteger) runModalForDirectory: (NSString*)path file: (NSString*)filename
 {
   [self _setupForDirectory: path file: filename];
   if ([filename length] > 0)
@@ -1108,9 +1117,9 @@ selectCellWithString: (NSString*)title
   return [NSApp runModalForWindow: self];
 }
 
-- (int) runModalForDirectory: (NSString *)path
-			file: (NSString *)filename
-	    relativeToWindow: (NSWindow*)window
+- (NSInteger) runModalForDirectory: (NSString *)path
+                              file: (NSString *)filename
+                  relativeToWindow: (NSWindow*)window
 {
   [self _setupForDirectory: path file: filename];
   if ([filename length] > 0)
@@ -1775,7 +1784,8 @@ createRowsForColumn: (int)column
       if (exists && (!isDir || !HAS_PATH_PREFIX(_directory, pathAndFile)))
 	{
 	  if (isDir && !_treatsFilePackagesAsDirectories
-	      && [ws isFilePackageAtPath: pathAndFile])
+	      && ([ws isFilePackageAtPath: pathAndFile]
+		  || [_allowedFileTypes containsObject: extension]))
 	    {
 	      isDir = NO;
 	    }
@@ -1808,7 +1818,15 @@ createRowsForColumn: (int)column
 
 	  cell = [matrix cellAtRow: addedRows column: 0];
 	  [cell setStringValue: file];
-	  
+
+	  {
+	    NSImage *icon = [[ws iconForFile: pathAndFile] copy];
+	    CGFloat iconSize = [cell cellSize].height - 1;
+	    [icon setSize: NSMakeSize(iconSize, iconSize)];
+	    [cell setImage: icon];
+	    [icon release];
+	  }
+
 	  if (isDir)
 	    [cell setLeaf: NO];
 	  else
