@@ -35,6 +35,7 @@
 #import <Foundation/NSGeometry.h>
 #import <Foundation/NSNotification.h>
 #import <Foundation/NSValue.h>
+#import <Foundation/NSUserDefaults.h>
 #import "AppKit/AppKitExceptions.h"
 #import "AppKit/NSApplication.h"
 #import "AppKit/NSInterfaceStyle.h"
@@ -273,7 +274,7 @@ static NSMutableArray *screenArray = nil;
       int			bps = 0;
       NSSize			screenResolution;
       NSString			*colorSpaceName = nil;
-      GSDisplayServer		*srv;
+      CGFloat                   scaleFactor;
 
       /*
        * This method generates a dictionary from the
@@ -293,13 +294,10 @@ static NSMutableArray *screenArray = nil;
 		  forKey: NSDeviceSize];
 
       // Add the NSDeviceResolution dictionary item
-      srv = GSCurrentServer();
-      if (srv != nil)
-	{
-	  screenResolution = [srv resolutionForScreen: _screenNumber];
-	  [devDesc setObject: [NSValue valueWithSize: screenResolution]
-		      forKey: NSDeviceResolution];
-	}
+      scaleFactor = [self userSpaceScaleFactor];
+      screenResolution = NSMakeSize(72.0 * scaleFactor, 72.0 * scaleFactor);
+      [devDesc setObject: [NSValue valueWithSize: screenResolution]
+		  forKey: NSDeviceResolution];
 
       // Add the bits per sample entry
       bps = NSBitsPerSampleFromDepth(_depth);
@@ -414,19 +412,26 @@ static NSMutableArray *screenArray = nil;
 
 - (float) userSpaceScaleFactor
 {
-  GSDisplayServer *srv;
-  NSSize dpi;
+  NSNumber *factor = [[NSUserDefaults standardUserDefaults]
+		       objectForKey: @"GSScaleFactor"];
 
-  srv = GSCurrentServer();
-  if (srv != nil)
+  if (factor != nil)
     {
-      dpi = [GSCurrentServer() resolutionForScreen: _screenNumber];
-      // take average for 72dpi
-      return (dpi.width + dpi.height) / 144;
+      return [factor floatValue];
     }
   else
-    {
-      return 1.0;
+    {    
+      GSDisplayServer *srv = GSCurrentServer();
+      if (srv != nil)
+      {
+        NSSize dpi = [GSCurrentServer() resolutionForScreen: _screenNumber];
+        // average the width and height
+        return (dpi.width + dpi.height) / 144.0;
+      }
+      else
+      {
+        return 1.0;
+      }
     }
 }
 
