@@ -29,19 +29,22 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSUserDefaults.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSButtonCell.h>
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSImage.h>
-#include <AppKit/NSMenu.h>
-#include "AppKit/NSMenuView.h"
-#include <AppKit/NSSearchFieldCell.h>
-#include "AppKit/NSWindow.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSUserDefaults.h>
+
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSButtonCell.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSPopUpButtonCell.h"
+#import "AppKit/NSSearchField.h"
+#import "AppKit/NSSearchFieldCell.h"
+#import "AppKit/NSWindow.h"
 
 @interface NSSearchFieldCell (Private)
 
@@ -82,8 +85,8 @@
 
       //_recent_searches = [[NSMutableArray alloc] init];
       //_recents_autosave_name = nil;
-      [self _loadSearches];
       _max_recents = 10;
+      [self _loadSearches];
     }
 
   return self;
@@ -126,7 +129,8 @@
 		       inView: controlView];
   [super drawWithFrame: [self searchTextRectForBounds: cellFrame] 
 	 inView: controlView];
-  [_cancel_button_cell drawWithFrame: [self cancelButtonRectForBounds: cellFrame] 
+  if ([[self stringValue] length] > 0)
+    [_cancel_button_cell drawWithFrame: [self cancelButtonRectForBounds: cellFrame] 
 		       inView: controlView];
 }
 
@@ -199,7 +203,7 @@
     }
   else
     {
-      searches = [NSMutableArray arrayWithArray:searches];
+      searches = [NSMutableArray arrayWithArray: searches];
     }
   ASSIGN(_recent_searches, searches);
   [self _saveSearches];
@@ -212,9 +216,9 @@
       ASSIGN(_recent_searches, [NSMutableArray array]);
     }
   if (searchTerm != nil && [searchTerm length] > 0
-	&& [_recent_searches indexOfObject:searchTerm] == NSNotFound)
+	&& [_recent_searches indexOfObject: searchTerm] == NSNotFound)
     {
-      [_recent_searches addObject:searchTerm];
+      [_recent_searches addObject: searchTerm];
       [self _saveSearches];
     }
 }
@@ -229,9 +233,9 @@
   ASSIGN(_menu_template, menu);
   if (menu)
     {
-      [[self searchButtonCell] setTarget:self];
-      [[self searchButtonCell] setAction:@selector(_openPopup:)];
-      [[self searchButtonCell] sendActionOn:NSLeftMouseDownMask];
+      [[self searchButtonCell] setTarget: self];
+      [[self searchButtonCell] setAction: @selector(_openPopup:)];
+      [[self searchButtonCell] sendActionOn: NSLeftMouseDownMask];
     }
   else
     {
@@ -255,7 +259,7 @@
 }
 
 - (void) setSearchButtonCell: (NSButtonCell *)cell
-{ 
+{
   ASSIGN(_search_button_cell, cell);
 }
 
@@ -272,8 +276,8 @@
   [c setEditable: NO];
   [c setImagePosition: NSImageOnly];
   [c setImage: [NSImage imageNamed: @"GSStop"]];
-  [c setAction: @selector(delete:)];
-  [c setTarget: nil];
+  [c setAction: @selector(clearSearch:)];
+  [c setTarget: self];
   [c setKeyEquivalent: @"\e"];
   [c setKeyEquivalentModifierMask: 0];
 }
@@ -295,7 +299,7 @@
 //  [c setTarget: [self target]];
   [c setAction: @selector(performClick:)];
   [c setTarget: self];
-  [c sendActionOn:NSLeftMouseUpMask];
+  [c sendActionOn: NSLeftMouseUpMask];
   [c setKeyEquivalent: @"\r"];
   [c setKeyEquivalentModifierMask: 0];
 }
@@ -361,7 +365,7 @@
 
 - (void) endEditing: (NSText *)editor
 {
-  [self addToRecentSearches:[[[editor string] copy] autorelease]];
+  [self addToRecentSearches: [[[editor string] copy] autorelease]];
   [super endEditing: editor];
   [[NSNotificationCenter defaultCenter] 
       removeObserver: self 
@@ -398,6 +402,7 @@
   NSRect rect;
   NSPoint thePoint;
   NSPoint location = [event locationInWindow];
+  NSText *currentEditor;
 
   thePoint = [controlView convertPoint: location fromView: nil];
 
@@ -420,6 +425,15 @@
 				      untilMouseUp: untilMouseUp];
     }
 
+  currentEditor = ([controlView isKindOfClass:[NSControl class]]
+		   ? [(NSControl *)controlView currentEditor]
+		   : nil);
+  if (currentEditor)
+    {
+      [currentEditor mouseDown: event];
+      return YES;
+    }
+
   return [super trackMouse: event 
 		inRect: [self searchTextRectForBounds: cellFrame]
 		ofView: controlView 
@@ -429,7 +443,8 @@
 - (void) textDidChange: (NSNotification *)notification
 { 
   NSText *textObject;
-
+  [_control_view setNeedsDisplay:YES];
+  
   // make textChanged send action (unless disabled)
   if (_sends_whole_search_string)
     {
@@ -440,10 +455,16 @@
   textObject = [notification object];
   // copy the current NSTextEdit string so that it can be read from the NSSearchFieldCell!
   [self setStringValue: [textObject string]];
-  [[self searchButtonCell] performClickWithFrame: 
-			       [self searchButtonRectForBounds: [_control_view bounds]] 
-			   inView: _control_view];
+  [NSApp sendAction:[self action] to:[self target] from:_control_view];
 }
+
+- (void) clearSearch:(id)sender
+{
+  [self setStringValue:@""];
+  [NSApp sendAction:[self action] to:[self target] from:_control_view];
+  [_control_view setNeedsDisplay:YES];
+}
+
 //
 // NSCoding protocol
 //
@@ -561,8 +582,9 @@
   NSRect cellFrame;
   int i;
   int recentCount = [_recent_searches count];
-  // NSRect textRect;
-
+  NSPopUpButtonCell *pbcell = [[NSPopUpButtonCell alloc] initTextCell:nil pullsDown:NO];
+  int selectedItemIndex = -1, newSelectedItemIndex;
+  
   template = [self searchMenuTemplate];
   popupmenu = [[NSMenu alloc] init];
 
@@ -573,56 +595,63 @@
       NSMenuItem *item, *newItem = nil;
 
       item = (NSMenuItem*)[template itemAtIndex: i];
+      if ([item state])
+        selectedItemIndex = [popupmenu numberOfItems]; // remember index of previously selected item
       tag = [item tag];
       if (tag == NSSearchFieldRecentsTitleMenuItemTag)
-	{
-	  if (recentCount > 0) // only show items with this tag if there are recent searches
-	    {
-	      newItem = [[item copy] autorelease];
-	    }
-	}
+        {
+          if (recentCount > 0) // only show items with this tag if there are recent searches
+            {
+              newItem = [[item copy] autorelease];
+            }
+        }
       else if (tag == NSSearchFieldClearRecentsMenuItemTag)
-	{
-	  if (recentCount > 0) // only show items with this tag if there are recent searches
-	    {
-	      newItem = [[item copy] autorelease];
-	      [newItem setTarget:self];
-	      [newItem setAction:@selector(_clearSearches:)];
-	    }
-	}
+        {
+          if (recentCount > 0) // only show items with this tag if there are recent searches
+            {
+              newItem = [[item copy] autorelease];
+              [newItem setTarget:self];
+              [newItem setAction:@selector(_clearSearches:)];
+            }
+        }
       else if (tag == NSSearchFieldNoRecentsMenuItemTag)
         { 
-	  if (recentCount == 0) // only show items with this tag if there are NO recent searches
-	    {
-	      newItem = [[item copy] autorelease];
-	    }
-	}
+          if (recentCount == 0) // only show items with this tag if there are NO recent searches
+            {
+              newItem = [[item copy] autorelease];
+            }
+        }
       else if (tag == NSSearchFieldRecentsMenuItemTag)
         {
-	  int j;
+          int j;
 
-	  for (j = 0; j < recentCount; j++)
-	    {
-	      id <NSMenuItem> searchItem = [popupmenu addItemWithTitle: 
-						    [_recent_searches objectAtIndex: j]
-						  action: 
-						    @selector(_searchForRecent:)
-						  keyEquivalent: 
-						    [item keyEquivalent]];
-	      [searchItem setTarget: self];
-	    }
-	}
+          for (j = 0; j < recentCount; j++)
+            {
+              id <NSMenuItem> searchItem = [popupmenu addItemWithTitle: 
+                                                                    [_recent_searches objectAtIndex: j]
+                                                                action: 
+                                                                    @selector(_searchForRecent:)
+                                                         keyEquivalent: 
+                                                                    [item keyEquivalent]];
+              [searchItem setTarget: self];
+            }
+        }
       else // copy all other items without special tags from the template into the popup
-	{
-	  newItem = [[item copy] autorelease];
-	}
+        {
+          newItem = [[item copy] autorelease];
+        }
 
       if (newItem != nil)
-	{
-	  [popupmenu addItem: newItem];
-	}
+        {
+          [popupmenu addItem: newItem];
+        }
     } 
 
+  [pbcell setMenu:popupmenu];
+  [pbcell selectItemAtIndex:selectedItemIndex];
+  [[popupmenu itemAtIndex:selectedItemIndex] setState:NSOffState]; // ensure that state resets fully
+  [[popupmenu itemAtIndex:selectedItemIndex] setState:NSOnState];
+
   // Prepare to display the popup
   cvWin = [_control_view window];
   cellFrame = [_control_view frame];
@@ -639,15 +668,32 @@
   // Last, display the window
   [[mr window] orderFrontRegardless];
 
-  [mr mouseDown:[NSApp currentEvent]];
+  [mr mouseDown: [NSApp currentEvent]];
+  newSelectedItemIndex = [pbcell indexOfSelectedItem];
+  if (newSelectedItemIndex != selectedItemIndex && newSelectedItemIndex != -1
+      && newSelectedItemIndex < [template numberOfItems])
+    {
+      int tag = [[template itemAtIndex:newSelectedItemIndex] tag];
+      if (tag != NSSearchFieldRecentsTitleMenuItemTag && tag != NSSearchFieldClearRecentsMenuItemTag
+          && tag != NSSearchFieldNoRecentsMenuItemTag && tag != NSSearchFieldRecentsMenuItemTag
+          && ![[template itemAtIndex:newSelectedItemIndex] isSeparatorItem])
+        {
+          //new selected item within the template that's not a template special item
+          [[template itemAtIndex:selectedItemIndex] setState:NSOffState];
+          [[template itemAtIndex:newSelectedItemIndex] setState:NSOnState];
+        }
+    }
   AUTORELEASE(popupmenu);
+  AUTORELEASE(pbcell);
 }
 
 - (void) _searchForRecent: (id)sender
 {
   NSString *searchTerm = [sender title];
-  [(id)_control_view setStringValue:searchTerm];
-  [self performClick:self];  // do the search
+
+  [self setStringValue: searchTerm];
+  [self performClick: self];  // do the search
+  [(id)_control_view selectText: self];
 }
 
 - (void) _clearSearches: (id)sender
@@ -659,6 +705,7 @@
 {
   NSArray *list;
   NSString *name = [self recentsAutosaveName];
+
   if (name)
     {
       list = [[NSUserDefaults standardUserDefaults] 
@@ -671,6 +718,7 @@
 {
   NSArray *list = [self recentSearches];
   NSString *name = [self recentsAutosaveName];
+
   if (name && list)
     {
       [[NSUserDefaults standardUserDefaults] 
