@@ -44,7 +44,8 @@
 #import "AppKit/NSWindow.h"
 #import <GNUstepGUI/GSTheme.h>
 
-DEFINE_RINT_IF_MISSING
+#import "GSGuiPrivate.h"
+
 #ifndef HAVE_ATAN2F
 #define atan2f atan2
 #endif
@@ -662,13 +663,20 @@ float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
   return _allowsTickMarkValuesOnly;
 }
 
+/* verified on Cocoa that a circular slider with one tick has two values: 0, 50 */
 - (double) closestTickMarkValueToValue: (double)aValue
 {
   double d, f;
+  int effectiveTicks;
 
   if (_numberOfTickMarks == 0)
     return aValue;
-  else if (_numberOfTickMarks == 1)
+  
+  effectiveTicks = _numberOfTickMarks;
+  if (_type == NSCircularSlider)
+    effectiveTicks++;
+
+  if (effectiveTicks == 1)
     return (_maxValue + _minValue) / 2;
 
   if (aValue < _minValue)
@@ -681,15 +689,19 @@ float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
     }
 
   d = _maxValue - _minValue;
-  f = ((aValue - _minValue)  * (_numberOfTickMarks - 1)) / d;
-  f = ((rint(f) * d) / (_numberOfTickMarks - 1)) + _minValue;
+  f = ((aValue - _minValue)  * (effectiveTicks - 1)) / d;
+  f = ((GSRoundTowardsInfinity(f) * d) / (effectiveTicks - 1)) + _minValue;
+
+  /* never return the maximum value, tested on Apple */
+  if (_type == NSCircularSlider && (f >= _maxValue))
+    f = _minValue;
 
   return f;
 }
 
-- (int) indexOfTickMarkAtPoint: (NSPoint)point
+- (NSInteger) indexOfTickMarkAtPoint: (NSPoint)point
 {
-  int i;
+  NSInteger i;
 
   for (i = 0; i < _numberOfTickMarks; i++)
     {
@@ -702,12 +714,12 @@ float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
   return NSNotFound;
 }
 
-- (int) numberOfTickMarks
+- (NSInteger) numberOfTickMarks
 {
   return _numberOfTickMarks;
 }
 
-- (NSRect) rectOfTickMarkAtIndex: (int)index
+- (NSRect) rectOfTickMarkAtIndex: (NSInteger)index
 {
   NSRect rect = _trackRect;
   float d;
@@ -742,7 +754,7 @@ float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
   _allowsTickMarkValuesOnly = flag;
 }
 
-- (void) setNumberOfTickMarks: (int)numberOfTickMarks
+- (void) setNumberOfTickMarks: (NSInteger)numberOfTickMarks
 {
   _numberOfTickMarks = numberOfTickMarks;
   if ((_control_view != nil) &&  
@@ -767,7 +779,7 @@ float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
   return _tickMarkPosition;
 }
 
-- (double) tickMarkValueAtIndex: (int)index
+- (double) tickMarkValueAtIndex: (NSInteger)index
 {
   if ((index < 0) || (index >= _numberOfTickMarks))
     {
@@ -782,7 +794,12 @@ float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
   if (index <= 0)
     return _minValue;
 
-  return _minValue + index * (_maxValue - _minValue) / (_numberOfTickMarks - 1);
+  if (_type == NSCircularSlider)
+    return _minValue + index * (_maxValue - _minValue) / _numberOfTickMarks;
+  if (_type == NSLinearSlider)
+    return _minValue + index * (_maxValue - _minValue) / (_numberOfTickMarks - 1);
+
+  return 0.0;
 }
 
 - (BOOL) trackMouse: (NSEvent*)theEvent
