@@ -46,6 +46,7 @@
 
 #import "AppKit/NSApplication.h"
 #import "AppKit/NSColor.h"
+#import "AppKit/NSCursor.h"
 #import "AppKit/NSEvent.h"
 #import "AppKit/NSGraphics.h"
 #import "AppKit/NSImage.h"
@@ -73,7 +74,6 @@ static NSNotificationCenter *nc = nil;
       _draggedBarWidth = 8; // default bigger than dividerThickness
       _isVertical = NO;
       ASSIGN(_dividerColor, [NSColor controlShadowColor]);
-      ASSIGN(_backgroundColor, [NSColor controlBackgroundColor]);
       ASSIGN(_dimpleImage, [NSImage imageNamed: @"common_Dimple"]); 
 
       _never_displayed_before = YES;
@@ -422,6 +422,7 @@ static NSNotificationCenter *nc = nil;
   BOOL liveResize = ![[NSUserDefaults standardUserDefaults] boolForKey: @"GSUseGhostResize"];
   NSRect oldRect; //only one can be dragged at a time
   BOOL lit = NO;
+  NSCursor *cursor;
 
   /*  if there are less the two subviews, there is nothing to do */
   if (count < 2)
@@ -530,6 +531,7 @@ static NSNotificationCenter *nc = nil;
       /* set the default limits on the dragging */
       minCoord = NSMinY(bigRect) + divVertical;
       maxCoord = NSHeight(bigRect) + NSMinY(bigRect) - divVertical;
+      cursor = [NSCursor resizeUpDownCursor];
     }
   else
     {
@@ -538,8 +540,11 @@ static NSNotificationCenter *nc = nil;
       /* set the default limits on the dragging */
       minCoord = NSMinX(bigRect) + divHorizontal;
       maxCoord = NSWidth(bigRect) + NSMinX(bigRect) - divHorizontal;
+      cursor = [NSCursor resizeLeftRightCursor];
     }
 
+  [cursor push];
+
   /* find out what the dragging limit is */
   if (_delegate)
     {
@@ -742,6 +747,8 @@ static NSNotificationCenter *nc = nil;
 
   [self setNeedsDisplay: YES];
 
+  [NSCursor pop];
+
   //[self display];
 }
 
@@ -1182,7 +1189,7 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
 
 - (BOOL) isOpaque
 {
-  return YES;
+  return (_backgroundColor != nil);
 }
 
 - (void) resizeSubviewsWithOldSize: (NSSize) oldSize
@@ -1232,13 +1239,70 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
   SET_DELEGATE_NOTIFICATION(WillResizeSubviews);
 }
 
+- (void) resetCursorRects
+{
+  NSArray *subs = [self subviews];
+  NSInteger i;
+  const NSInteger count = [subs count];
+  const NSRect visibleRect = [self visibleRect];
+  NSCursor *cursor;
+
+  if (_isVertical)
+    {
+      cursor = [NSCursor resizeLeftRightCursor];
+    }
+  else
+    {
+      cursor = [NSCursor resizeUpDownCursor];
+    }
+
+  for (i = 0; i < (count-1); i++)
+    {
+      NSView *v = [subs objectAtIndex: i];
+      NSRect divRect = [v frame];
+      if (_isVertical == NO)
+        {
+          divRect.origin.y = NSMaxY (divRect);
+          divRect.size.height = _dividerWidth;
+        }
+      else
+        {
+          divRect.origin.x = NSMaxX (divRect);
+          divRect.size.width = _dividerWidth;
+        }
+      divRect = NSIntersectionRect(divRect, visibleRect);
+
+      if (!NSEqualRects(NSZeroRect, divRect))
+	{
+	  [self addCursorRect: divRect cursor: cursor];
+	}
+    }
+}
+
+- (NSColor*) dividerColor
+{
+  return _dividerColor;
+}
+
+- (NSSplitViewDividerStyle) dividerStyle
+{
+  // FIXME
+  return NSSplitViewDividerStyleThick;
+}
+
+- (void) setDividerStyle: (NSSplitViewDividerStyle)dividerStyle
+{
+  // FIXME
+}
+
 /*
  * NSCoding protocol
  */
 - (void) encodeWithCoder: (NSCoder *)aCoder
 {
   [super encodeWithCoder: aCoder];
-  if([aCoder allowsKeyedCoding])
+
+  if ([aCoder allowsKeyedCoding])
     {
       [aCoder encodeBool: _isVertical forKey: @"NSIsVertical"];
       [aCoder encodeObject: _autosaveName forKey: @"NSAutosaveName"];
@@ -1268,7 +1332,11 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
 - (id) initWithCoder: (NSCoder *)aDecoder
 {
   self = [super initWithCoder: aDecoder];
- 
+  if (self == nil)
+    {
+      return nil;
+    }
+
   if ([aDecoder allowsKeyedCoding])
     {
       id subview = nil;
@@ -1287,7 +1355,6 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
       _dividerWidth = [self dividerThickness];
       _draggedBarWidth = 8; // default bigger than dividerThickness
       ASSIGN(_dividerColor, [NSColor controlShadowColor]);
-      ASSIGN(_backgroundColor, [NSColor controlBackgroundColor]);
       ASSIGN(_dimpleImage, [NSImage imageNamed: @"common_Dimple"]); 
       _never_displayed_before = YES;
       _is_pane_splitter = YES;
@@ -1373,11 +1440,6 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
   ASSIGN(_backgroundColor, aColor);
 }
 
-- (NSColor*) dividerColor
-{
-  return _dividerColor;
-}
-
 - (void) setDividerColor: (NSColor*) aColor
 {
   ASSIGN(_dividerColor, aColor);
