@@ -26,18 +26,22 @@
    Boston, MA 02110-1301, USA.
 */
 
+#import <Foundation/NSKeyValueCoding.h>
+#import <Foundation/NSValue.h>
 #import "AppKit/NSColor.h"
 #import "AppKit/NSEvent.h"
 #import "AppKit/NSFont.h"
+#import "AppKit/NSFontManager.h"
 #import "AppKit/NSForm.h"
 #import "AppKit/NSGraphics.h"
 #import "AppKit/NSImage.h"
+#import "AppKit/NSKeyValueBinding.h"
 #import "AppKit/NSMatrix.h"
 #import "AppKit/NSTabView.h"
 #import "AppKit/NSTabViewItem.h"
 #import "AppKit/NSWindow.h"
-#import "AppKit/PSOperators.h"
 #import "GNUstepGUI/GSTheme.h"
+#import "GSBindingHelpers.h"
 
 @implementation NSTabView
 
@@ -49,6 +53,11 @@
   if (self == [NSTabView class])
     {
       [self setVersion: 2];
+
+      [self exposeBinding: NSSelectedIndexBinding];
+      [self exposeBinding: NSFontBinding];
+      [self exposeBinding: NSFontNameBinding];
+      [self exposeBinding: NSFontSizeBinding];
     }
 }
 
@@ -92,7 +101,7 @@
 }
 
 - (void) insertTabViewItem: (NSTabViewItem*)tabViewItem
-                   atIndex: (int)index
+                   atIndex: (NSInteger)index
 {
   [tabViewItem _setTabView: self];
   [_items insertObject: tabViewItem atIndex: index];
@@ -145,15 +154,15 @@
   [self setNeedsDisplay: YES];
 }
 
-- (int) indexOfTabViewItem: (NSTabViewItem*)tabViewItem
+- (NSInteger) indexOfTabViewItem: (NSTabViewItem*)tabViewItem
 {
   return [_items indexOfObject: tabViewItem];
 }
 
-- (int) indexOfTabViewItemWithIdentifier: (id)identifier
+- (NSInteger) indexOfTabViewItemWithIdentifier: (id)identifier
 {
-  unsigned howMany = [_items count];
-  unsigned i;
+  NSUInteger howMany = [_items count];
+  NSUInteger i;
 
   for (i = 0; i < howMany; i++)
     {
@@ -166,12 +175,12 @@
   return NSNotFound;
 }
 
-- (int) numberOfTabViewItems
+- (NSInteger) numberOfTabViewItems
 {
   return [_items count];
 }
 
-- (NSTabViewItem*) tabViewItemAtIndex: (int)index
+- (NSTabViewItem*) tabViewItemAtIndex: (NSInteger)index
 {
   return [_items objectAtIndex: index];
 }
@@ -254,7 +263,8 @@
       if (selectedView != nil)
         {
           [self addSubview: selectedView];
-	  [selectedView setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
+          // FIXME: We should not change this mask
+          [selectedView setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
           [selectedView setFrame: [self contentRect]];
           [_window makeFirstResponder: [_selected initialFirstResponder]];
         }
@@ -270,7 +280,7 @@
     }
 }
 
-- (void) selectTabViewItemAtIndex: (int)index
+- (void) selectTabViewItemAtIndex: (NSInteger)index
 {
   if (index < 0 || index >= [_items count])
     [self selectTabViewItem: nil];
@@ -280,14 +290,14 @@
 
 - (void) selectTabViewItemWithIdentifier: (id)identifier 
 {
-  int index = [self indexOfTabViewItemWithIdentifier: identifier];
+  NSInteger index = [self indexOfTabViewItemWithIdentifier: identifier];
 
   [self selectTabViewItemAtIndex: index];
 }
 
 - (void) takeSelectedTabViewItemFromSender: (id)sender
 {
-  int index = -1;
+  NSInteger index = -1;
 
   if ([sender respondsToSelector: @selector(indexOfSelectedItem)] == YES)
     {
@@ -295,9 +305,9 @@
     }
   else if ([sender isKindOfClass: [NSMatrix class]] == YES)
     {
-      int cols = [sender numberOfColumns];
-      int row = [sender selectedRow];
-      int col = [sender selectedColumn];
+      NSInteger cols = [sender numberOfColumns];
+      NSInteger row = [sender selectedRow];
+      NSInteger col = [sender selectedColumn];
 
       if (row >= 0 && col >= 0)
         {
@@ -454,8 +464,8 @@
  */
 - (NSTabViewItem*) tabViewItemAtPoint: (NSPoint)point
 {
-  int howMany = [_items count];
-  int i;
+  NSInteger howMany = [_items count];
+  NSInteger i;
 
   for (i = 0; i < howMany; i++)
     {
@@ -476,7 +486,14 @@
   
   if (anItem != nil  &&  ![anItem isEqual: _selected])
     {
+      GSKeyValueBinding *theBinding;
+
       [self selectTabViewItem: anItem];
+
+      theBinding = [GSKeyValueBinding getBinding: NSSelectedIndexBinding 
+                                       forObject: self];
+      if (theBinding != nil)
+        [theBinding reverseSetValueFor: NSSelectedIndexBinding];
     }
 }
 
@@ -557,6 +574,14 @@
         {
           [self setFont: [aDecoder decodeObjectForKey: @"NSFont"]];
         }
+      if ([aDecoder containsValueForKey: @"NSTvFlags"])
+        {
+          int vFlags = [aDecoder decodeIntForKey: @"NSTvFlags"];
+
+          [self setControlTint: ((vFlags & 0x70000000) >> 28)];
+          [self setControlSize: ((vFlags & 0x0c000000) >> 26)];
+          [self setTabViewType: (vFlags & 0x00000007)];
+        }
       if ([aDecoder containsValueForKey: @"NSTabViewItems"])
         {
           ASSIGN(_items, [aDecoder decodeObjectForKey: @"NSTabViewItems"]);
@@ -566,15 +591,6 @@
           [self selectTabViewItem: [aDecoder decodeObjectForKey: 
                                                  @"NSSelectedTabViewItem"]];
         }
-      if ([aDecoder containsValueForKey: @"NSTvFlags"])
-        {
-          int vFlags = [aDecoder decodeIntForKey: @"NSTvFlags"];
-
-          [self setControlTint: ((vFlags & 0x70000000) >> 28)];
-          [self setControlSize: ((vFlags & 0x0c000000) >> 26)];
-          [self setTabViewType: (vFlags & 0x00000007)];
-        }
-      [self setAutoresizesSubviews: YES];
     }
   else
     {
@@ -620,4 +636,47 @@
     }
   return self;
 }
+
+- (void) setValue: (id)anObject forKey: (NSString*)aKey
+{
+  if ([aKey isEqual: NSSelectedIndexBinding])
+    {
+      [self selectTabViewItemAtIndex: [anObject intValue]];
+    }
+  else if ([aKey isEqual: NSFontNameBinding])
+    {
+      [self setFont: [[NSFontManager sharedFontManager] convertFont: [self font] 
+                                                             toFace: anObject]];
+    }
+  else if ([aKey isEqual: NSFontSizeBinding])
+    {
+      [self setFont: [[NSFontManager sharedFontManager] convertFont: [self font]
+                                                             toSize: [anObject doubleValue]]];
+    }
+  else
+    {
+      [super setValue: anObject forKey: aKey];
+    }
+}
+
+- (id) valueForKey: (NSString*)aKey
+{
+  if ([aKey isEqual: NSSelectedIndexBinding])
+    {
+      return [NSNumber numberWithInt: [self indexOfTabViewItem: 
+                                              [self selectedTabViewItem]]];
+    }
+  else if ([aKey isEqual: NSFontNameBinding])
+    {
+      return [[self font] fontName];
+    }
+  else if ([aKey isEqual: NSFontSizeBinding])
+    {
+      return [NSNumber numberWithDouble: (double)[[self font] pointSize]];
+    }
+  else
+    {
+      return [super valueForKey: aKey];
+    }
+}
 @end
