@@ -30,11 +30,17 @@
 #import "AppKit/NSGraphics.h"
 #import "AppKit/NSImage.h"
 #import "AppKit/NSStringDrawing.h"
+#import "AppKit/NSTabView.h"
 #import "AppKit/NSTabViewItem.h"
 #import "AppKit/PSOperators.h"
 
 @implementation NSTabViewItem
 
+- (id) init
+{
+  return [self initWithIdentifier: @""];
+}
+
 - (id) initWithIdentifier: (id)identifier
 {
   self = [super init];
@@ -43,6 +49,7 @@
     {
       ASSIGN(_ident, identifier);
       _state = NSBackgroundTab;
+      _view = [NSView new];
       // Use the window background colour as default, not the control background colour.
       [self setColor: [NSColor windowBackgroundColor]];
     }
@@ -209,7 +216,7 @@
       [aCoder encodeObject:_label];
       [aCoder encodeObject:_view];
       [aCoder encodeObject:_color];
-      [aCoder encodeValueOfObjCType: @encode(NSTabState) at: &_state];
+      [aCoder encodeValueOfObjCType: @encode(int) at: &_state];
       [aCoder encodeObject:_first_responder];
       [aCoder encodeObject:_tabview];
     }
@@ -233,7 +240,7 @@
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_label];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_view];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_color];
-      [aDecoder decodeValueOfObjCType: @encode(NSTabState) at:&_state];
+      [aDecoder decodeValueOfObjCType: @encode(int) at:&_state];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_first_responder];
       AUTORELEASE(_first_responder);
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_tabview];
