@@ -85,7 +85,7 @@
 + (void) initialize
 {
   if (self == [NSTableColumn class])
-    [self setVersion: 2];
+    [self setVersion: 3];
 }
 
 /*
@@ -116,6 +116,7 @@
   _headerCell = [[NSTableHeaderCell alloc] init];
   _dataCell = [[NSTextFieldCell alloc] init];
   [_dataCell setLineBreakMode: NSLineBreakByTruncatingTail];
+  [_dataCell setEditable: YES];
   _headerToolTip = nil;
 
   _sortDescriptorPrototype = nil;
@@ -493,6 +494,8 @@ to YES. */
       
       [aCoder encodeObject: _headerCell];
       [aCoder encodeObject: _dataCell];
+
+      [aCoder encodeObject: _sortDescriptorPrototype];
     }
 }
 
@@ -565,7 +568,7 @@ to YES. */
       if (!self)
         return nil;
 
-     if (version == 2)
+      if (version >= 2)
         {
           _identifier = RETAIN([aDecoder decodeObject]);
           [aDecoder decodeValueOfObjCType: @encode(float) at: &_width];
@@ -575,6 +578,11 @@ to YES. */
           [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_is_editable];
           _headerCell = RETAIN([aDecoder decodeObject]);
           _dataCell   = RETAIN([aDecoder decodeObject]);
+
+          if (version >= 3)
+            {
+              _sortDescriptorPrototype = RETAIN([aDecoder decodeObject]);
+            }
         }
       else
         {
