@@ -28,7 +28,7 @@
 #import "AppKit/NSFont.h"
 #import "AppKit/NSImage.h"
 #import "AppKit/NSTableHeaderCell.h"
-#include "GNUstepGUI/GSTheme.h"
+#import "GNUstepGUI/GSTheme.h"
 
 @implementation NSTableHeaderCell
 
@@ -44,6 +44,7 @@
   [self setBackgroundColor: [NSColor controlShadowColor]];
   [self setDrawsBackground: YES];
   [self setFont: [NSFont titleBarFontOfSize: 0]];
+  [self setWraps: NO];
   // This is not exactly true 
   _cell.is_bezeled = YES;
   _cell.is_bordered = NO;
@@ -53,12 +54,7 @@
 
 - (NSRect) drawingRectForBounds: (NSRect)theRect
 {
-  NSSize borderSize;
-
-  // This adjustment must match the drawn border
-  borderSize = NSMakeSize(1, 1);
-
-  return NSInsetRect(theRect, borderSize.width, borderSize.height);
+  return [[GSTheme theme] tableHeaderCellDrawingRectForBounds: theRect];
 }
 
 - (void) _drawBorderAndBackgroundWithFrame: (NSRect)cellFrame 
