@@ -565,6 +565,17 @@ static NSNumber *float_plus_one(NSNumber *cur)
   [self insertText: @"\t"];
 }
 
+- (void) insertContainerBreak: (id)sender
+{
+  unichar ch = NSFormFeedCharacter;
+  [self insertText: [NSString stringWithCharacters: &ch length: 1]];
+}
+
+- (void) insertLineBreak: (id)sender
+{
+  unichar ch = NSLineSeparatorCharacter;
+  [self insertText: [NSString stringWithCharacters: &ch length: 1]];
+}
 
 - (void) deleteForward: (id)sender
 {
@@ -666,9 +677,6 @@ static NSNumber *float_plus_one(NSNumber *cur)
 - (void) deleteToEndOfLine: (id)sender
 {
   NSRange range = [self rangeForUserTextChange];
-  NSRange linerange;
-  unsigned maxRange;
-  unsigned endCorrection = 0;
   NSDictionary *attributes;
 
   if (range.location == NSNotFound)
@@ -676,38 +684,82 @@ static NSNumber *float_plus_one(NSNumber *cur)
       return;
     }
 
-  linerange = [[_textStorage string] lineRangeForRange: 
-					 NSMakeRange(range.location, 0)];
-  maxRange = NSMaxRange (linerange);
-  
-  if (maxRange == range.location)
+  /* If the selection is not empty delete it, otherwise delete up to the
+     next line end from the insertion point or the delete the line end
+     itself when the insertion point is already at the end of the line. */
+  if (range.length == 0)
+    {
+      NSUInteger start, end, contentsEnd;
+
+      [[_textStorage string] getLineStart: &start
+				      end: &end
+			      contentsEnd: &contentsEnd
+				 forRange: range];
+      if (range.location == contentsEnd)
+	{
+	  range = NSMakeRange(contentsEnd, end - contentsEnd);
+	}
+      else
+	{
+	  range.length = contentsEnd - range.location;
+	}
+      if (range.length == 0)
+	{
+	  return;
+	}
+    }
+
+  if (![self shouldChangeTextInRange: range  replacementString: @""])
     {
       return;
     }
 
-  // Only delete the linebreak, if the line is empty.
-  if (linerange.length > 1)
+  ASSIGN(killBuffer, [[_textStorage string] substringWithRange: range]);
+  attributes = RETAIN([_textStorage attributesAtIndex: range.location
+				       effectiveRange: NULL]);
+  [_textStorage beginEditing];
+  [_textStorage deleteCharactersInRange: range];
+  [_textStorage endEditing];
+  [self setTypingAttributes: attributes];
+  RELEASE(attributes);
+  [self didChangeText];
+}
+
+- (void) deleteToEndOfParagraph: (id)sender
+{
+  NSRange range = [self rangeForUserTextChange];
+  NSDictionary *attributes;
+
+  if (range.location == NSNotFound)
     {
-      // Treat the last line special, as it does not have to end in a leine break.
-      if (maxRange == [_textStorage length])
-        {
-	  unichar u = [[_textStorage string] characterAtIndex: (maxRange - 1)];
-	  if (u == '\n'  ||  u == '\r')
-	    {
-	      endCorrection = 1;
-	    }
-	  else
-	    {
-	      endCorrection = 0;
-	    }
-	}
-      else
-        {
-	  endCorrection = 1;
-	}
+      return;
     }
 
-  range = NSMakeRange(range.location, maxRange - range.location - endCorrection);
+  /* If the selection is not empty delete it, otherwise delete up to
+     the next paragraph end from the insertion point or the delete the
+     paragraph end itself when the insertion point is already at the
+     end of the paragraph. */
+  if (range.length == 0)
+    {
+      NSUInteger start, end, contentsEnd;
+
+      [[_textStorage string] getParagraphStart: &start
+					   end: &end
+				   contentsEnd: &contentsEnd
+				      forRange: range];
+      if (range.location == contentsEnd)
+	{
+	  range = NSMakeRange(contentsEnd, end - contentsEnd);
+	}
+      else
+	{
+	  range.length = contentsEnd - range.location;
+	}
+      if (range.length == 0)
+	{
+	  return;
+	}
+    }
 
   if (![self shouldChangeTextInRange: range  replacementString: @""])
     {
@@ -975,6 +1027,25 @@ check if there was a reason for that.
     }
 }
 
+- (void) moveLeftAndModifySelection: (id)sender
+{
+  NSParagraphStyle *parStyle;
+  NSWritingDirection writingDirection;
+  
+  parStyle = [[self typingAttributes]
+	  	objectForKey: NSParagraphStyleAttributeName];
+  writingDirection = [parStyle baseWritingDirection];
+  
+  if (writingDirection == NSWritingDirectionRightToLeft)
+    {
+        [self moveForwardAndModifySelection: sender];
+    }
+  else
+    {
+        [self moveBackwardAndModifySelection: sender];
+    }
+}
+
 - (void) moveRight: (id)sender
 {
   NSRange range = [self selectedRange];
@@ -995,6 +1066,24 @@ check if there was a reason for that.
     }
 }
 
+- (void) moveRightAndModifySelection: (id)sender
+{
+  NSParagraphStyle *parStyle;
+  NSWritingDirection writingDirection;
+  
+  parStyle = [[self typingAttributes]
+	  	objectForKey: NSParagraphStyleAttributeName];
+  writingDirection = [parStyle baseWritingDirection];
+  
+  if (writingDirection == NSWritingDirectionRightToLeft)
+    {
+        [self moveBackwardAndModifySelection: sender];
+    }
+  else
+    {
+        [self moveForwardAndModifySelection: sender];
+    }
+}
 
 - (void) moveBackward: (id)sender
 {
@@ -1089,6 +1178,82 @@ check if there was a reason for that.
 	 select: YES];
 }
 
+- (void) moveWordLeft: (id)sender
+{
+  NSParagraphStyle *parStyle;
+  NSWritingDirection writingDirection;
+  
+  parStyle = [[self typingAttributes]
+	  	objectForKey: NSParagraphStyleAttributeName];
+  writingDirection = [parStyle baseWritingDirection];
+  
+  if (writingDirection == NSWritingDirectionRightToLeft)
+    {
+        [self moveWordForward: sender];
+    }
+  else
+    {
+        [self moveWordBackward: sender];
+    }
+}
+
+- (void) moveWordLeftAndModifySelection: (id)sender
+{
+  NSParagraphStyle *parStyle;
+  NSWritingDirection writingDirection;
+  
+  parStyle = [[self typingAttributes]
+	  	objectForKey: NSParagraphStyleAttributeName];
+  writingDirection = [parStyle baseWritingDirection];
+  
+  if (writingDirection == NSWritingDirectionRightToLeft)
+    {
+        [self moveWordForwardAndModifySelection: sender];
+    }
+  else
+    {
+        [self moveWordBackwardAndModifySelection: sender];
+    }
+}
+
+- (void) moveWordRight: (id)sender
+{
+  NSParagraphStyle *parStyle;
+  NSWritingDirection writingDirection;
+  
+  parStyle = [[self typingAttributes]
+	  	objectForKey: NSParagraphStyleAttributeName];
+  writingDirection = [parStyle baseWritingDirection];
+  
+  if (writingDirection == NSWritingDirectionRightToLeft)
+    {
+        [self moveWordBackward: sender];
+    }
+  else
+    {
+        [self moveWordForward: sender];
+    }
+}
+
+- (void) moveWordRightAndModifySelection: (id)sender
+{
+  NSParagraphStyle *parStyle;
+  NSWritingDirection writingDirection;
+  
+  parStyle = [[self typingAttributes]
+	  	objectForKey: NSParagraphStyleAttributeName];
+  writingDirection = [parStyle baseWritingDirection];
+  
+  if (writingDirection == NSWritingDirectionRightToLeft)
+    {
+        [self moveWordBackwardAndModifySelection: sender];
+    }
+  else
+    {
+        [self moveWordForwardAndModifySelection: sender];
+    }
+}
+
 - (void) moveToBeginningOfDocument: (id)sender
 {
   [self _moveTo: 0
@@ -1552,7 +1717,10 @@ and layout is left-to-right */
 - (void) setBaseWritingDirection: (NSWritingDirection)direction
                            range: (NSRange)range
 {
-  // FIXME
+  if (!_tf.is_rich_text)
+    return;
+
+  [_textStorage setBaseWritingDirection: direction range: range];
 }
 
 - (void) toggleBaseWritingDirection: (id)sender
