@@ -208,7 +208,8 @@
 
 - (void) setDocumentEdited: (BOOL)flag
 {
-  [[self window] setDocumentEdited: flag];
+  if ([self isWindowLoaded])
+    [[self window] setDocumentEdited: flag];
 }
 
 - (void) setWindowFrameAutosaveName:(NSString *)name
@@ -294,22 +295,28 @@
     {
       // Do all the notifications.  Yes, the docs say this should
       // be implemented here instead of in -loadWindow itself.
+
+      // Note: The docs say that windowController{Will,Did}LoadNib: are sent
+      // to the window controller's document, but Apple's implementation
+      // really sends them to the owner of the nib. Since this behavior is
+      // more useful, in particular when non-document classes use a window
+      // controller, we implement it here too.
       [self windowWillLoad];
-      if ([_document respondsToSelector:
-		       @selector(windowControllerWillLoadNib:)])
+      if (_owner != self &&
+	  [_owner respondsToSelector: @selector(windowControllerWillLoadNib:)])
 	{
-	  [_document windowControllerWillLoadNib:self];
+	  [_owner windowControllerWillLoadNib: self];
 	}
 
       [self loadWindow];
       if ([self isWindowLoaded]) 
       {
         [self _windowDidLoad];
-        if ([_document respondsToSelector:
-	  	       @selector(windowControllerDidLoadNib:)])
-        {
-          [_document windowControllerDidLoadNib:self];
-        }
+	if (_owner != self &&
+	    [_owner respondsToSelector: @selector(windowControllerDidLoadNib:)])
+	{
+	  [_owner windowControllerDidLoadNib: self];
+	}
       }
     }
 
@@ -368,8 +375,14 @@
       else
         {
           [_window setReleasedWhenClosed: YES];
+          [_window setDocumentEdited: [_document isDocumentEdited]];
         }
 
+      /* Make sure window sizes itself right */
+      if ([_window_frame_autosave_name length] > 0)
+        {
+          [_window setFrameAutosaveName: _window_frame_autosave_name];
+        }
     }
 }
 
@@ -402,7 +415,7 @@
       NSString *title = [self windowTitleForDocumentDisplayName: displayName];
 
       /* If they just want to display the filename, use the fancy method */
-      /* NB For compatibility with Mac OS X, a document display name is equal
+      /* NB For compatibility with Mac OS X, a document's display name is equal
          to its last path component, so we check for that here too */
       if (filename != nil &&
 	  ([title isEqualToString: filename] ||
@@ -437,13 +450,6 @@
   _wcFlags.nib_is_loaded = YES;
 
   [self synchronizeWindowTitleWithDocumentName];
-  
-  /* Make sure window sizes itself right */
-  if ([_window_frame_autosave_name length] > 0)
-    {
-      [_window setFrameUsingName: _window_frame_autosave_name];
-      [_window setFrameAutosaveName: _window_frame_autosave_name];
-    }
 
   if ([self shouldCascadeWindows])
     {
