@@ -329,6 +329,7 @@ NSString *NSPrintJobFeatures = @"PrintJobFeatures";
 NSString *NSPrintLastPage = @"PrintLastPage";
 NSString *NSPrintLeftMargin = @"PrintLeftMargin";
 NSString *NSPrintManualFeed = @"PrintManualFeed";
+NSString *NSPrintMustCollate = @"PrintMustCollate";
 NSString *NSPrintOrientation = @"PrintOrientation";
 NSString *NSPrintPagesPerSheet = @"PrintPagesPerSheet";
 NSString *NSPrintPaperFeed = @"PrintPaperFeed";
@@ -343,6 +344,13 @@ NSString *NSPrintTopMargin = @"PrintTopMargin";
 NSString *NSPrintHorizontalPagination = @"PrintHorizontalPagination";
 NSString *NSPrintVerticalPagination = @"PrintVerticalPagination";
 NSString *NSPrintVerticallyCentered = @"PrintVerticallyCentered";
+NSString *NSPrintPagesAcross = @"PrintPagesAcross";
+NSString *NSPrintPagesDown = @"PrintPagesDown";
+NSString *NSPrintTime = @"PrintTime";
+NSString *NSPrintDetailedErrorReporting = @"PrintDetailedErrorReporting";
+NSString *NSPrintFaxNumber = @"PrintFaxNumber";
+NSString *NSPrintPrinterName = @"PrintPrinterName";
+NSString *NSPrintHeaderAndFooter = @"PrintHeaderAndFooter";
 
 NSString *NSPrintPageDirection = @"NSPrintPageDirection";
 
@@ -353,6 +361,11 @@ NSString  *NSPrintPreviewJob = @"PrintPreviewJob";
 NSString  *NSPrintSaveJob = @"PrintSaveJob";
 NSString  *NSPrintSpoolJob = @"PrintSpoolJob";
 
+// Print Panel
+NSString *NSPrintPanelAccessorySummaryItemNameKey = @"name";
+NSString *NSPrintPanelAccessorySummaryItemDescriptionKey = @"description";
+NSString *NSPrintPhotoJobStyleHint = @"Photo";
+
 // NSSplitView notifications
 NSString *NSSplitViewDidResizeSubviewsNotification =
 @"NSSplitViewDidResizeSubviewsNotification";
@@ -488,6 +501,14 @@ NSString *NSWorkspaceWillPowerOffNotification =
 @"NSWorkspaceWillPowerOffNotification";
 NSString *NSWorkspaceWillUnmountNotification =
 @"NSWorkspaceWillUnmountNotification";
+NSString *NSWorkspaceDidWakeNotification =
+@"NSWorkspaceDidWakeNotification";
+NSString *NSWorkspaceSessionDidBecomeActiveNotification =
+@"NSWorkspaceSessionDidBecomeActiveNotification";
+NSString *NSWorkspaceSessionDidResignActiveNotification =
+@"NSWorkspaceSessionDidResignActiveNotification";
+NSString *NSWorkspaceWillSleepNotification =
+@"NSWorkspaceWillSleepNotification";
 
 /*
  *	NSStringDrawing NSAttributedString additions
@@ -663,6 +684,7 @@ NSString *NSFontNameBinding = @"fontName";
 NSString *NSFontSizeBinding = @"fontSize";
 NSString *NSHiddenBinding = @"hidden";
 NSString *NSSelectedIndexBinding = @"selectedIndex";
+NSString *NSSelectedObjectBinding = @"selectedObject";
 NSString *NSSelectedTagBinding = @"selectedTag";
 NSString *NSTextColorBinding = @"textColor";
 NSString *NSTitleBinding = @"title";
