@@ -26,11 +26,12 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
-#include "RTFConsumer.h"
-#include "RTFConsumerFunctions.h"
-#include "RTFProducer.h"
+#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
+#import <GNUstepGUI/GSHelpAttachment.h>
+#import "RTFConsumer.h"
+#import "RTFConsumerFunctions.h"
+#import "RTFProducer.h"
 
 /*  we have to satisfy the scanner with a stream reading function */
 typedef struct {
@@ -62,11 +63,13 @@ readString (StringContext *ctxt)
   NSMutableParagraphStyle *paragraph;
   NSColor *fgColour;
   NSColor *bgColour;
+  NSColor *ulColour;
   NSString *fontName;
   float fontSize;
   BOOL bold;
   BOOL italic;
-  BOOL underline;
+  NSInteger underline;
+  NSInteger strikethrough;
   int script;
 
   float real_fi, real_li;
@@ -75,6 +78,7 @@ readString (StringContext *ctxt)
 - (NSFont*) currentFont;
 - (NSNumber*) script;
 - (NSNumber*) underline;
+- (NSNumber*) strikethrough;
 - (void) resetParagraphStyle;
 - (void) resetFont;
 - (void) addTab: (float)location  type: (NSTextTabType)type;
@@ -97,6 +101,7 @@ readString (StringContext *ctxt)
   RELEASE(fontName);
   RELEASE(fgColour);
   RELEASE(bgColour);
+  RELEASE(ulColour);
   [super dealloc];
 }
 
@@ -108,6 +113,7 @@ readString (StringContext *ctxt)
   RETAIN(new->fontName);
   RETAIN(new->fgColour);
   RETAIN(new->bgColour);
+  RETAIN(new->ulColour);
 
   return new;
 }
@@ -182,8 +188,16 @@ readString (StringContext *ctxt)
 
 - (NSNumber*) underline
 {
-  if (underline)
-    return [NSNumber numberWithInt: NSSingleUnderlineStyle];
+  if (underline != NSUnderlineStyleNone)
+    return [NSNumber numberWithInteger: underline];
+  else
+    return nil;
+}
+
+- (NSNumber*) strikethrough
+{
+  if (strikethrough != NSUnderlineStyleNone)
+    return [NSNumber numberWithInteger: strikethrough];
   else
     return nil;
 }
@@ -207,10 +221,12 @@ readString (StringContext *ctxt)
   italic = NO;
   bold = NO;
 
-  underline = NO;
+  underline = NSUnderlineStyleNone;
+  strikethrough = NSUnderlineStyleNone;
   script = 0;
   DESTROY(fgColour);
   DESTROY(bgColour);
+  DESTROY(ulColour);
 
   changed = YES;
 }
@@ -266,6 +282,8 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 - (void) push;
 - (void) pop;
 - (void) appendString: (NSString*)string;
+- (void) appendHelpLink: (NSString*)fileName marker: (NSString *)markerName;
+- (void) appendHelpMarker: (NSString*)markerName;
 
 @end
 
@@ -464,6 +482,7 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
           if (attachment == nil)
             {
               NSLog(@"No attachment at %d", oldPosition);
+              RELEASE(image);
               return;
             }
         
@@ -633,11 +652,16 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 			   ps, NSParagraphStyleAttributeName,
 			   nil];
 	  DESTROY(ps);
-	  if (attr->underline)
+	  if ([attr underline])
 	    {
 	      [attributes setObject: [attr underline]
 			  forKey: NSUnderlineStyleAttributeName];
 	    }
+	  if ([attr strikethrough])
+	    {
+	      [attributes setObject: [attr strikethrough]
+			  forKey: NSStrikethroughStyleAttributeName];
+	    }
 	  if (attr->script)
 	    {
 	      [attributes setObject: [attr script]
@@ -653,7 +677,12 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 	      [attributes setObject: attr->bgColour 
 			  forKey: NSBackgroundColorAttributeName];
 	    }
-	  
+	  if (attr->ulColour != nil)
+	    {
+	      [attributes setObject: attr->ulColour 
+			  forKey: NSUnderlineColorAttributeName];
+	    }
+  
 	  [result setAttributes: attributes 
 		  range: NSMakeRange(oldPosition, textlen)];
 	  DESTROY(attributes);
@@ -662,6 +691,84 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
     }
 }
 
+- (void) appendHelpLink: (NSString*)fileName marker: (NSString*)markerName
+{
+  int  oldPosition = [result length];
+  NSRange insertionRange = NSMakeRange(oldPosition,0);
+
+  if (!ignore)
+    {
+      GSHelpLinkAttachment* attachment;
+      RTFAttribute* attr = [self attr];
+      NSMutableDictionary* attributes = nil;
+      NSMutableAttributedString* str = nil;
+
+      attachment =
+	[[GSHelpLinkAttachment alloc]
+	  initWithFileName: fileName
+		markerName: markerName];
+      if (attachment == nil)
+	{
+	  NSLog(@"No attachment at %d", oldPosition);
+	  return;
+	}
+        
+      attributes = [[NSMutableDictionary alloc]
+		     initWithObjectsAndKeys:
+		       [attr currentFont], NSFontAttributeName,
+		       attr->paragraph, NSParagraphStyleAttributeName,
+		       nil];
+          
+      str = (NSMutableAttributedString*) [NSMutableAttributedString 
+                     attributedStringWithAttachment: attachment];
+
+      [str addAttributes: attributes range: NSMakeRange (0, [str length])];
+          
+      [result replaceCharactersInRange: insertionRange withAttributedString: str];
+      attr->changed = YES;
+      RELEASE(attributes);
+      RELEASE(attachment);
+    }
+}
+
+- (void) appendHelpMarker: (NSString*)markerName
+{
+  int  oldPosition = [result length];
+  NSRange insertionRange = NSMakeRange(oldPosition,0);
+
+  if (!ignore)
+    {
+      GSHelpMarkerAttachment* attachment;
+      RTFAttribute* attr = [self attr];
+      NSMutableDictionary* attributes = nil;
+      NSMutableAttributedString* str = nil;
+
+      attachment =
+	[[GSHelpMarkerAttachment alloc] initWithMarkerName: markerName];
+      if (attachment == nil)
+	{
+	  NSLog(@"No attachment at %d", oldPosition);
+	  return;
+	}
+        
+      attributes = [[NSMutableDictionary alloc]
+		     initWithObjectsAndKeys:
+		       [attr currentFont], NSFontAttributeName,
+		       attr->paragraph, NSParagraphStyleAttributeName,
+		       nil];
+          
+      str = (NSMutableAttributedString*) [NSMutableAttributedString 
+                     attributedStringWithAttachment: attachment];
+
+      [str addAttributes: attributes range: NSMakeRange (0, [str length])];
+          
+      [result replaceCharactersInRange: insertionRange withAttributedString: str];
+      attr->changed = YES;
+      RELEASE(attributes);
+      RELEASE(attachment);
+    }
+}
+
 @end
 
 #undef IGNORE
@@ -683,8 +790,10 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 #define ITALIC CTXT->italic
 #define BOLD CTXT->bold
 #define UNDERLINE CTXT->underline
+#define STRIKETHROUGH CTXT->strikethrough
 #define FGCOLOUR CTXT->fgColour
 #define BGCOLOUR CTXT->bgColour
+#define ULCOLOUR CTXT->ulColour
 
 #define PAPERSIZE @"PaperSize"
 #define LEFTMARGIN @"LeftMargin"
@@ -698,7 +807,7 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 */
 
 /* handle errors (this is the yacc error mech)	*/
-void GSRTFerror (const char *msg)
+void GSRTFerror (void *ctxt, void *lctxt, const char *msg)
 {
 /*  [NSException raise:NSInvalidArgumentException
 	       format:@"Syntax error in RTF: %s", msg];*/
@@ -1102,6 +1211,19 @@ void GSRTFcolorfg (void *ctxt, int color)
   CHANGED = YES;
 }
 
+void GSRTFunderlinecolor (void *ctxt, int color)
+{
+  if ([COLOURS count] <= (unsigned int)color)
+    {
+      ASSIGN (ULCOLOUR, [NSColor blackColor]);
+    }
+  else
+    {
+      ASSIGN (ULCOLOUR, [COLOURS objectAtIndex: color]);
+    }
+  CHANGED = YES;
+}
+
 void GSRTFsubscript (void *ctxt, int script)
 {
   script = (int) (-halfpoints2points(script) / 3.0);
@@ -1142,11 +1264,36 @@ void GSRTFbold (void *ctxt, BOOL state)
     }
 }
 
-void GSRTFunderline (void *ctxt, BOOL state)
+void GSRTFunderline (void *ctxt, BOOL state, NSInteger style)
 {
-  if (state != UNDERLINE)
+  if (state == NO)
     {
-      UNDERLINE = state;
+      style = NSUnderlineStyleNone;
+    }
+
+  if (UNDERLINE != style)
+    {
+      UNDERLINE = style;
+      CHANGED = YES;
+    }
+}
+
+void GSRTFstrikethrough (void *ctxt, NSInteger style)
+{
+  if (STRIKETHROUGH != style)
+    {
+      STRIKETHROUGH = style;
+      CHANGED = YES;
+    }
+}
+
+void GSRTFstrikethroughDouble (void *ctxt)
+{
+  const NSInteger style = NSUnderlineStyleDouble | NSUnderlinePatternSolid;
+
+  if (STRIKETHROUGH != style)
+    {
+      STRIKETHROUGH = style;
       CHANGED = YES;
     }
 }
@@ -1162,3 +1309,34 @@ void GSRTFNeXTGraphic (void *ctxt, const char *fileName, int width, int height)
   [(RTFDConsumer *)ctxt appendImage: [NSString stringWithCString: fileName]];
 }
 
+void GSRTFNeXTHelpLink (void *ctxt, int num, const char *markername,
+			const char *linkFilename, const char *linkMarkername)
+{
+  NSRange range;
+  NSString *fileName = [NSString stringWithCString: linkFilename];
+  NSString *markerName = [NSString stringWithCString: linkMarkername];
+
+  range = [fileName rangeOfString: @";"];
+  if (range.location != NSNotFound)
+    fileName = [fileName substringToIndex:range.location];
+
+  range = [markerName rangeOfString: @";"];
+  if (range.location == 0)
+    markerName = nil;
+  else if (range.location != NSNotFound)
+    markerName = [markerName substringToIndex:range.location];
+
+  [(RTFDConsumer *)ctxt appendHelpLink: fileName marker: markerName];
+}
+
+void GSRTFNeXTHelpMarker (void *ctxt, int num, const char *markername)
+{
+  NSRange range;
+  NSString *markerName = [NSString stringWithCString: markername];
+
+  range = [markerName rangeOfString: @";"];
+  if (range.location != NSNotFound)
+    markerName = [markerName substringToIndex:range.location];
+  [(RTFDConsumer *)ctxt appendHelpMarker: markerName];
+}
+
