@@ -708,6 +708,10 @@
       else if ([attributeName isEqualToString: NSFontAttributeName])
         {
         }
+      else if ([attributeName isEqualToString: NSLinkAttributeName])
+	{
+	  [result appendString: @"}}"];
+	}
       else
         {
           NSLog(@"(removal) Missing handling of '%@' attributes.",
@@ -1083,6 +1087,29 @@
 		}
             }
         }
+      else if ([attributeName isEqualToString: NSLinkAttributeName])
+	{
+	  id dest = [attributesToAdd objectForKey:
+              NSLinkAttributeName];
+
+	  NSString *destString = @"";
+
+	  if ([dest isKindOfClass: [NSURL class]])
+	    {
+	      destString = [dest absoluteString];
+	    }
+	  else if ([dest isKindOfClass: [NSString class]])
+	    {
+	      destString = [[NSURL URLWithString: dest] absoluteString];
+	    }
+
+	  // NOTE: RTF control characters (backslash, curly braces)
+	  // will be escaped by -absoluteString, so the result is safe to 
+	  // concatenate into the RTF stream
+
+	  [result appendString: 
+		    [NSString stringWithFormat: @"{\\field{\\*\\fldinst HYPERLINK \"%@\"}{\\fldrslt ", destString]];
+	}
       else
         {
           NSLog(@"(addition) Missing handling of '%@' attributes.",
@@ -1159,13 +1186,11 @@
 {
   NSString *string;
   NSMutableString *result;
-  unsigned loc;
   unsigned length;
   NSRange effectiveRange;
 
   string = [text string];
   result = (NSMutableString *)[NSMutableString string];
-  loc = 0;
   length = [string length];
   effectiveRange = NSMakeRange(0, 0);
 
@@ -1188,7 +1213,7 @@
 
       [self _setAttributesOfLastRun: attributes];
 
-      RELEASE(pool);
+      [pool drain];
     }
 
   [self _setAttributesOfLastRun: nil]; // cleanup, should be unneccessary
